@@ -22,6 +22,7 @@
 #define SERIALIZER_HPP
 
 #include <algorithm>
+#include <cstdint>
 #include <functional>
 #include <map>
 #include <memory>
@@ -73,6 +74,12 @@ decltype(auto) make_variant(std::size_t index)
 template<class T>
 using remove_cvr_t = std::remove_cv_t<std::remove_reference_t<T>>;
 
+template <typename T>
+struct is_unique_ptr : std::false_type {};
+
+template <typename T>
+struct is_unique_ptr<std::unique_ptr<T>> : std::true_type {};
+
 } // namespace detail
 
 /*! \brief Class for (de-)serializing.
@@ -94,7 +101,11 @@ public:
     void operator()(const T& data)
     {
         if constexpr (is_ptr<T>::value) {
-            ptr(data);
+            if constexpr (detail::is_unique_ptr<T>::value) {
+                unique_ptr(data);
+            } else {
+                shared_ptr(data);
+            }
         } else if constexpr (is_pair_or_tuple<T>::value) {
             tuple(data);
         } else if constexpr (is_variant<T>::value) {
@@ -127,13 +138,16 @@ public:
     template<class T>
     void pack(const T& data)
     {
+        m_ptrmap.clear();
         m_op = Operation::PACKSIZE;
         m_packSize = 0;
         (*this)(data);
         m_position = 0;
         m_buffer.resize(m_packSize);
+        m_ptrmap.clear();
         m_op = Operation::PACK;
         (*this)(data);
+        m_ptrmap.clear();
     }
 
     //! \brief Call this to serialize data.
@@ -142,13 +156,16 @@ public:
     template<class... Args>
     void pack(const Args&... data)
     {
+        m_ptrmap.clear();
         m_op = Operation::PACKSIZE;
         m_packSize = 0;
         variadic_call(data...);
         m_position = 0;
         m_buffer.resize(m_packSize);
+        m_ptrmap.clear();
         m_op = Operation::PACK;
         variadic_call(data...);
+        m_ptrmap.clear();
     }
 
     //! \brief Call this to de-serialize data.
@@ -158,8 +175,10 @@ public:
     void unpack(T& data)
     {
         m_position = 0;
+        m_ptrmap.clear();
         m_op = Operation::UNPACK;
         (*this)(data);
+        m_ptrmap.clear();
     }
 
     //! \brief Call this to de-serialize data.
@@ -169,8 +188,10 @@ public:
     void unpack(Args&... data)
     {
         m_position = 0;
+        m_ptrmap.clear();
         m_op = Operation::UNPACK;
         variadic_call(data...);
+        m_ptrmap.clear();
     }
 
     //! \brief Returns current position in buffer.
@@ -186,25 +207,6 @@ public:
     }
 
 protected:
-    /// Utility function for missing data() member function in FieldVector of DUNE 2.6
-    template<typename Vector>
-    const typename Vector::value_type* getVectorData(const Vector& data)
-    {
-        if (data.size() == 0)
-            return nullptr;
-        else
-            return &(data[0]);
-    }
-
-    /// Utility function for missing data() member function in FieldVector of DUNE 2.6
-    template<typename Vector>
-    typename Vector::value_type* getVectorData(Vector& data)
-    {
-        if (data.size() == 0)
-            return nullptr;
-        else
-            return &(data[0]);
-    }
     //! \brief Handler for vectors.
     //! \tparam T Type for vector elements
     //! \param data The vector to (de-)serialize
@@ -214,16 +216,22 @@ protected:
         if constexpr (std::is_pod_v<typename Vector::value_type>) {
           if (m_op == Operation::PACKSIZE) {
               (*this)(data.size());
-              m_packSize += m_packer.packSize(data.data(), data.size());
+              if (data.size() > 0) {
+                  m_packSize += m_packer.packSize(data.data(), data.size());
+              }
           } else if (m_op == Operation::PACK) {
               (*this)(data.size());
-              m_packer.pack(getVectorData(data), data.size(), m_buffer, m_position);
+              if (data.size() > 0) {
+                  m_packer.pack(data.data(), data.size(), m_buffer, m_position);
+              }
           } else if (m_op == Operation::UNPACK) {
               std::size_t size = 0;
               (*this)(size);
               auto& data_mut = const_cast<Vector&>(data);
               data_mut.resize(size);
-              m_packer.unpack(getVectorData(data_mut), size, m_buffer, m_position);
+              if (size > 0) {
+                  m_packer.unpack(data_mut.data(), size, m_buffer, m_position);
+              }
           }
         } else {
             if (m_op == Operation::UNPACK) {
@@ -272,12 +280,12 @@ protected:
 
         if constexpr (std::is_pod_v<T>) {
             if (m_op == Operation::PACKSIZE)
-                m_packSize += m_packer.packSize(getVectorData(data), data.size());
+                m_packSize += m_packer.packSize(data.data(), data.size());
             else if (m_op == Operation::PACK)
-                m_packer.pack(getVectorData(data), data.size(), m_buffer, m_position);
+                m_packer.pack(data.data(), data.size(), m_buffer, m_position);
             else if (m_op == Operation::UNPACK) {
                 auto& data_mut = const_cast<Array&>(data);
-                m_packer.unpack(getVectorData(data_mut), data_mut.size(), m_buffer, m_position);
+                m_packer.unpack(data_mut.data(), data_mut.size(), m_buffer, m_position);
             }
         } else {
             std::for_each(data.begin(), data.end(), std::ref(*this));
@@ -314,6 +322,8 @@ protected:
                 T res{};
                 (*this)(res);
                 const_cast<std::optional<T>&>(data) = res;
+            } else {
+                const_cast<std::optional<T>&>(data) = std::nullopt;
             }
         } else {
             (*this)(data.has_value());
@@ -363,7 +373,7 @@ protected:
             (*this)(size);
             auto& data_mut = const_cast<Set&>(data);
             for (size_t i = 0; i < size; ++i) {
-                typename Set::value_type entry;
+                typename Set::value_type entry{};
                 (*this)(entry);
                 data_mut.insert(entry);
             }
@@ -540,26 +550,57 @@ protected:
         T, std::void_t<decltype(std::declval<T>().serializeOp(std::declval<Serializer<Packer>&>()))>
     > : public std::true_type {};
 
-    //! \brief Handler for smart pointers.
+    //! \brief Handler for shared pointers.
     template<class PtrType>
-    void ptr(const PtrType& data)
+    void shared_ptr(const PtrType& data)
     {
         using T1 = typename PtrType::element_type;
-        bool value = data ? true : false;
-        (*this)(value);
-        if (m_op == Operation::UNPACK && value) {
-            const_cast<PtrType&>(data).reset(new T1);
+        std::uintptr_t data_ptr = reinterpret_cast<std::uintptr_t>(data.get());
+        (*this)(data_ptr);
+        if (!data_ptr)
+            return;
+        if (m_op == Operation::PACK || m_op == Operation::PACKSIZE) {
+            if (m_ptrmap.count(data_ptr) == 0) {
+                (*this)(*data);
+                m_ptrmap[data_ptr].reset();
+            }
+        } else {  // m_op == Operation::UNPACK
+            if (m_ptrmap.count(data_ptr) == 0) {
+                const_cast<PtrType&>(data) = std::make_shared<T1>();
+                m_ptrmap[data_ptr] = std::static_pointer_cast<void>(data);
+                (*this)(*data);
+            } else {
+                const_cast<PtrType&>(data) = std::static_pointer_cast<T1>(m_ptrmap[data_ptr]);
+            }
         }
-        if (data) {
-            (*this)(*data);
+    }
+
+    template<class PtrType>
+    void unique_ptr(const PtrType& data)
+    {
+        using T1 = typename PtrType::element_type;
+
+        if (m_op != Operation::UNPACK) {
+            (*this)(data ? 1 : 0);
+            if (data) {
+                (*this)(*data);
+            }
+        } else {
+            int ptr = 0;
+            (*this)(ptr);
+            if (ptr == 1) {
+                const_cast<PtrType&>(data) = std::make_unique<T1>();
+                (*this)(*data);
+            }
         }
     }
 
     const Packer& m_packer; //!< Packer to use
     Operation m_op = Operation::PACKSIZE; //!< Current operation
     size_t m_packSize = 0; //!< Required buffer size after PACKSIZE has been done
-    int m_position = 0; //!< Current position in buffer
+    size_t m_position = 0; //!< Current position in buffer
     std::vector<char> m_buffer; //!< Buffer for serialized data
+    std::map<std::uintptr_t, std::shared_ptr<void>> m_ptrmap; //!< Map to keep track of which pointer data has been serialized and actual pointers during unpacking
 };
 
 }
