@@ -138,10 +138,10 @@ namespace Opm {
         , m_inputGrid(         deck, nullptr )
         , m_inputNnc(          m_inputGrid, deck)
         , m_gridDims(          deck )
-        , field_props(         deck, m_runspec.phases(), m_inputGrid, m_tables)
+        , field_props(         deck, m_runspec.phases(), m_inputGrid, m_tables, m_runspec.numComps())
         , m_simulationConfig(  m_eclipseConfig.init().restartRequested(), deck, field_props)
         , aquifer_config(      m_tables, m_inputGrid, deck, field_props)
-        , compositional_config(  deck, m_runspec)
+        , compositional_config(deck, m_runspec)
         , m_transMult(         GridDims(deck), deck, field_props)
         , tracer_config(       m_deckUnitSystem, deck)
         , m_micppara(          deck)
@@ -152,8 +152,6 @@ namespace Opm {
         this->reportNumberOfActivePhases();
 
         this->conveyNumericalAquiferEffects();
-        this->m_inputGrid.resetACTNUM(this->field_props.actnum());
-        this->field_props.reset_actnum(this->getInputGrid().getACTNUM());
         if (field_props.has_double("MINPVV")) {
             this->m_inputGrid.setMINPVV(field_props.get_global_double("MINPVV"));
             field_props.deleteMINPVV();
@@ -302,10 +300,19 @@ namespace Opm {
         return m_inputNnc;
     }
 
+    const std::vector<NNCdata>& EclipseState::getPinchNNC() const {
+        return m_pinchNnc;
+    }
+
     void EclipseState::setInputNNC(const NNC& nnc) {
         m_inputNnc = nnc;
     }
 
+    void EclipseState::setPinchNNC(std::vector<NNCdata>&& nnc) {
+        m_pinchNnc = nnc;
+        std::sort(m_pinchNnc.begin(), m_pinchNnc.end());
+    }
+
     void EclipseState::appendInputNNC(const std::vector<NNCdata>& nnc) {
         for (const auto& nnc_data : nnc ) {
             this->m_inputNnc.addNNC(nnc_data.cell1, nnc_data.cell2, nnc_data.trans);
@@ -316,6 +323,10 @@ namespace Opm {
         return !m_inputNnc.input().empty();
     }
 
+    bool EclipseState::hasPinchNNC() const {
+        return !m_pinchNnc.empty();
+    }
+
     bool EclipseState::hasInputLGR() const {
         return m_lgrs.size() != 0;
     }
@@ -331,7 +342,8 @@ namespace Opm {
     }
 
 
-    std::string EclipseState::getTitle() const {
+    const std::string& EclipseState::getTitle() const
+    {
         return m_title;
     }
 
@@ -339,14 +351,28 @@ namespace Opm {
         return this->aquifer_config;
     }
 
+    const CompositionalConfig& EclipseState::compositionalConfig() const {
+        return this->compositional_config;
+    }
+
     const TracerConfig& EclipseState::tracer() const {
         return this->tracer_config;
     }
 
+    void EclipseState::prune_global_for_schedule_run()
+    {
+        this->field_props.prune_global_for_schedule_run();
+    }
+
     void EclipseState::reset_actnum(const std::vector<int>& new_actnum) {
         this->field_props.reset_actnum(new_actnum);
     }
 
+    void EclipseState::set_active_indices(const std::vector<int>& indices)
+    {
+        this->field_props.set_active_indices(indices);
+    }
+
     void EclipseState::pruneDeactivatedAquiferConnections(const std::vector<std::size_t>& deactivated_cells) {
         if (this->aquifer_config.hasAnalyticalAquifer())
             this->aquifer_config.pruneDeactivatedAquiferConnections(deactivated_cells);
