@@ -1,5 +1,4 @@
 /*
-
   Copyright 2014 Statoil ASA.
 
   This file is part of the Open Porous Media project (OPM).
@@ -26,6 +25,7 @@
 #include <opm/common/ErrorMacros.hpp>
 #include <opm/common/OpmLog/OpmLog.hpp>
 #include <opm/common/utility/numeric/calculateCellVol.hpp>
+#include <opm/common/utility/numeric/VectorOps.hpp>
 #include <opm/common/utility/String.hpp>
 
 #include <opm/io/eclipse/EclFile.hpp>
@@ -86,8 +86,8 @@ std::optional<UnitSystem> make_grid_units(const std::string& grid_unit) {
 void apply_GRIDUNIT(const UnitSystem& deck_units, const UnitSystem& grid_units, std::vector<double>& data)
 {
     double scale_factor = grid_units.getDimension(UnitSystem::measure::length).getSIScaling() / deck_units.getDimension(UnitSystem::measure::length).getSIScaling();
-    for (auto& v : data)
-        v *= scale_factor;
+    std::transform(data.begin(), data.end(), data.begin(),
+                   [scale_factor](const auto& v) { return v * scale_factor; });
 }
 
 }
@@ -443,7 +443,9 @@ EclipseGrid::EclipseGrid(const Deck& deck, const int * actnum)
         }
     }
 
-    void EclipseGrid::initGridFromEGridFile(Opm::EclIO::EclFile& egridfile, std::string fileName){
+    void EclipseGrid::initGridFromEGridFile(Opm::EclIO::EclFile& egridfile,
+                                            const std::string&   fileName)
+    {
         OpmLog::info(fmt::format("\nCreating grid from: {} ", fileName));
 
         if (!egridfile.hasKey("GRIDHEAD")) {
@@ -462,51 +464,56 @@ EclipseGrid::EclipseGrid(const Deck& deck, const int * actnum)
             throw std::invalid_argument("file: " + fileName + " is not a valid egrid file, GRIDUNIT not found");
         }
 
-        const std::vector<std::string>& gridunit = egridfile.get<std::string>("GRIDUNIT");
+        {
+            const std::vector<int>& gridhead = egridfile.get<int>("GRIDHEAD");
 
-        const std::vector<int>& gridhead = egridfile.get<int>("GRIDHEAD");
-        const std::array<int, 3> dims = {gridhead[1], gridhead[2], gridhead[3]};
+            this->m_nx = gridhead[1];
+            this->m_ny = gridhead[2];
+            this->m_nz = gridhead[3];
+        }
 
-        m_nx=dims[0];
-        m_ny=dims[1];
-        m_nz=dims[2];
+        {
+            const std::vector<float>& coord_f = egridfile.get<float>("COORD");
+            const std::vector<float>& zcorn_f = egridfile.get<float>("ZCORN");
 
-        const std::vector<float>& coord_f = egridfile.get<float>("COORD");
-        const std::vector<float>& zcorn_f = egridfile.get<float>("ZCORN");
-
-        m_coord.assign(coord_f.begin(), coord_f.end());
-        m_zcorn.assign(zcorn_f.begin(), zcorn_f.end());
-
-        if (gridunit[0] != "METRES") {
+            m_coord.assign(coord_f.begin(), coord_f.end());
+            m_zcorn.assign(zcorn_f.begin(), zcorn_f.end());
+        }
 
+        if (const auto& gridunit = egridfile.get<std::string>("GRIDUNIT");
+            gridunit[0] != "METRES")
+        {
             constexpr auto length = ::Opm::UnitSystem::measure::length;
 
-            if (gridunit[0] == "FEET"){
-                Opm::UnitSystem units(Opm::UnitSystem::UnitType::UNIT_TYPE_FIELD );
+            if (gridunit[0] == "FEET") {
+                Opm::UnitSystem units(Opm::UnitSystem::UnitType::UNIT_TYPE_FIELD);
                 units.to_si(length, m_coord);
                 units.to_si(length, m_zcorn);
-            } else if (gridunit[0] == "CM"){
-                Opm::UnitSystem units(Opm::UnitSystem::UnitType::UNIT_TYPE_LAB );
+            }
+            else if (gridunit[0] == "CM") {
+                Opm::UnitSystem units(Opm::UnitSystem::UnitType::UNIT_TYPE_LAB);
                 units.to_si(length, m_coord);
                 units.to_si(length, m_zcorn);
-            } else {
+            }
+            else {
                 std::string message = "gridunit '" + gridunit[0] + "' doesn't correspong to a valid unit system";
                 throw std::invalid_argument(message);
             }
         }
 
-        if ((egridfile.hasKey("ACTNUM")) && (m_useActnumFromGdfile)) {
-            const std::vector<int>& actnum  = egridfile.get<int>("ACTNUM");
-            resetACTNUM( actnum );
-        } else
-            resetACTNUM( );
+        if (egridfile.hasKey("ACTNUM") && m_useActnumFromGdfile) {
+            resetACTNUM(egridfile.get<int>("ACTNUM"));
+        }
+        else {
+            this->resetACTNUM();
+        }
 
-        if (egridfile.hasKey("MAPAXES"))
+        if (egridfile.hasKey("MAPAXES")) {
             this->m_mapaxes = std::make_optional<MapAxes>(egridfile);
+        }
 
-
-        ZcornMapper mapper( getNX(), getNY(), getNZ());
-        zcorn_fixed = mapper.fixupZCORN( m_zcorn );
+        ZcornMapper mapper(getNX(), getNY(), getNZ());
+        zcorn_fixed = mapper.fixupZCORN(m_zcorn);
     }
 
     bool EclipseGrid::keywInputBeforeGdfile(const Deck& deck, const std::string& keyword) const {
@@ -570,12 +577,6 @@ EclipseGrid::EclipseGrid(const Deck& deck, const int * actnum)
         return m_active_to_global.at(active_index);
     }
 
-    size_t EclipseGrid::getGlobalIndex(size_t i, size_t j, size_t k) const {
-
-        return GridDims::getGlobalIndex(i,j,k);
-    }
-
-
     bool EclipseGrid::isPinchActive( ) const {
         return m_pinch.has_value();
     }
@@ -609,24 +610,31 @@ EclipseGrid::EclipseGrid(const Deck& deck, const int * actnum)
         return m_minpvVector;
     }
 
-    void EclipseGrid::initBinaryGrid(const Deck& deck) {
+    void EclipseGrid::initBinaryGrid(const Deck& deck)
+    {
+        const auto& gdfile = deck["GDFILE"].back().getRecord(0);
 
-        const DeckKeyword& gdfile_kw = deck["GDFILE"].back();
-        const std::string& gdfile_arg = gdfile_kw.getRecord(0).getItem("filename").getTrimmedString(0);
-        std::string filename = deck.makeDeckPath(gdfile_arg);
+        const auto formatted = EclIO::EclFile::Formatted {
+            gdfile.getItem("formatted").getTrimmedString(0).front() == 'F'
+        };
 
-        std::unique_ptr<Opm::EclIO::EclFile> egridfile;
-        // Turns out some windows applications export .DATA files with relative
-        // GDFILE keywords with a windows formatted path, if open fails we give
-        // it one more try with the replacement '\'' -> '/'.
+        auto filename = deck
+            .makeDeckPath(gdfile.getItem("filename").getTrimmedString(0));
+
+        std::unique_ptr<EclIO::EclFile> egridfile{};
+
+        // Some windows applications export .DATA files with relative GDFILE
+        // keywords with a windows formatted path.  If open fails we give it
+        // one more try with the replacement '\'' -> '/'.
         try {
-            egridfile = std::make_unique<Opm::EclIO::EclFile>(filename);
-        } catch (const std::runtime_error& ) {
+            egridfile = std::make_unique<Opm::EclIO::EclFile>(filename, formatted);
+        }
+        catch (const std::runtime_error&) {
             std::replace(filename.begin(), filename.end(), '\\', '/');
-            egridfile = std::make_unique<Opm::EclIO::EclFile>(filename);
+            egridfile = std::make_unique<Opm::EclIO::EclFile>(filename, formatted);
         }
 
-        initGridFromEGridFile(*egridfile, filename);
+        this->initGridFromEGridFile(*egridfile, filename);
     }
 
     void EclipseGrid::initCartesianGrid(const Deck& deck) {
@@ -712,13 +720,11 @@ EclipseGrid::EclipseGrid(const Deck& deck, const int * actnum)
 
 
         for (int  n=0; n<4; n++) {
-            double xt = m_coord[pind[n]];
-            double yt = m_coord[pind[n] + 1];
-            double zt = m_coord[pind[n] + 2];
+            const double xt = m_coord[pind[n]];
+            const double yt = m_coord[pind[n] + 1];
+            const double zt = m_coord[pind[n] + 2];
 
-            double xb = m_coord[pind[n] + 3];
-            double yb = m_coord[pind[n] + 4];
-            double zb = m_coord[pind[n]+5];
+            const double zb = m_coord[pind[n]+5];
 
             if (zt == zb) {
                 X[n] = xt;
@@ -727,6 +733,8 @@ EclipseGrid::EclipseGrid(const Deck& deck, const int * actnum)
                 Y[n] = yt;
                 Y[n + 4] = yt;
             } else {
+                const double xb = m_coord[pind[n] + 3];
+                const double yb = m_coord[pind[n] + 4];
                 X[n] = xt + (xb-xt) / (zt-zb) * (zt - Z[n]);
                 X[n+4] = xt + (xb-xt) / (zt-zb) * (zt-Z[n+4]);
 
@@ -897,12 +905,12 @@ EclipseGrid::EclipseGrid(const Deck& deck, const int * actnum)
 
         for (std::size_t j = 0; j < ny; j++) {
 
-            double y0 = 0;
             double zt = tops[0];
             double zb = zt + sum_kdir[0 + 0*nx];
 
             if (j == 0) {
                 double x0 = 0.0;
+                double y0 = 0;
 
                 coord.push_back(x0);
                 coord.push_back(y0);
@@ -1116,9 +1124,7 @@ EclipseGrid::EclipseGrid(const Deck& deck, const int * actnum)
             throw std::invalid_argument("TOPS keyword should have exactly " + std::to_string( this->getNX() * this->getNY() ) + " elements");
 
         {
-            double total_angle = 0;
-            for (auto theta : dthetav)
-                total_angle += theta;
+            double total_angle = std::accumulate(dthetav.begin(), dthetav.end(), 0.0);
 
             if (std::abs( total_angle - 360 ) < 0.01)
                 m_circle = deck.hasKeyword<ParserKeywords::CIRCLE>();
@@ -1385,7 +1391,6 @@ EclipseGrid::EclipseGrid(const Deck& deck, const int * actnum)
     std::vector<double> EclipseGrid::createTOPSVector(const std::array<int, 3>& dims,
             const std::vector<double>& DZ, const Deck& deck)
     {
-        double z_tolerance = 1e-6;
         size_t volume = dims[0] * dims[1] * dims[2];
         size_t area = dims[0] * dims[1];
         const auto& TOPSKeyWord = deck.get<ParserKeywords::TOPS>().back();
@@ -1402,6 +1407,7 @@ EclipseGrid::EclipseGrid(const Deck& deck, const int * actnum)
                 if (targetIndex >= initialTOPSize)
                     TOPS[targetIndex] = nextValue;
                 else {
+                    constexpr double z_tolerance = 1e-6;
                     if (std::abs(nextValue - TOPS[targetIndex]) < z_tolerance)
                         TOPS[targetIndex] = nextValue;
                 }
@@ -1419,12 +1425,11 @@ std::vector<double> EclipseGrid::createDVector(const std::array<int,3>& dims, st
             const std::string& DVKey, const Deck& deck)
     {
         size_t volume = dims[0] * dims[1] * dims[2];
-        size_t area = dims[0] * dims[1];
         std::vector<double> D;
         if (deck.hasKeyword(DKey)) {
             D = deck[DKey].back().getSIDoubleData();
 
-
+            const size_t area = dims[0] * dims[1];
             if (D.size() >= area && D.size() < volume) {
                 /*
                   Only the top layer is required; for layers below the
@@ -1524,6 +1529,16 @@ std::vector<double> EclipseGrid::createDVector(const std::array<int,3>& dims, st
         return this->cellActive(globalIndex);
     }
 
+    bool EclipseGrid::cellActiveAfterMINPV( size_t i , size_t j , size_t k, double cell_porv ) const {
+        assertIJK(i,j,k);
+
+        size_t globalIndex = getGlobalIndex(i,j,k);
+        if (!this->cellActive(globalIndex))
+            return false;
+
+        return m_minpvMode == MinpvMode::Inactive || cell_porv >= m_minpvVector[globalIndex];
+    }
+
     const std::vector<double>& EclipseGrid::activeVolume() const {
         if (!this->active_volume.has_value()) {
             std::vector<double> volume(this->m_nactive);
@@ -1537,8 +1552,8 @@ std::vector<double> EclipseGrid::createDVector(const std::array<int,3>& dims, st
                 this->getCellCorners(global_index, X, Y, Z );
                 if (m_rv && m_thetav) {
                     const auto[i,j,k] = this->getIJK(global_index);
-                    auto& r = *m_rv;
-                    auto& t = *m_thetav;
+                    const auto& r = *m_rv;
+                    const auto& t = *m_thetav;
                     volume[active_index] = calculateCylindricalCellVol(r[i], r[i+1], t[j], Z[4] - Z[0]);
                 } else
                     volume[active_index] = calculateCellVol(X, Y, Z);
@@ -1564,8 +1579,8 @@ std::vector<double> EclipseGrid::createDVector(const std::array<int,3>& dims, st
         this->getCellCorners(globalIndex, X, Y, Z );
         if (m_rv && m_thetav) {
             const auto[i,j,k] = this->getIJK(globalIndex);
-            auto& r = *m_rv;
-            auto& t = *m_thetav;
+            const auto& r = *m_rv;
+            const auto& t = *m_thetav;
            return calculateCylindricalCellVol(r[i], r[i+1], t[j], Z[4] - Z[0]);
         } else {
             return calculateCellVol(X, Y, Z);
@@ -1641,6 +1656,57 @@ std::vector<double> EclipseGrid::createDVector(const std::array<int,3>& dims, st
         }
     }
 
+
+    std::tuple<std::array<double, 3>,std::array<double, 3>,std::array<double, 3>>
+    EclipseGrid::getCellAndBottomCenterNormal(size_t globalIndex) const {
+        // bottom face is spanned by corners 4, ..., 7 and connects cells
+        // with vertical index k and k+1
+        assertGlobalIndex( globalIndex );
+        std::array<double,8> X;
+        std::array<double,8> Y;
+        std::array<double,8> Z;
+        this->getCellCorners(globalIndex, X, Y, Z );
+        std::array<double,3> bottomCenter{{std::accumulate(X.begin() + 4, X.end(), 0.0) / 4.0,
+                                           std::accumulate(Y.begin() + 4, Y.end(), 0.0) / 4.0,
+                                           std::accumulate(Z.begin() + 4, Z.end(), 0.0) / 4.0}};
+
+        // Calculate normal scaled with area via the triangles spanned by the center and two neighboring
+        // corners.
+        std::array<double,3> bottomFaceNormal{};
+        // Reorder counter-clock.wise
+        std::array<std::size_t,4> bottomIndices = {4, 5, 7, 6};
+        std::array<double,3> oldCorner{ *(X.begin() + 6),
+                                        *(Y.begin() + 6),
+                                        *(Z.begin() + 6)};
+
+        for(const auto cornerIndex: bottomIndices)
+        {
+            std::array<double,3> newCorner{ *(X.begin() + cornerIndex), *(Y.begin() + cornerIndex),
+                                            *(Z.begin() + cornerIndex) };
+            std::array<double,3> side1, side2, normalTriangle;
+            std::transform(oldCorner.begin(), oldCorner.end(), bottomCenter.begin(),
+                           side1.begin(), std::minus<double>());
+            std::transform(newCorner.begin(), newCorner.end(), bottomCenter.begin(),
+                           side2.begin(), std::minus<double>());
+
+            cross(side1, side2, normalTriangle);
+            // use std::plus to make normal point outwards (Z-axis points downwards)
+            std::transform(bottomFaceNormal.begin(), bottomFaceNormal.end(),
+                           normalTriangle.begin(), bottomFaceNormal.begin(),
+                           std::plus<double>());
+            oldCorner = newCorner;
+        }
+
+        std::transform(bottomFaceNormal.begin(), bottomFaceNormal.end(),
+                       bottomFaceNormal.begin(), [](const double& a){ return 0.5 * a;});
+        std::array<double,3> cellCenter = { std::accumulate(X.begin(), X.end(), 0.0) / 8.0,
+                                             std::accumulate(Y.begin(), Y.end(), 0.0) / 8.0,
+                                             std::accumulate(Z.begin(), Z.end(), 0.0) / 8.0 };
+            return std::make_tuple(cellCenter,
+                                   bottomCenter,
+                                   bottomFaceNormal);
+    }
+
     std::array<double, 3> EclipseGrid::getCellCenter(size_t globalIndex) const {
         assertGlobalIndex( globalIndex );
         std::array<double,8> X;
@@ -1760,6 +1826,10 @@ std::vector<double> EclipseGrid::createDVector(const std::array<int,3>& dims, st
         return this->getCellDepth(globalIndex);
     }
 
+    const std::map<size_t, std::array<int,2>>& EclipseGrid::getAquiferCellTabnums() const {
+        return m_aquifer_cell_tabnums;
+    }
+
     const std::vector<int>& EclipseGrid::getACTNUM( ) const {
 
         return m_actnum;
@@ -1807,7 +1877,7 @@ std::vector<double> EclipseGrid::createDVector(const std::array<int,3>& dims, st
         std::vector<float> zcorn_f;
         zcorn_f.resize(m_zcorn.size());
 
-        if (m_input_coord.has_value()) {
+        if (m_input_zcorn.has_value()) {
             std::transform(m_input_zcorn.value().begin(), m_input_zcorn.value().end(), zcorn_f.begin(), convert_length);
         } else {
             std::transform(m_zcorn.begin(), m_zcorn.end(), zcorn_f.begin(), convert_length);
@@ -1969,6 +2039,11 @@ std::vector<double> EclipseGrid::createDVector(const std::array<int,3>& dims, st
 
                 if (! record.getItem<AQUNUM::DEPTH>().defaultApplied(0))
                     this->m_aquifer_cell_depths.insert_or_assign(global_index, record.getItem<AQUNUM::DEPTH>().getSIDouble(0));
+
+                // Create map global_index -> (PVTNUM, SATNUM) to allow QC during FieldProps creation
+                const int pvtnum = record.getItem<AQUNUM::PVT_TABLE_NUM>().defaultApplied(0) ? 0 : record.getItem<AQUNUM::PVT_TABLE_NUM>().get<int>(0);
+                const int satnum = record.getItem<AQUNUM::SAT_TABLE_NUM>().defaultApplied(0) ? 0 : record.getItem<AQUNUM::SAT_TABLE_NUM>().get<int>(0);
+                this->m_aquifer_cell_tabnums.insert_or_assign(global_index, std::array<int,2>{pvtnum, satnum});
             }
         }
     }
