@@ -19,6 +19,8 @@
 #ifndef FIELDPROPS_HPP
 #define FIELDPROPS_HPP
 
+#include <opm/common/utility/OpmInputError.hpp>
+
 #include <opm/input/eclipse/EclipseState/Grid/Box.hpp>
 #include <opm/input/eclipse/EclipseState/Grid/FieldData.hpp>
 #include <opm/input/eclipse/EclipseState/Grid/Keywords.hpp>
@@ -42,6 +44,7 @@
 #include <stdexcept>
 #include <string>
 #include <string_view>
+#include <type_traits>
 #include <unordered_map>
 #include <unordered_set>
 #include <utility>
@@ -132,15 +135,15 @@ namespace ALIAS {
 
 namespace GRID {
 static const std::unordered_map<std::string, keyword_info<double>> double_keywords = {{"DISPERC",keyword_info<double>{}.unit_string("Length")},
-                                                                                      {"MINPVV",  keyword_info<double>{}.unit_string("ReservoirVolume").global_kw(true)},
-                                                                                      {"MULTPV",  keyword_info<double>{}.init(1.0)},
+                                                                                      {"MINPVV",  keyword_info<double>{}.init(0.0).unit_string("ReservoirVolume").global_kw(true)},
+                                                                                      {"MULTPV",  keyword_info<double>{}.init(1.0).mult(true)},
                                                                                       {"NTG",     keyword_info<double>{}.init(1.0)},
                                                                                       {"PORO",    keyword_info<double>{}.distribute_top(true)},
-                                                                                      {"PERMX",   keyword_info<double>{}.unit_string("Permeability").distribute_top(true)},
-                                                                                      {"PERMY",   keyword_info<double>{}.unit_string("Permeability").distribute_top(true)},
-                                                                                      {"PERMZ",   keyword_info<double>{}.unit_string("Permeability").distribute_top(true)},
-                                                                                      {"PERMR",   keyword_info<double>{}.unit_string("Permeability").distribute_top(true)},
-                                                                                      {"PERMTHT",   keyword_info<double>{}.unit_string("Permeability").distribute_top(true)},
+                                                                                      {"PERMX",   keyword_info<double>{}.unit_string("Permeability").distribute_top(true).global_kw_until_edit()},
+                                                                                      {"PERMY",   keyword_info<double>{}.unit_string("Permeability").distribute_top(true).global_kw_until_edit()},
+                                                                                      {"PERMZ",   keyword_info<double>{}.unit_string("Permeability").distribute_top(true).global_kw_until_edit()},
+                                                                                      {"PERMR",   keyword_info<double>{}.unit_string("Permeability").distribute_top(true).global_kw_until_edit()},
+                                                                                      {"PERMTHT",   keyword_info<double>{}.unit_string("Permeability").distribute_top(true).global_kw_until_edit()},
                                                                                       {"TEMPI",   keyword_info<double>{}.unit_string("Temperature")},
                                                                                       {"THCONR",  keyword_info<double>{}.unit_string("Energy/AbsoluteTemperature*Length*Time")},
                                                                                       {"THCONSF", keyword_info<double>{}},
@@ -181,7 +184,7 @@ namespace EDIT {
   due to the special treatment of the TRAN fields.
 */
 
-static const std::unordered_map<std::string, keyword_info<double>> double_keywords = {{"MULTPV",  keyword_info<double>{}.init(1.0)},
+static const std::unordered_map<std::string, keyword_info<double>> double_keywords = {{"MULTPV",  keyword_info<double>{}.init(1.0).mult(true)},
                                                                                       {"PORV",    keyword_info<double>{}.unit_string("ReservoirVolume")},
                                                                                       {"MULTX",   keyword_info<double>{}.init(1.0).mult(true)},
                                                                                       {"MULTX-",  keyword_info<double>{}.init(1.0).mult(true)},
@@ -283,6 +286,10 @@ static const std::unordered_map<std::string, keyword_info<double>> double_keywor
                                                                                       {"RVW",      keyword_info<double>{}.unit_string("OilDissolutionFactor")}
                                                                                       };
 
+static const std::unordered_map<std::string, keyword_info<double>> composition_keywords = {{"XMF", keyword_info<double>{}},
+                                                                                           {"YMF", keyword_info<double>{}},
+                                                                                           {"ZMF", keyword_info<double>{}},
+                                                                                          };
 }
 
 namespace SCHEDULE {
@@ -304,7 +311,7 @@ keyword_info<T> global_kw_info(const std::string& name, bool allow_unsupported =
 bool is_oper_keyword(const std::string& name);
 } // end namespace keywords
 
-} // end namespace FieldProps
+} // end namespace Fieldprops
 
 class FieldProps {
 public:
@@ -331,71 +338,203 @@ public:
         }
     };
 
+    /// Property array existence status
     enum class GetStatus {
-         OK = 1,
-         INVALID_DATA = 2,               // std::runtime_error
-         MISSING_KEYWORD = 3,            // std::out_of_range
-         NOT_SUPPPORTED_KEYWORD = 4      // std::logic_error
+        /// Property exists and its property data is fully defined.
+        OK = 1,
+
+        /// Property array has not been fully initialised.
+        ///
+        /// At least one data element is uninitialised or has an empty
+        /// default.
+        ///
+        /// Will throw an exception of type \code std::runtime_error
+        /// \endcode in FieldDataManager::verify_status().
+        INVALID_DATA = 2,
+
+        /// Property has not yet been defined in the input file.
+        ///
+        /// Will throw an exception of type \code std::out_of_range \endcode
+        /// in FieldDataManager::verify_status().
+        MISSING_KEYWORD = 3,
+
+        /// Named property is not known to the internal handling mechanism.
+        ///
+        /// Might for instance be because the client code requested a
+        /// property of type \c int, but the name is only known as property
+        /// of type \c double, or because there was a misprint in the
+        /// property name.
+        ///
+        /// Will throw an exception of type \code std::logic_error \endcode
+        /// in FieldDataManager::verify_status().
+        NOT_SUPPPORTED_KEYWORD = 4,
     };
 
+    /// Wrapper type for field properties
+    ///
+    /// \tparam T Property element type.  Typically \c double or \c int.
     template<typename T>
-    struct FieldDataManager {
+    struct FieldDataManager
+    {
+        /// Property name.
         const std::string& keyword;
+
+        /// Request status.
         GetStatus status;
-        using Data = Fieldprops::FieldData<T>;
-        const Data * data_ptr;
 
-        FieldDataManager(const std::string& k, GetStatus s, const Data * d) :
-            keyword(k),
-            status(s),
-            data_ptr(d)
-        { }
+        /// Property data.
+        const Fieldprops::FieldData<T>* data_ptr;
 
+        /// Constructor
+        ///
+        /// \param[in] k Property name
+        /// \param[in] s Request status
+        /// \param[in] d Property data.  Pass \c nullptr for missing property data.
+        FieldDataManager(const std::string& k, GetStatus s, const Fieldprops::FieldData<T>* d)
+            : keyword(k)
+            , status(s)
+            , data_ptr(d)
+        {}
 
-        void verify_status() const {
+        /// Validate result of \code try_get<>() \endcode request
+        ///
+        /// Throws an exception of type \code OpmInputError \endcode if
+        /// \code this->status \endcode is not \code GetStatus::OK \endcode.
+        /// Does nothing otherwise.
+        ///
+        /// \param[in] Input keyword which prompted request.
+        ///
+        /// \param[in] descr Textual description of context in which request
+        ///   occurred.
+        ///
+        /// \param[in] operation Name of operation which prompted request.
+        void verify_status(const KeywordLocation& loc,
+                           const std::string&     descr,
+                           const std::string&     operation) const
+        {
+            switch (this->status) {
+            case FieldProps::GetStatus::OK:
+                return;
+
+            case FieldProps::GetStatus::INVALID_DATA:
+                throw OpmInputError {
+                    descr + " " + this->keyword +
+                    " is not fully initialised for " + operation,
+                    loc
+                };
+
+            case FieldProps::GetStatus::MISSING_KEYWORD:
+                throw OpmInputError {
+                    descr + " " + this->keyword +
+                    " does not exist in input deck for " + operation,
+                    loc
+                };
+
+            case FieldProps::GetStatus::NOT_SUPPPORTED_KEYWORD:
+                throw OpmInputError {
+                    descr + " " + this->keyword +
+                    " is not supported for " + operation,
+                    loc
+                };
+            }
+        }
+
+        /// Validate result of \code try_get<>() \endcode request
+        ///
+        /// Throws an exception as outlined in \c GetStatus if \code
+        /// this->status \endcode is not \code GetStatus::OK \endcode.  Does
+        /// nothing otherwise.
+        void verify_status() const
+        {
             switch (status) {
             case FieldProps::GetStatus::OK:
                 return;
+
             case FieldProps::GetStatus::INVALID_DATA:
                 throw std::runtime_error("The keyword: " + keyword + " has not been fully initialized");
+
             case FieldProps::GetStatus::MISSING_KEYWORD:
                 throw std::out_of_range("No such keyword in deck: " + keyword);
+
             case FieldProps::GetStatus::NOT_SUPPPORTED_KEYWORD:
                 throw std::logic_error("The keyword  " + keyword + " is not supported");
             }
         }
 
-        const std::vector<T>* ptr() const {
-            if (this->data_ptr)
-                return std::addressof(this->data_ptr->data);
-            else
-                return nullptr;
+        /// Access underlying property data elements
+        ///
+        /// Returns \c nullptr if property data is not fully defined.
+        const std::vector<T>* ptr() const
+        {
+            return (this->data_ptr != nullptr)
+                ? &this->data_ptr->data
+                : nullptr;
         }
 
-        const std::vector<T>& data() const {
+        /// Access underlying property data elements
+        ///
+        /// Throws an exception as outlined in \c GetStatus if property data
+        /// is not fully defined.
+        const std::vector<T>& data() const
+        {
             this->verify_status();
             return this->data_ptr->data;
         }
 
-        const Data& field_data() const {
+        /// Read-only access to contained FieldData object
+        ///
+        /// Throws an exception as outlined in \c GetStatus if property data
+        /// is not fully defined.
+        const Fieldprops::FieldData<T>& field_data() const
+        {
             this->verify_status();
             return *this->data_ptr;
         }
 
-        bool valid() const {
-            return (this->status == GetStatus::OK);
+        /// Property validity predicate.
+        ///
+        /// Returns true if property exists and has fully defined data
+        /// elements.  False otherwise.
+        bool valid() const
+        {
+            return this->status == GetStatus::OK;
         }
+    };
 
+    /// Options to restrict or relax a try_get() request.
+    ///
+    /// For instance, the source array of a COPY operation must exist and be
+    /// fully defined, whereas the target array need not be either.  On the
+    /// other hand, certain use cases might want to look up property names
+    /// of an unsupported keyword type--e.g., transmissibility keywords
+    /// among the integer properties--and those requests should specify the
+    /// AllowUnsupported flag.
+    ///
+    /// Note: We would ideally use strong enums (i.e., "enum class") for
+    /// this, but those don't mesh very well with bitwise operations.
+    enum TryGetFlags : unsigned int {
+        /// Whether or not to permit looking up property names of unmatching
+        /// types.
+        AllowUnsupported = (1u << 0),
+
+        /// Whether or not the property must already exist.
+        MustExist = (1u << 1),
     };
 
     /// Normal constructor for FieldProps.
-    FieldProps(const Deck& deck, const Phases& phases, const EclipseGrid& grid, const TableManager& table_arg);
+    FieldProps(const Deck& deck,
+               const Phases& phases,
+               EclipseGrid& grid,
+               const TableManager& table_arg,
+               const std::size_t ncomps);
 
     /// Special case constructor used to process ACTNUM only.
     FieldProps(const Deck& deck, const EclipseGrid& grid);
 
     void reset_actnum(const std::vector<int>& actnum);
 
+    void prune_global_for_schedule_run();
+
     void apply_numerical_aquifers(const NumericalAquifers& numerical_aquifers);
 
     const std::string& default_region() const;
@@ -412,18 +551,39 @@ public:
     template <typename T>
     std::vector<std::string> keys() const;
 
+    /// Request read-only property array from internal cache
+    ///
+    /// Will create property array if permitted and possible.
+    ///
+    /// \tparam T Property element type.  Typically \c double or \c int.
+    ///
+    /// \param[in] keyword Property name
+    ///
+    /// \param[in] flags Options to limit or relax request processing.
+    ///   Treated as a bitwise mask of \c TryGetFlags.
+    ///
+    /// \return Access structure for requested property array.
     template <typename T>
     FieldDataManager<T>
-    try_get(const std::string& keyword, const bool allow_unsupported = false)
+    try_get(const std::string& keyword, const unsigned int flags = 0u)
     {
+        const auto allow_unsupported =
+            (flags & TryGetFlags::AllowUnsupported) != 0u;
+
         if (!allow_unsupported && !FieldProps::template supported<T>(keyword)) {
             return { keyword, GetStatus::NOT_SUPPPORTED_KEYWORD, nullptr };
         }
 
         const auto has0 = this->template has<T>(keyword);
+        if (!has0 && ((flags & TryGetFlags::MustExist) != 0)) {
+            // Client requested a property which must exist, e.g., as a
+            // source array for a COPY operation, but the property has not
+            // (yet) been defined in the run's input.
+            return { keyword, GetStatus::MISSING_KEYWORD, nullptr };
+        }
 
-        const auto& field_data =
-            this->template init_get<T>(keyword, std::is_same<T,double>::value && allow_unsupported);
+        const auto& field_data = this->template
+            init_get<T>(keyword, std::is_same_v<T, double> && allow_unsupported);
 
         if (field_data.valid() || allow_unsupported) {
             // Note: FieldDataManager depends on init_get<>() producing a
@@ -432,11 +592,15 @@ public:
         }
 
         if (! has0) {
+            // Client requested a property which did not exist and which
+            // could not be created from a default description.
             this->template erase<T>(keyword);
 
             return { keyword, GetStatus::MISSING_KEYWORD, nullptr };
         }
 
+        // If we get here then the property exists but has not been fully
+        // defined yet.
         return { keyword, GetStatus::INVALID_DATA, nullptr };
     }
 
@@ -489,7 +653,7 @@ public:
     std::vector<bool> defaulted(const std::string& keyword)
     {
         const auto& field = this->template init_get<T>(keyword);
-        std::vector<bool> def(field.size());
+        std::vector<bool> def(field.numCells());
 
         for (std::size_t i = 0; i < def.size(); ++i) {
             def[i] = value::defaulted(field.value_status[i]);
@@ -533,6 +697,7 @@ public:
     void handle_schedule_keywords(const std::vector<DeckKeyword>& keywords);
     bool tran_active(const std::string& keyword) const;
     void apply_tran(const std::string& keyword, std::vector<double>& data);
+    void apply_tranz_global(const std::vector<size_t>& indices, std::vector<double>& data) const;
     bool operator==(const FieldProps& other) const;
     static bool rst_cmp(const FieldProps& full_arg, const FieldProps& rst_arg);
 
@@ -545,15 +710,19 @@ public:
 
     void deleteMINPVV();
 
+    void set_active_indices(const std::vector<int>& indices);
+
 private:
+    void processMULTREGP(const Deck& deck);
     void scanGRIDSection(const GRIDSection& grid_section);
     void scanGRIDSectionOnlyACTNUM(const GRIDSection& grid_section);
     void scanEDITSection(const EDITSection& edit_section);
     void scanPROPSSection(const PROPSSection& props_section);
     void scanREGIONSSection(const REGIONSSection& regions_section);
-    void scanSOLUTIONSection(const SOLUTIONSection& solution_section);
+    void scanSOLUTIONSection(const SOLUTIONSection& solution_section, const std::size_t ncomps);
     double getSIValue(const std::string& keyword, double raw_value) const;
     double getSIValue(ScalarOperation op, const std::string& keyword, double raw_value) const;
+
     template <typename T>
     void erase(const std::string& keyword);
 
@@ -577,34 +746,59 @@ private:
     }
 
     template <typename T>
-    void operate(const DeckRecord& record, Fieldprops::FieldData<T>& target_data, const Fieldprops::FieldData<T>& src_data, const std::vector<Box::cell_index>& index_list);
+    void operate(const DeckRecord& record,
+                 Fieldprops::FieldData<T>& target_data,
+                 const Fieldprops::FieldData<T>& src_data,
+                 const std::vector<Box::cell_index>& index_list,
+                 const bool global = false);
 
     template <typename T>
-    static void apply(ScalarOperation op, std::vector<T>& data, std::vector<value::status>& value_status, T scalar_value, const std::vector<Box::cell_index>& index_list);
+    Fieldprops::FieldData<T>&
+    init_get(const std::string& keyword, bool allow_unsupported = false);
 
     template <typename T>
-    Fieldprops::FieldData<T>& init_get(const std::string& keyword, bool allow_unsupported = false);
+    Fieldprops::FieldData<T>&
+    init_get(const std::string& keyword,
+             const Fieldprops::keywords::keyword_info<T>& kw_info,
+             const bool multiplier_in_edit = false);
 
-    template <typename T>
-    Fieldprops::FieldData<T>& init_get(const std::string& keyword, const Fieldprops::keywords::keyword_info<T>& kw_info, bool multiplier_in_edit = false);
+    std::string region_name(const DeckItem& region_item) const;
+
+    std::pair<std::vector<Box::cell_index>,bool>
+    region_index(const std::string& region_name, int region_value);
 
-    std::string region_name(const DeckItem& region_item);
-    std::vector<Box::cell_index> region_index( const std::string& region_name, int region_value );
     void handle_OPERATE(const DeckKeyword& keyword, Box box);
     void handle_operation(Section section, const DeckKeyword& keyword, Box box);
+    void handle_operateR(const DeckKeyword& keyword);
     void handle_region_operation(const DeckKeyword& keyword);
     void handle_COPY(const DeckKeyword& keyword, Box box, bool region);
-    void distribute_toplayer(Fieldprops::FieldData<double>& field_data, const std::vector<double>& deck_data, const Box& box);
+    void distribute_toplayer(Fieldprops::FieldData<double>& field_data,
+                             const std::vector<double>& deck_data,
+                             const Box& box);
+
     double get_beta(const std::string& func_name, const std::string& target_array, double raw_beta);
     double get_alpha(const std::string& func_name, const std::string& target_array, double raw_alpha);
 
     void handle_keyword(Section section, const DeckKeyword& keyword, Box& box);
-    void handle_double_keyword(Section section, const Fieldprops::keywords::keyword_info<double>& kw_info, const DeckKeyword& keyword, const std::string& keyword_name, const Box& box);
-    void handle_double_keyword(Section section, const Fieldprops::keywords::keyword_info<double>& kw_info, const DeckKeyword& keyword, const Box& box);
-    void handle_int_keyword(const Fieldprops::keywords::keyword_info<int>& kw_info, const DeckKeyword& keyword, const Box& box);
+    void handle_double_keyword(Section section,
+                               const Fieldprops::keywords::keyword_info<double>& kw_info,
+                               const DeckKeyword& keyword,
+                               const std::string& keyword_name,
+                               const Box& box);
+
+    void handle_double_keyword(Section section,
+                               const Fieldprops::keywords::keyword_info<double>& kw_info,
+                               const DeckKeyword& keyword,
+                               const Box& box);
+
+    void handle_int_keyword(const Fieldprops::keywords::keyword_info<int>& kw_info,
+                            const DeckKeyword& keyword,
+                            const Box& box);
+
     void init_satfunc(const std::string& keyword, Fieldprops::FieldData<double>& satfunc);
     void init_porv(Fieldprops::FieldData<double>& porv);
     void init_tempi(Fieldprops::FieldData<double>& tempi);
+
     std::string canonical_fipreg_name(const std::string& fipreg);
     const std::string& canonical_fipreg_name(const std::string& fipreg) const;
 
@@ -626,6 +820,7 @@ private:
     Phases m_phases;
     SatFuncControls m_satfuncctrl;
     std::vector<int> m_actnum;
+    std::unordered_map<int,int> m_active_index;
     std::vector<double> cell_volume;
     std::vector<double> cell_depth;
     const std::string m_default_region;
