@@ -162,7 +162,9 @@ struct SummaryConfigContext {
          {"GMGP",     {"RGMGP", "FGMGP"}},
          {"GMDS",     {"RGMDS", "FGMDS"}},
          {"GMTR",     {"RGMTR", "FGMTR"}},
+         {"GMST",     {"RGMST", "FGMST"}},
          {"GMMO",     {"RGMMO", "FGMMO"}},
+         {"GMUS",     {"RGMUS", "FGMUS"}},
          {"GKTR",     {"RGKTR", "FGKTR"}},
          {"GKMO",     {"RGKMO", "FGKMO"}},
     };
@@ -236,7 +238,7 @@ struct SummaryConfigContext {
         static const keyword_set ratekw {
             "OPR", "GPR", "WPR", "GLIR", "LPR", "NPR", "CPR", "VPR", "TPR", "TPC",
 
-            "OFR", "OFRF", "OFRS", "OFR+", "OFR-",
+            "OFR", "OFRF", "OFRS", "OFR+", "OFR-", "TFR",
             "GFR", "GFRF", "GFRS", "GFR+", "GFR-",
             "WFR", "WFR+", "WFR-",
 
@@ -249,6 +251,7 @@ struct SummaryConfigContext {
             "OIGR", "GIGR", "WIGR",
             "OIRH", "GIRH", "WIRH",
             "OVIR", "GVIR", "WVIR",
+            "GMIR",
 
             "OPI", "OPP", "GPI", "GPP", "WPI", "WPP",
 
@@ -257,7 +260,7 @@ struct SummaryConfigContext {
 
         return is_in_set(ratekw, keyword.substr(1))
             || ((keyword.length() > 4) &&
-                is_in_set({ "TPR", "TPC", "TIR", "TIC" },
+                is_in_set({ "TPR", "TPC", "TIR", "TIC", "TFR" },
                           keyword.substr(1, 3)));
     }
 
@@ -282,7 +285,7 @@ struct SummaryConfigContext {
             "WFT", "WFT+", "WFT-",
 
             "WIT", "OIT", "GIT", "LIT", "NIT", "CIT", "VIT", "TIT",
-            "WITH", "OITH", "GITH", "WVIT", "OVIT", "GVIT",
+            "WITH", "OITH", "GITH", "WVIT", "OVIT", "GVIT", "GMIT",
 
             "AQT", "AQTG", "NQT",
         };
@@ -504,18 +507,26 @@ void handleMissingAquifer( const ParseContext& parseContext,
 
 inline void keywordW( SummaryConfig::keyword_list& list,
                       const std::vector<std::string>& well_names,
-                      SummaryConfigNode baseWellParam) {
-    for (const auto& wname : well_names)
-        list.push_back( baseWellParam.namedEntity(wname) );
+                      SummaryConfigNode baseWellParam)
+{
+    std::transform(well_names.begin(), well_names.end(),
+                   std::back_inserter(list),
+                   [&baseWellParam](const auto& wname)
+                   {
+                       return baseWellParam.namedEntity(wname);
+                   });
 }
 
 inline void keywordAquifer( SummaryConfig::keyword_list& list,
                             const std::vector<int>& aquiferIDs,
                             SummaryConfigNode baseAquiferParam)
 {
-    for (const auto& id : aquiferIDs) {
-        list.push_back(baseAquiferParam.number(id));
-    }
+    std::transform(aquiferIDs.begin(), aquiferIDs.end(),
+                   std::back_inserter(list),
+                   [&baseAquiferParam](const auto& id)
+                   {
+                       return baseAquiferParam.number(id);
+                   });
 }
 
 // later check whether parseContext and errors are required
@@ -611,8 +622,12 @@ inline void keywordCL(SummaryConfig::keyword_list& list,
 
             node.namedEntity(wname);
             if (ijk_defaulted) {
-                for (const auto& conn : all_connections)
-                    list.push_back(node.number(1 + conn.global_index()));
+                std::transform(all_connections.begin(), all_connections.end(),
+                               std::back_inserter(list),
+                               [&node](const auto& conn)
+                               {
+                                    return node.number(1 + conn.global_index());
+                               });
             } else {
                 const auto& ijk = getijk(record);
                 auto global_index = dims.getGlobalIndex(ijk[0], ijk[1], ijk[2]);
@@ -800,12 +815,14 @@ void keyword_node( SummaryConfig::keyword_list& list,
     .isUserDefined( is_udq(keyword.name()) );
 
     if( keyword.empty() ||
-        !keyword.getDataRecord().getDataItem().hasValue( 0 ) ) {
-
-        for (const auto& node_name : node_names) {
-            list.push_back( param.namedEntity(node_name) );
-        }
-
+        !keyword.getDataRecord().getDataItem().hasValue( 0 ) )
+    {
+        std::transform(node_names.begin(), node_names.end(),
+                       std::back_inserter(list),
+                       [&param](const auto& node_name)
+                       {
+                           return param.namedEntity(node_name);
+                       });
         return;
     }
 
@@ -1070,9 +1087,8 @@ void keywordR(SummaryConfig::keyword_list& list,
         }
     }
     else {
-        for (const auto& region_id : context.regions.at(region_name.value())) {
-            regions.push_back(region_id);
-        }
+        const auto cregions = context.regions.at(region_name.value());
+        regions.assign(cregions.begin(), cregions.end());
     }
 
     // See comment on function roew() in Summary.cpp for this weirdness.
@@ -1096,9 +1112,12 @@ void keywordR(SummaryConfig::keyword_list& list,
     .fip_region   (region_name.value())
     .isUserDefined(is_udq(keyword));
 
-    for (const auto& region : regions) {
-        list.push_back(param.number(region));
-    }
+    std::transform(regions.begin(), regions.end(),
+                   std::back_inserter(list),
+                   [&param](const auto& region)
+                   {
+                       return param.number(region);
+                   });
 }
 
 
@@ -1154,10 +1173,11 @@ inline void keywordMISC( SummaryConfig::keyword_list& list,
              */
             for( const auto& connection : well.getConnections() ) {
                 auto cijk = getijk( connection );
-                int global_index = 1 + dims.getGlobalIndex(cijk[0], cijk[1], cijk[2]);
 
-                if( ijk_defaulted || ( cijk == getijk(record) ) )
+                if( ijk_defaulted || ( cijk == getijk(record) ) ) {
+                    const int global_index = 1 + dims.getGlobalIndex(cijk[0], cijk[1], cijk[2]);
                     list.push_back( param.number(global_index) );
+                }
             }
         }
     }
@@ -1190,7 +1210,8 @@ inline void keywordMISC( SummaryConfig::keyword_list& list,
             [&kw](const char* known)
         {
             return kw == known;
-        });
+        }) 
+        || is_in_set({ "STFR", "STFC" }, kw.substr(0, 4));
     }
 
 
@@ -1991,12 +2012,13 @@ const SummaryConfigNode& SummaryConfig::operator[](std::size_t index) const {
 }
 
 
-bool SummaryConfig::match(const std::string& keywordPattern) const {
-    for (const auto& keyword : this->short_keywords) {
-        if (shmatch(keywordPattern, keyword))
-            return true;
-    }
-    return false;
+bool SummaryConfig::match(const std::string& keywordPattern) const
+{
+    return std::any_of(this->short_keywords.begin(), this->short_keywords.end(),
+                        [&keywordPattern](const auto& keyword)
+                        {
+                            return shmatch(keywordPattern, keyword);
+                        });
 }
 
 SummaryConfig::keyword_list
