@@ -316,7 +316,6 @@ namespace Opm {
     void ParseContext::update(const std::string& keyString , InputErrorAction action) {
         std::vector<std::string> keys = split_string(keyString, ":|");
         for (const auto& input_key : keys) {
-            std::vector<std::string> matching_keys;
             size_t wildcard_pos = input_key.find("*");
 
             if (wildcard_pos == std::string::npos) {
@@ -328,6 +327,26 @@ namespace Opm {
         }
     }
 
+    void ParseContext::setInputSkipMode(const std::string& skip_mode) {
+        this->m_input_skip_mode = skip_mode;
+    }
+
+    bool ParseContext::isActiveSkipKeyword(const std::string& deck_name) const {
+        if (deck_name.compare(0, 4, "SKIP") != 0)
+            return false;
+
+        if (deck_name == "SKIP")
+            return true;
+
+        if (deck_name == "SKIP100" && (this->m_input_skip_mode == "100" || this->m_input_skip_mode == "all"))
+            return true;
+
+        if (deck_name == "SKIP300" && (this->m_input_skip_mode == "300" || this->m_input_skip_mode == "all"))
+            return true;
+
+        return false;
+    }
+
     const std::string ParseContext::PARSE_EXTRA_RECORDS = "PARSE_EXTRA_RECORDS";
     const std::string ParseContext::PARSE_UNKNOWN_KEYWORD = "PARSE_UNKNOWN_KEYWORD";
     const std::string ParseContext::PARSE_RANDOM_TEXT = "PARSE_RANDOM_TEXT";
