@@ -79,15 +79,22 @@ namespace {
     /// Those are allowed before RUNSPEC.
     /// \return whether keyword is global (ECHO, NOECHO, INCLUDE,
     ///         COLUMNS, FORMFEED, SKIP, SKIP100, SKIP300, END)
-    bool isGlobalKeyword(const Opm::DeckKeyword& keyword ) {
+    bool isGlobalKeyword(const Opm::DeckKeyword& keyword )
+    {
         const auto& name = keyword.name();
-        for( const auto& x : { "ECHO", "NOECHO",
-                              "INCLUDE", "COLUMNS", "FORMFEED",
-                              "SKIP", "ENDSKIP",
-                              "SKIP100", "SKIP300" } )
-            if( name == x ) return true;
-
-        return false;
+        using namespace std::string_view_literals;
+        const auto kw_list = std::array {
+            "ECHO"sv,
+            "NOECHO"sv,
+            "INCLUDE"sv,
+            "COLUMNS"sv,
+            "FORMFEED"sv,
+            "SKIP"sv,
+            "ENDSKIP"sv,
+            "SKIP100"sv,
+            "SKIP300"sv
+        };
+        return std::find(kw_list.begin(), kw_list.end(), name) != kw_list.end();
     }
 
     // If ROCKOPTS does NOT exist, then the number of records is NTPVT (= TABDIMS(2))
@@ -962,9 +969,8 @@ std::unique_ptr<RawKeyword> tryParseKeyword( ParserState& parserState, const Par
         if( line.empty() && !rawKeyword ) continue;
         if( line.empty() && !is_title ) continue;
 
-        std::string keywordString;
         std::string deck_name = str::make_deck_name( line );
-        if (deck_name == "SKIP" || deck_name == "SKIP100" || deck_name == "SKIP300") {
+        if (parserState.parseContext.isActiveSkipKeyword(deck_name)) {
             skip = true;
             auto msg = fmt::format("{:5} Reading {:<8} in {} line {} \n      ... ignoring everything until 'ENDSKIP' ... ", "", "SKIP", parserState.current_path().string(), parserState.line());
             OpmLog::info(msg);
@@ -993,7 +999,7 @@ std::unique_ptr<RawKeyword> tryParseKeyword( ParserState& parserState, const Par
                  container.
             */
             if (ParserKeyword::validDeckName(deck_name)) {
-                auto ptr = newRawKeyword( deck_name, parserState, parser, line );
+                const auto ptr = newRawKeyword( deck_name, parserState, parser, line );
                 if (ptr) {
                     rawKeyword.reset( ptr );
                     parserKeyword = parser.getParserKeywordFromDeckName(rawKeyword->getKeywordName());
@@ -1132,9 +1138,10 @@ void cleanup_deck_keyword_list(ParserState& parserState, const std::set<Opm::Ecl
     
     std::vector<std::string> keyw_names;
     keyw_names.reserve(parserState.deck.size());
-    
-    for (auto& dk_keyw : parserState.deck)
-        keyw_names.push_back(dk_keyw.name());
+
+    std::transform(parserState.deck.begin(), parserState.deck.end(),
+                   std::back_inserter(keyw_names),
+                   [](const auto& dk_keyw) { return dk_keyw.name(); });
     
     if (ignore_runspec){
 
@@ -1614,12 +1621,15 @@ bool parseState( ParserState& parserState, const Parser& parser ) {
         return m_deckParserKeywords.size();
     }
 
-    const ParserKeyword* Parser::matchingKeyword(const std::string_view& name) const {
-        for (auto iter = m_wildCardKeywords.begin(); iter != m_wildCardKeywords.end(); ++iter) {
-            if (iter->second->matches(name))
-                return iter->second;
-        }
-        return nullptr;
+    const ParserKeyword* Parser::matchingKeyword(const std::string_view& name) const
+    {
+        const auto it = std::find_if(m_wildCardKeywords.begin(),
+                                     m_wildCardKeywords.end(),
+                                     [&name](const auto& wild)
+                                     {
+                                         return wild.second->matches(name);
+                                     });
+        return it != m_wildCardKeywords.end() ? it->second : nullptr;
     }
 
     bool Parser::hasWildCardKeyword(const std::string& internalKeywordName) const {
@@ -1662,15 +1672,15 @@ void Parser::addParserKeyword( ParserKeyword parserKeyword ) {
 
     this->keyword_storage.push_back( std::move( parserKeyword ) );
     const ParserKeyword * ptr = std::addressof(this->keyword_storage.back());
-    std::string_view name( ptr->getName() );
-
     for (const auto& deck_name : ptr->deck_names())
     {
         m_deckParserKeywords[deck_name] = ptr;
     }
 
-    if (ptr->hasMatchRegex())
+    if (ptr->hasMatchRegex()) {
+        std::string_view name( ptr->getName() );
         m_wildCardKeywords[ name ] = ptr;
+    }
 
     if (ptr->isCodeKeyword())
         this->code_keywords.emplace_back( ptr->getName(), ptr->codeEnd() );
@@ -1778,13 +1788,22 @@ std::vector<std::string> Parser::getAllDeckNames () const {
 #endif
 
 
-    static bool isSectionDelimiter( const DeckKeyword& keyword ) {
+    static bool isSectionDelimiter( const DeckKeyword& keyword )
+    {
         const auto& name = keyword.name();
-        for( const auto& x : { "RUNSPEC", "GRID", "EDIT", "PROPS",
-                               "REGIONS", "SOLUTION", "SUMMARY", "SCHEDULE" } )
-            if( name == x ) return true;
+        using namespace std::string_view_literals;
+        const auto delimiters = std::array {
+            "RUNSPEC"sv,
+            "GRID"sv,
+            "EDIT"sv,
+            "PROPS"sv,
+            "REGIONS"sv,
+            "SOLUTION"sv,
+            "SUMMARY"sv,
+            "SCHEDULE"sv
+        };
 
-        return false;
+        return std::find(delimiters.begin(), delimiters.end(), name) != delimiters.end();
     }
 
     bool DeckSection::checkSectionTopology(const Deck& deck,
