@@ -40,6 +40,7 @@
 #include <opm/input/eclipse/Schedule/ScheduleState.hpp>
 #include <opm/input/eclipse/Schedule/ScheduleStatic.hpp>
 #include <opm/input/eclipse/Schedule/Well/PAvg.hpp>
+#include <opm/input/eclipse/Schedule/Well/Well.hpp>
 #include <opm/input/eclipse/Schedule/WriteRestartFileEvents.hpp>
 #include <opm/input/eclipse/Units/UnitSystem.hpp>
 
@@ -88,8 +89,22 @@ namespace Opm
     class Schedule {
     public:
         Schedule() = default;
-        ~Schedule() = default;
+
         explicit Schedule(std::shared_ptr<const Python> python_handle);
+
+        /*! \brief Construct a Schedule object from a deck.
+         *  \param deck Deck to construct Schedule from
+         *  \param fp Field property manager
+         *  \param runspec Run specification parameters to use
+         *  \param parseContext Parsing context
+         *  \param errors Error configuration
+         *  \param python Python interpreter to use
+         *  \param lowActionParsingStrictness Reduce parsing strictness for actions
+         *  \param keepKeywords Keep the schdule keywords even if there are no actions
+         *  \param output_interval Output interval to use
+         *  \param rst Restart state to use
+         *  \param tracer_config Tracer configuration to use
+         */
         Schedule(const Deck& deck,
                  const EclipseGrid& grid,
                  const FieldPropsManager& fp,
@@ -97,6 +112,8 @@ namespace Opm
                  const ParseContext& parseContext,
                  ErrorGuard& errors,
                  std::shared_ptr<const Python> python,
+                 const bool lowActionParsingStrictness = false,
+                 const bool keepKeywords = true,
                  const std::optional<int>& output_interval = {},
                  const RestartIO::RstState* rst = nullptr,
                  const TracerConfig* tracer_config = nullptr);
@@ -109,6 +126,8 @@ namespace Opm
                  const ParseContext& parseContext,
                  T&& errors,
                  std::shared_ptr<const Python> python,
+                 const bool lowActionParsingStrictness = false,
+                 const bool keepKeywords = true,
                  const std::optional<int>& output_interval = {},
                  const RestartIO::RstState* rst = nullptr,
                  const TracerConfig* tracer_config = nullptr);
@@ -118,6 +137,8 @@ namespace Opm
                  const FieldPropsManager& fp,
                  const Runspec &runspec,
                  std::shared_ptr<const Python> python,
+                 const bool lowActionParsingStrictness = false,
+                 const bool keepKeywords = true,
                  const std::optional<int>& output_interval = {},
                  const RestartIO::RstState* rst = nullptr,
                  const TracerConfig* tracer_config = nullptr);
@@ -127,6 +148,8 @@ namespace Opm
                  const ParseContext& parseContext,
                  ErrorGuard& errors,
                  std::shared_ptr<const Python> python,
+                 const bool lowActionParsingStrictness = false,
+                 const bool keepKeywords = true,
                  const std::optional<int>& output_interval = {},
                  const RestartIO::RstState* rst = nullptr);
 
@@ -136,12 +159,16 @@ namespace Opm
                  const ParseContext& parseContext,
                  T&& errors,
                  std::shared_ptr<const Python> python,
+                 const bool lowActionParsingStrictness = false,
+                 const bool keepKeywords = true,
                  const std::optional<int>& output_interval = {},
                  const RestartIO::RstState* rst = nullptr);
 
         Schedule(const Deck& deck,
                  const EclipseState& es,
                  std::shared_ptr<const Python> python,
+                 const bool lowActionParsingStrictness = false,
+                 const bool keepKeywords = true,
                  const std::optional<int>& output_interval = {},
                  const RestartIO::RstState* rst = nullptr);
 
@@ -151,6 +178,8 @@ namespace Opm
                  const std::optional<int>& output_interval = {},
                  const RestartIO::RstState* rst = nullptr);
 
+        ~Schedule() = default;
+
         static Schedule serializationTestObject();
 
         /*
@@ -203,6 +232,9 @@ namespace Opm
         std::unordered_set<int> getAquiferFluxSchedule() const;
         std::vector<Well> getWells(std::size_t timeStep) const;
         std::vector<Well> getWellsatEnd() const;
+
+        const std::unordered_map<std::string, std::set<int>>& getPossibleFutureConnections() const;
+
         void shut_well(const std::string& well_name, std::size_t report_step);
         void shut_well(const std::string& well_name);
         void stop_well(const std::string& well_name, std::size_t report_step);
@@ -241,7 +273,15 @@ namespace Opm
           updating internal datastructures after the ACTIONX keywords have been
           applied.
         */
-        SimulatorUpdate applyAction(std::size_t reportStep, const Action::ActionX& action, const std::vector<std::string>& matching_wells, const std::unordered_map<std::string, double>& wellpi);
+        SimulatorUpdate applyAction(std::size_t reportStep,
+                                    const Action::ActionX& action,
+                                    const std::vector<std::string>& matching_wells,
+                                    const std::unordered_map<std::string, double>& wellpi);
+
+        SimulatorUpdate applyAction(std::size_t reportStep,
+                                    const Action::ActionX& action,
+                                    const std::vector<std::string>& matching_wells,
+                                    const std::unordered_map<std::string, float>& wellpi);
         /*
           The runPyAction() will run the Python script in a PYACTION keyword. In
           the case of Schedule updates the recommended way of doing that from
@@ -290,6 +330,7 @@ namespace Opm
             serializer(this->completed_cells);
             serializer(this->m_treat_critical_as_non_critical);
             serializer(this->current_report_step);
+            serializer(this->m_lowActionParsingStrictness);
             serializer(this->simUpdateFromPython);
 
             this->template pack_unpack<PAvg>(serializer);
@@ -317,6 +358,20 @@ namespace Opm
             this->template pack_unpack_map<int, VFPInjTable>(serializer);
             this->template pack_unpack_map<std::string, Group>(serializer);
             this->template pack_unpack_map<std::string, Well>(serializer);
+
+            // If we are deserializing we need to setup the pointer to the
+            // unit system since this is process specific. This is safe
+            // because we set the same value in all well instances.
+            // We do some redundant assignments as these are shared_ptr's
+            // with multiple pointers to any given instance, but it is not
+            // significant so let's keep it simple.
+            if (!serializer.isSerializing()) {
+                for (auto& snapshot : snapshots) {
+                    for (auto& well : snapshot.wells) {
+                        well.second->updateUnitSystem(&m_static.m_unit_system);
+                    }
+                }
+            }
         }
 
         template <typename T, class Serializer>
@@ -470,6 +525,17 @@ namespace Opm
         WriteRestartFileEvents restart_output{};
         CompletedCells completed_cells{};
 
+        // Boolean indicating the strictness of parsing process for ActionX and PyAction.
+        // If lowActionParsingStrictness is true, the simulator tries to apply unsupported
+        // keywords, if lowActionParsingStrictness is false, the simulator only applies
+        // supported keywords.
+        bool m_lowActionParsingStrictness = false;
+
+        // This unordered_map contains possible future connections of wells that might get added through an ACTIONX.
+        // For parallel runs, this unordered_map is retrieved by the grid partitioner to ensure these connections
+        // end up on the same partition.
+        std::unordered_map<std::string, std::set<int>> possibleFutureConnections;
+
         // The current_report_step is set to the current report step when a PYACTION call is executed.
         // This is needed since the Schedule object does not know the current report step of the simulator and
         // we only allow PYACTIONS for the current and future report steps. 
@@ -512,6 +578,7 @@ namespace Opm
                                     const ScheduleGrid& grid,
                                     const std::unordered_map<std::string, double> * target_wellpi,
                                     const std::string& prefix,
+                                    const bool keepKeywords,
                                     const bool log_to_debug = false);
         void addACTIONX(const Action::ActionX& action);
         void addGroupToGroup( const std::string& parent_group, const std::string& child_group);
@@ -539,7 +606,7 @@ namespace Opm
                            std::set<std::string>* compsegs_wells = nullptr);
 
         void internalWELLSTATUSACTIONXFromPYACTION(const std::string& well_name, std::size_t report_step, const std::string& wellStatus);
-        void prefetch_cell_properties(const ScheduleGrid& grid, const DeckKeyword& keyword);
+        void prefetchPossibleFutureConnections(const ScheduleGrid& grid, const DeckKeyword& keyword);
         void store_wgnames(const DeckKeyword& keyword);
         std::vector<std::string> wellNames(const std::string& pattern,
                                            const HandlerContext& context,
