@@ -25,12 +25,15 @@
 
 #include <opm/io/eclipse/SummaryNode.hpp>
 
+#include <algorithm>
 #include <cstddef>
 #include <cstring>
 #include <ctime>
 #include <iomanip>
+#include <limits>
 #include <ostream>
 #include <set>
+#include <stdexcept>
 #include <string>
 #include <string_view>
 #include <unordered_map>
@@ -40,6 +43,47 @@
 
 namespace {
 
+    bool is_udq(std::string_view keyword)
+    {
+        // Does 'keyword' match one of the patterns
+        //   AU*, BU*, CU*, FU*, GU*, RU*, SU*, or WU*?
+        using sz_t = std::string_view::size_type;
+
+        return (keyword.size() > sz_t{1})
+            && (keyword[1] == 'U')
+            && (keyword.find_first_of("WGFCRBSA") == sz_t{0});
+    }
+
+    bool is_well_udq(std::string_view keyword)
+    {
+        // Does 'keyword' match the pattern
+        //   WU*?
+        using sz_t = std::string_view::size_type;
+
+        return (keyword.size() > sz_t{1})
+            && (keyword.compare(0, 2, "WU") == 0);
+    }
+
+    bool is_group_udq(std::string_view keyword)
+    {
+        // Does 'keyword' match the pattern
+        //   GU*?
+        using sz_t = std::string_view::size_type;
+
+        return (keyword.size() > sz_t{1})
+            && (keyword.compare(0, 2, "GU") == 0);
+    }
+
+    bool is_segment_udq(std::string_view keyword)
+    {
+        // Does 'keyword' match the pattern
+        //   SU*?
+        using sz_t = std::string_view::size_type;
+
+        return (keyword.size() > sz_t{1})
+            && (keyword.compare(0, 2, "SU") == 0);
+    }
+
     bool is_total(const std::string& key)
     {
         static const std::vector<std::string> totals = {
@@ -49,7 +93,7 @@ namespace {
             "TPT", "TIT", "CPT", "CIT", "SPT", "SIT", "EPT", "EIT", "TPTHEA", "TITHEA",
             "OFT", "OFT+", "OFT-", "OFTG", "OFTL",
             "GFT", "GFT+", "GFT-", "GFTG", "GFTL",
-            "WFT", "WFT+", "WFT-",
+            "WFT", "WFT+", "WFT-", "GMIT",
         };
 
         auto sep_pos = key.find(':');
@@ -59,12 +103,11 @@ namespace {
             return false;
 
         if (sep_pos == std::string::npos) {
-            for (const auto& total : totals) {
-                if (key.compare(1, total.size(), total) == 0)
-                    return true;
-            }
-
-            return false;
+            return std::any_of(totals.begin(), totals.end(),
+                               [&key](const auto& total)
+                               {
+                                   return key.compare(1, total.size(), total) == 0;
+                               });
         } else
             return is_total(key.substr(0,sep_pos));
     }
@@ -118,8 +161,10 @@ namespace {
             return {};
 
         std::vector<std::string> l;
-        for (const auto& pair : var1_iter->second)
-            l.push_back(pair.first);
+        std::transform(var1_iter->second.begin(), var1_iter->second.end(),
+                       std::back_inserter(l),
+                       [](const auto& pair) { return pair.first; });
+
         return l;
     }
 
@@ -164,14 +209,17 @@ namespace {
 namespace Opm
 {
 
-    SummaryState::SummaryState(time_point sim_start_arg)
-        : sim_start(sim_start_arg)
+    SummaryState::SummaryState(const time_point sim_start_arg,
+                               const double     udqUndefined)
+        : sim_start     { sim_start_arg }
+        , udq_undefined { udqUndefined }
     {
         this->update_elapsed(0);
     }
 
-    SummaryState::SummaryState(std::time_t sim_start_arg)
-        : SummaryState { TimeService::from_time_t(sim_start_arg) }
+    SummaryState::SummaryState(const std::time_t sim_start_arg)
+        : SummaryState { TimeService::from_time_t(sim_start_arg),
+                         std::numeric_limits<double>::lowest() }
     {}
 
     void SummaryState::set(const std::string& key, double value)
@@ -207,36 +255,49 @@ namespace Opm
 
     bool SummaryState::has(const std::string& key) const
     {
-        return this->values.find(key) != this->values.end();
+        return (this->values.find(key) != this->values.end()) || is_udq(key);
     }
 
-    bool SummaryState::has_well_var(const std::string& well, const std::string& var) const
+    bool SummaryState::has_well_var(const std::string& well,
+                                    const std::string& var) const
     {
-        return has_var(this->well_values, var, well);
+        return has_var(this->well_values, var, well)
+            || is_well_udq(var);
     }
 
     bool SummaryState::has_well_var(const std::string& var) const
     {
-        return this->well_values.count(var) != 0;
+        return (this->well_values.count(var) != 0) || is_well_udq(var);
     }
 
-    bool SummaryState::has_group_var(const std::string& group, const std::string& var) const
+    bool SummaryState::has_group_var(const std::string& group,
+                                     const std::string& var) const
     {
-        return has_var(this->group_values, var, group);
+        return has_var(this->group_values, var, group) || is_group_udq(var);
     }
 
     bool SummaryState::has_group_var(const std::string& var) const
     {
-        return this->group_values.count(var) != 0;
+        return (this->group_values.count(var) != 0) || is_group_udq(var);
     }
 
-    bool SummaryState::has_conn_var(const std::string& well, const std::string& var, std::size_t global_index) const
+    bool SummaryState::has_conn_var(const std::string& well,
+                                    const std::string& var,
+                                    const std::size_t  global_index) const
     {
-        if (!has_var(this->conn_values, var, well))
-            return false;
+        // Connection Values = [var][well][index] -> double
 
-        const auto& index_map = this->conn_values.at(var).at(well);
-        return (index_map.count(global_index) > 0);
+        auto varPos = this->conn_values.find(var);
+        if (varPos == this->conn_values.end()) {
+            return false;
+        }
+
+        auto wellPos = varPos->second.find(well);
+        if (wellPos == varPos->second.end()) {
+            return false;
+        }
+
+        return wellPos->second.find(global_index) != wellPos->second.end();
     }
 
     bool SummaryState::has_segment_var(const std::string& well,
@@ -255,7 +316,8 @@ namespace Opm
             return false;
         }
 
-        return wellPos->second.find(segment) != wellPos->second.end();
+        return (wellPos->second.find(segment) != wellPos->second.end())
+            || is_segment_udq(var);
     }
 
     bool SummaryState::has_region_var(const std::string& regSet,
@@ -277,37 +339,54 @@ namespace Opm
         return regSetPos->second.find(region) != regSetPos->second.end();
     }
 
-    void SummaryState::update(const std::string& key, double value) {
-        if (is_total(key))
-            this->values[key] += value;
-        else
-            this->values[key] = value;
+    void SummaryState::update(const std::string& key, double value)
+    {
+        auto& val_ref = this->values[key];
+
+        if (is_total(key)) {
+            val_ref += value;
+        }
+        else {
+            val_ref = value;
+        }
     }
 
-    void SummaryState::update_well_var(const std::string& well, const std::string& var, double value) {
-        std::string key = var + ":" + well;
+    void SummaryState::update_well_var(const std::string& well,
+                                       const std::string& var,
+                                       const double       value)
+    {
+        auto& val_ref  = this->values[fmt::format("{}:{}", var, well)];
+        auto& wval_ref = this->well_values[var][well];
+
         if (is_total(var)) {
-            this->values[key] += value;
-            this->well_values[var][well] += value;
-        } else {
-            this->values[key] = value;
-            this->well_values[var][well] = value;
+            val_ref  += value;
+            wval_ref += value;
         }
+        else {
+            val_ref = wval_ref = value;
+        }
+
         if (this->m_wells.count(well) == 0) {
             this->m_wells.insert(well);
             this->well_names.reset();
         }
     }
 
-    void SummaryState::update_group_var(const std::string& group, const std::string& var, double value) {
-        std::string key = var + ":" + group;
+    void SummaryState::update_group_var(const std::string& group,
+                                        const std::string& var,
+                                        const double       value)
+    {
+        auto& val_ref  = this->values[fmt::format("{}:{}", var, group)];
+        auto& gval_ref = this->group_values[var][group];
+
         if (is_total(var)) {
-            this->values[key] += value;
-            this->group_values[var][group] += value;
-        } else {
-            this->values[key] = value;
-            this->group_values[var][group] = value;
+            val_ref  += value;
+            gval_ref += value;
         }
+        else {
+            val_ref = gval_ref = value;
+        }
+
         if (this->m_groups.count(group) == 0) {
             this->m_groups.insert(group);
             this->group_names.reset();
@@ -319,31 +398,17 @@ namespace Opm
         this->elapsed += delta;
     }
 
-    void SummaryState::update_udq(const UDQSet& udq_set, double undefined_value)
+    void SummaryState::update_udq(const UDQSet& udq_set)
     {
         const auto var_type = udq_set.var_type();
         if (var_type == UDQVarType::WELL_VAR) {
-            const std::vector<std::string> wells = this->wells(); // Intentional copy
-            for (const auto& well : wells) {
-                if (! udq_set.has(well)) {
-                    this->update_well_var(well, udq_set.name(), undefined_value);
-                    continue;
-                }
-
-                const auto& udq_value = udq_set[well].value();
-                this->update_well_var(well, udq_set.name(), udq_value.value_or(undefined_value));
+            for (const auto& udq_value : udq_set) {
+                this->update_well_var(udq_value.wgname(), udq_set.name(), udq_value.value().value_or(this->udq_undefined));
             }
         }
         else if (var_type == UDQVarType::GROUP_VAR) {
-            const std::vector<std::string> groups = this->groups(); // Intentional copy
-            for (const auto& group : groups) {
-                if (! udq_set.has(group)) {
-                    this->update_group_var(group, udq_set.name(), undefined_value);
-                    continue;
-                }
-
-                const auto& udq_value = udq_set[group].value();
-                this->update_group_var(group, udq_set.name(), udq_value.value_or(undefined_value));
+            for (const auto& udq_value : udq_set) {
+                this->update_group_var(udq_value.wgname(), udq_set.name(), udq_value.value().value_or(this->udq_undefined));
             }
         }
         else if (var_type == UDQVarType::SEGMENT_VAR) {
@@ -351,24 +416,29 @@ namespace Opm
                 this->update_segment_var(udq_value.wgname(),
                                          udq_set.name(),
                                          udq_value.number(),
-                                         udq_value.value().value_or(undefined_value));
+                                         udq_value.value().value_or(this->udq_undefined));
             }
         }
         else {
             const auto& udq_var = udq_set[0].value();
-            this->update(udq_set.name(), udq_var.value_or(undefined_value));
+            this->update(udq_set.name(), udq_var.value_or(this->udq_undefined));
         }
     }
 
-    void SummaryState::update_conn_var(const std::string& well, const std::string& var, std::size_t global_index, double value)
+    void SummaryState::update_conn_var(const std::string& well,
+                                       const std::string& var,
+                                       const std::size_t  global_index,
+                                       const double       value)
     {
-        std::string key = var + ":" + well + ":" + std::to_string(global_index);
+        auto& val_ref  = this->values[fmt::format("{}:{}:{}", var, well, global_index)];
+        auto& cval_ref = this->conn_values[var][well][global_index];
+
         if (is_total(var)) {
-            this->values[key] += value;
-            this->conn_values[var][well][global_index] += value;
-        } else {
-            this->values[key] = value;
-            this->conn_values[var][well][global_index] = value;
+            val_ref  += value;
+            cval_ref += value;
+        }
+        else {
+            val_ref = cval_ref = value;
         }
     }
 
@@ -377,7 +447,7 @@ namespace Opm
                                           const std::size_t  segment,
                                           const double       value)
     {
-        auto& val_ref  = this->values[var + ':' + well + ':' + std::to_string(segment)];
+        auto& val_ref  = this->values[fmt::format("{}:{}:{}", var, well, segment)];
         auto& sval_ref = this->segment_values[var][well][segment];
 
         if (is_total(var)) {
@@ -410,20 +480,33 @@ namespace Opm
 
     double SummaryState::get(const std::string& key) const
     {
-        const auto iter = this->values.find(key);
-        if (iter == this->values.end())
-            throw std::out_of_range("No such key: " + key);
+        auto iter = this->values.find(key);
+        if (iter != this->values.end()) {
+            return iter->second;
+        }
 
-        return iter->second;
+        if (is_udq(key)) {
+            return this->udq_undefined;
+        }
+
+        throw std::out_of_range {
+            fmt::format("Summary vector {} is unknown", key)
+        };
     }
 
-    double SummaryState::get(const std::string& key, double default_value) const
+    double SummaryState::get(const std::string& key,
+                             const double       default_value) const
     {
-        const auto iter = this->values.find(key);
-        if (iter == this->values.end())
-            return default_value;
+        auto iter = this->values.find(key);
+        if (iter != this->values.end()) {
+            return iter->second;
+        }
 
-        return iter->second;
+        if (is_udq(key)) {
+            return this->udq_undefined;
+        }
+
+        return default_value;
     }
 
     double SummaryState::get_elapsed() const
@@ -431,59 +514,248 @@ namespace Opm
         return this->elapsed;
     }
 
-    double SummaryState::get_well_var(const std::string& well, const std::string& var) const
+    double SummaryState::get_well_var(const std::string& well,
+                                      const std::string& var) const
     {
-        return this->well_values.at(var).at(well);
+        const auto use_udq_fallback = is_well_udq(var);
+
+        auto varPos = this->well_values.find(var);
+        if (varPos == this->well_values.end()) {
+            if (! use_udq_fallback) {
+                throw std::invalid_argument {
+                    fmt::format("Summary vector {} does not "
+                                "exist at the well level", var)
+                };
+            }
+
+            return this->udq_undefined;
+        }
+
+        auto wellPos = varPos->second.find(well);
+        if (wellPos == varPos->second.end()) {
+            if (! use_udq_fallback) {
+                throw std::invalid_argument {
+                    fmt::format("Summary vector {} does not "
+                                "exist at the well level for well {}",
+                                var, well)
+                };
+            }
+
+            return this->udq_undefined;
+        }
+
+        return wellPos->second;
     }
 
-    double SummaryState::get_group_var(const std::string& group, const std::string& var) const
+    double SummaryState::get_group_var(const std::string& group,
+                                       const std::string& var) const
     {
-        return this->group_values.at(var).at(group);
+        const auto use_udq_fallback = is_group_udq(var);
+
+        auto varPos = this->group_values.find(var);
+        if (varPos == this->group_values.end()) {
+            if (! use_udq_fallback) {
+                throw std::invalid_argument {
+                    fmt::format("Summary vector {} does not "
+                                "exist at the group level", var)
+                };
+            }
+
+            return this->udq_undefined;
+        }
+
+        auto groupPos = varPos->second.find(group);
+        if (groupPos == varPos->second.end()) {
+            if (! use_udq_fallback) {
+                throw std::invalid_argument {
+                    fmt::format("Summary vector {} does not "
+                                "exist at the group level for group {}",
+                                var, group)
+                };
+            }
+
+            return this->udq_undefined;
+        }
+
+        return groupPos->second;
     }
 
-    double SummaryState::get_conn_var(const std::string& well, const std::string& var, std::size_t global_index) const
+    double SummaryState::get_conn_var(const std::string& well,
+                                      const std::string& var,
+                                      const std::size_t  global_index) const
     {
-        return this->conn_values.at(var).at(well).at(global_index);
+        auto varPos = this->conn_values.find(var);
+        if (varPos == this->conn_values.end()) {
+            throw std::invalid_argument {
+                fmt::format("Summary vector {} does not "
+                            "exist at the connection level", var)
+            };
+        }
+
+        auto wellPos = varPos->second.find(well);
+        if (wellPos == varPos->second.end()) {
+            throw std::invalid_argument {
+                fmt::format("Summary vector {} does not "
+                            "exist at the connection "
+                            "level for well {}",
+                            var, well)
+            };
+        }
+
+        auto connPos = wellPos->second.find(global_index);
+        if (connPos == wellPos->second.end()) {
+            throw std::invalid_argument {
+                fmt::format("Summary vector {} does not "
+                            "exist for connection {} "
+                            "in well {}",
+                            var, global_index, well)
+            };
+        }
+
+        return connPos->second;
     }
 
     double SummaryState::get_segment_var(const std::string& well,
                                          const std::string& var,
                                          const std::size_t  segment) const
     {
-        return this->segment_values.at(var).at(well).at(segment);
+        const auto use_udq_fallback = is_segment_udq(var);
+
+        auto varPos = this->segment_values.find(var);
+        if (varPos == this->segment_values.end()) {
+            if (! use_udq_fallback) {
+                throw std::invalid_argument {
+                    fmt::format("Summary vector {} does not "
+                                "exist at the segment level", var)
+                };
+            }
+
+            return this->udq_undefined;
+        }
+
+        auto wellPos = varPos->second.find(well);
+        if (wellPos == varPos->second.end()) {
+            if (! use_udq_fallback) {
+                throw std::invalid_argument {
+                    fmt::format("Summary vector {} does not "
+                                "exist at the segment "
+                                "level for well {}",
+                                var, well)
+                };
+            }
+
+            return this->udq_undefined;
+        }
+
+        auto segPos = wellPos->second.find(segment);
+        if (segPos == wellPos->second.end()) {
+            if (! use_udq_fallback) {
+                throw std::invalid_argument {
+                    fmt::format("Summary vector {} does not "
+                                "exist for segment {} "
+                                "in well {}",
+                                var, segment, well)
+                };
+            }
+
+            return this->udq_undefined;
+        }
+
+        return segPos->second;
     }
 
     double SummaryState::get_region_var(const std::string& regSet,
                                         const std::string& var,
                                         const std::size_t  region) const
     {
-        return this->region_values
-            .at(EclIO::SummaryNode::normalise_region_keyword(var))
-            .at(normalise_region_set_name(regSet))
-            .at(region);
+        auto varPos = this->region_values.find(EclIO::SummaryNode::normalise_region_keyword(var));
+        if (varPos == this->region_values.end()) {
+            throw std::invalid_argument {
+                fmt::format("Summary vector {} does not "
+                            "exist at the region level", var)
+            };
+        }
+
+        auto regSetPos = varPos->second.find(normalise_region_set_name(regSet));
+        if (regSetPos == varPos->second.end()) {
+            throw std::invalid_argument {
+                fmt::format("Summary vector {} does not "
+                            "exist at the region "
+                            "level for region set {}",
+                            var, regSet)
+            };
+        }
+
+        auto regionPos = regSetPos->second.find(region);
+        if (regionPos == regSetPos->second.end()) {
+            throw std::invalid_argument {
+                fmt::format("Summary vector {} does not "
+                            "exist for region {} "
+                            "in region set {}",
+                            var, region, regSet)
+            };
+        }
+
+        return regionPos->second;
     }
 
-    double SummaryState::get_well_var(const std::string& well, const std::string& var, double default_value) const
+    double SummaryState::get_well_var(const std::string& well,
+                                      const std::string& var,
+                                      const double       default_value) const
     {
-        if (this->has_well_var(well, var))
-            return this->get_well_var(well, var);
+        const auto fallback = is_well_udq(var)
+            ? this->udq_undefined
+            : default_value;
 
-        return default_value;
+        auto varPos = this->well_values.find(var);
+        if (varPos == this->well_values.end()) {
+            return fallback;
+        }
+
+        auto wellPos = varPos->second.find(well);
+        return (wellPos == varPos->second.end())
+            ? fallback
+            : wellPos->second;
     }
 
-    double SummaryState::get_group_var(const std::string& group, const std::string& var, double default_value) const
+    double SummaryState::get_group_var(const std::string& group,
+                                       const std::string& var,
+                                       const double       default_value) const
     {
-        if (this->has_group_var(group, var))
-            return this->get_group_var(group, var);
+        const auto fallback = is_group_udq(var)
+            ? this->udq_undefined
+            : default_value;
 
-        return default_value;
+        auto varPos = this->group_values.find(var);
+        if (varPos == this->group_values.end()) {
+            return fallback;
+        }
+
+        auto groupPos = varPos->second.find(group);
+        return (groupPos == varPos->second.end())
+            ? fallback
+            : groupPos->second;
     }
 
-    double SummaryState::get_conn_var(const std::string& well, const std::string& var, std::size_t global_index, double default_value) const
+    double SummaryState::get_conn_var(const std::string& well,
+                                      const std::string& var,
+                                      const std::size_t  global_index,
+                                      const double       default_value) const
     {
-        if (this->has_conn_var(well, var, global_index))
-            return this->get_conn_var(well, var, global_index);
-        return default_value;
+        auto varPos = this->conn_values.find(var);
+        if (varPos == this->conn_values.end()) {
+            return default_value;
+        }
+
+        auto wellPos = varPos->second.find(well);
+        if (wellPos == varPos->second.end()) {
+            return default_value;
+        }
+
+        auto connPos = wellPos->second.find(global_index);
+        return (connPos == wellPos->second.end())
+            ? default_value
+            : connPos->second;
     }
 
     double SummaryState::get_segment_var(const std::string& well,
@@ -509,8 +781,8 @@ namespace Opm
 
     const std::vector<std::string>& SummaryState::wells() const
     {
-        if (!this->well_names) {
-            this->well_names = std::vector<std::string>(this->m_wells.begin(), this->m_wells.end());
+        if (!this->well_names.has_value()) {
+            this->well_names.emplace(this->m_wells.begin(), this->m_wells.end());
         }
 
         return *this->well_names;
@@ -523,8 +795,8 @@ namespace Opm
 
     const std::vector<std::string>& SummaryState::groups() const
     {
-        if (!this->group_names) {
-            this->group_names = std::vector<std::string>(this->m_groups.begin(), this->m_groups.end());
+        if (!this->group_names.has_value()) {
+            this->group_names.emplace(this->m_groups.begin(), this->m_groups.end());
         }
 
         return *this->group_names;
@@ -585,6 +857,7 @@ namespace Opm
     bool SummaryState::operator==(const SummaryState& other) const
     {
         return (this->sim_start == other.sim_start)
+            && (this->udq_undefined == other.udq_undefined)
             && (this->elapsed == other.elapsed)
             && (this->values == other.values)
             && (this->well_values == other.well_values)
@@ -601,7 +874,7 @@ namespace Opm
 
     SummaryState SummaryState::serializationTestObject()
     {
-        auto st = SummaryState{TimeService::from_time_t(101)};
+        auto st = SummaryState{TimeService::from_time_t(101), 1.234};
 
         st.elapsed = 1.0;
         st.values = {{"test1", 2.0}};
