@@ -22,6 +22,7 @@
 #include <opm/io/eclipse/rst/state.hpp>
 
 #include <opm/common/OpmLog/KeywordLocation.hpp>
+#include <opm/common/OpmLog/OpmLog.hpp>
 #include <opm/common/utility/OpmInputError.hpp>
 
 #include <opm/input/eclipse/Schedule/MSW/SegmentMatcher.hpp>
@@ -274,17 +275,17 @@ namespace Opm {
 
     void UDQConfig::add_unit(const std::string& keyword, const std::string& quoted_unit)
     {
-        const auto unit = strip_quotes(quoted_unit);
+        const auto Unit = strip_quotes(quoted_unit);
         const auto pair_ptr = this->units.find(keyword);
         if (pair_ptr != this->units.end()) {
-            if (pair_ptr->second != unit) {
+            if (pair_ptr->second != Unit) {
                 throw std::invalid_argument("Illegal to change unit of UDQ keyword runtime");
             }
 
             return;
         }
 
-        this->units[keyword] = unit;
+        this->units[keyword] = Unit;
     }
 
     void UDQConfig::add_update(const std::string&              keyword,
@@ -300,6 +301,11 @@ namespace Opm {
         }
 
         if (this->m_definitions.count(keyword) == 0) {
+            if (this->m_assignments.count(keyword) > 0) {
+                OpmLog::warning(fmt::format("UDQ variable {} is constant, so UPDATE will have no effect.", keyword));
+                return;
+            }
+
             throw OpmInputError {
                 fmt::format("UDQ variable: {} must be defined before you can use UPDATE", keyword),
                 location
@@ -308,8 +314,8 @@ namespace Opm {
 
         const auto update_status = UDQ::updateType(data[0]);
 
-        auto& define = this->m_definitions[keyword];
-        define.update_status(update_status, report_step);
+        auto& def = this->m_definitions[keyword];
+        def.update_status(update_status, report_step);
     }
 
     void UDQConfig::add_record(SegmentMatcherFactory  create_segment_matcher,
@@ -330,7 +336,8 @@ namespace Opm {
             this->add_unit(quantity, data.front());
         }
         else if (action == UDQAction::ASSIGN) {
-            const auto selector = std::vector<std::string>(data.begin(), data.end() - 1);
+            auto selector = std::vector<std::string>(data.begin(), data.end() - 1);
+            std::transform(selector.cbegin(), selector.cend(), selector.begin(), strip_quotes);
             const auto value = std::stod(data.back());
             this->add_assign(quantity,
                              std::move(create_segment_matcher),
@@ -413,10 +420,10 @@ namespace Opm {
                 ? this->unit(key) : std::string{};
 
             if (index.action == UDQAction::DEFINE) {
-                res.push_back(UDQInput(index, this->m_definitions.at(key), u));
+                res.emplace_back(index, &this->m_definitions.at(key), u);
             }
             else if (index.action == UDQAction::ASSIGN) {
-                res.push_back(UDQInput(index, this->m_assignments.at(key), u));
+                res.emplace_back(index, &this->m_assignments.at(key), u);
             }
         }
 
@@ -439,8 +446,8 @@ namespace Opm {
     {
         std::vector<UDQAssign> ret;
 
-        for (const auto& [key, input] : this->input_index) {
-            if (input.action == UDQAction::ASSIGN) {
+        for (const auto& [key, Input] : this->input_index) {
+            if (Input.action == UDQAction::ASSIGN) {
                 ret.push_back(this->m_assignments.at(key));
             }
         }
@@ -487,7 +494,7 @@ namespace Opm {
 
     UDQInput UDQConfig::operator[](const std::string& keyword) const
     {
-        const auto index_iter = this->input_index.find(keyword);
+        auto index_iter = this->input_index.find(keyword);
         if (index_iter == this->input_index.end()) {
             throw std::invalid_argument("Keyword: '" + keyword +
                                         "' not recognized as ASSIGN/DEFINE UDQ");
@@ -497,11 +504,11 @@ namespace Opm {
             ? this->unit(keyword) : std::string{};
 
         if (index_iter->second.action == UDQAction::ASSIGN) {
-            return UDQInput(this->input_index.at(keyword), this->m_assignments.at(keyword), u);
+            return { index_iter->second, &this->m_assignments.at(keyword), u };
         }
 
         if (index_iter->second.action == UDQAction::DEFINE) {
-            return UDQInput(this->input_index.at(keyword), this->m_definitions.at(keyword), u);
+            return { index_iter->second, &this->m_definitions.at(keyword), u };
         }
 
         throw std::logic_error("Internal error - should not be here");
@@ -524,11 +531,11 @@ namespace Opm {
             ? this->unit(keyword) : std::string{};
 
         if (index.action == UDQAction::ASSIGN) {
-            return UDQInput(index, this->m_assignments.at(keyword), u);
+            return { index, &this->m_assignments.at(keyword), u };
         }
 
         if (index.action == UDQAction::DEFINE) {
-            return UDQInput(index, this->m_definitions.at(keyword), u);
+            return { index, &this->m_definitions.at(keyword), u };
         }
 
         throw std::logic_error("Internal error - should not be here");
@@ -552,6 +559,7 @@ namespace Opm {
             && (this->m_assignments == data.m_assignments)
             && (this->m_tables == data.m_tables)
             && (this->units == data.units)
+            && (this->define_order == data.define_order)
             && (this->input_index == data.input_index)
             && (this->type_count == data.type_count)
             && (this->pending_assignments_ == data.pending_assignments_)
@@ -601,7 +609,7 @@ namespace Opm {
     }
 
     void UDQConfig::eval_define(const std::size_t report_step,
-                                UDQState&         udq_state,
+                                const UDQState&   udq_state,
                                 UDQContext&       context) const
     {
         auto var_type_bit = [](const UDQVarType var_type)
