@@ -155,6 +155,7 @@ namespace Opm {
             serializer(m_assignments);
             serializer(m_tables);
             serializer(units);
+            serializer(define_order);
             serializer(input_index);
             serializer(type_count);
             serializer(pending_assignments_);
@@ -196,7 +197,7 @@ namespace Opm {
                          UDQContext&     context) const;
 
         void eval_define(std::size_t report_step,
-                         UDQState&   udq_state,
+                         const UDQState& udq_state,
                          UDQContext& context) const;
 
         void add_named_assign(const std::string&              quantity,
