@@ -55,6 +55,7 @@
 #include "../MSW/Compsegs.hpp"
 #include "../eval_uda.hpp"
 
+#include <algorithm>
 #include <cassert>
 #include <cmath>
 #include <cstddef>
@@ -294,7 +295,7 @@ Well::Well(const RestartIO::RstWell& rst_well,
     allow_cross_flow(rst_well.allow_xflow == 1),
     automatic_shutin(def_automatic_shutin),
     pvt_table(rst_well.pvt_table),
-    unit_system(unit_system_arg),
+    unit_system(&unit_system_arg),
     udq_undefined(udq_undefined_arg),
     wtype(rst_well.wtype),
     guide_rate(guideRate(rst_well)),
@@ -314,11 +315,11 @@ Well::Well(const RestartIO::RstWell& rst_well,
     wvfpexp(explicitTHPOptions(rst_well)),
     wdfac(std::make_shared<WDFAC>(rst_well)),
     status(status_from_int(rst_well.well_status)),
-    well_temperature(Metric::TemperatureOffset + ParserKeywords::STCOND::TEMPERATURE::defaultValue),
+    well_inj_temperature(std::nullopt),
     well_inj_mult(std::nullopt)
 {
     if (this->wtype.producer()) {
-        auto p = std::make_shared<WellProductionProperties>(this->unit_system, wname);
+        auto p = std::make_shared<WellProductionProperties>(*this->unit_system, wname);
 
         p->whistctl_cmode = (rst_whistctl_cmode > 0)
             ? producer_cmode_from_int(rst_whistctl_cmode)
@@ -363,7 +364,7 @@ Well::Well(const RestartIO::RstWell& rst_well,
         p->addProductionControl(Well::ProducerCMode::BHP);
         if (! p->predictionMode) {
             p->BHPTarget.update(0.0);
-            p->setBHPLimit(rst_well.bhp_target_double);
+            p->setBHPLimit(unit_system->to_si(UnitSystem::measure::pressure, rst_well.bhp_target_float));
             p->controlMode = producer_cmode_from_int(rst_well.hist_requested_control);
         }
         else if (this->isAvailableForGroupControl())
@@ -372,7 +373,7 @@ Well::Well(const RestartIO::RstWell& rst_well,
         this->updateProduction(std::move(p));
     }
     else {
-        auto i = std::make_shared<WellInjectionProperties>(this->unit_system, wname);
+        auto i = std::make_shared<WellInjectionProperties>(*this->unit_system, wname);
         i->VFPTableNumber = rst_well.vfp_table;
         i->predictionMode = this->prediction_mode;
 
@@ -441,9 +442,9 @@ Well::Well(const RestartIO::RstWell& rst_well,
         if (!rst_well.tracer_concentration_injection.empty()) {
             auto tracer = std::make_shared<WellTracerProperties>(this->getTracerProperties());
             for (std::size_t tracer_index = 0; tracer_index < tracer_config.size(); tracer_index++) {
-                const auto& name = tracer_config[tracer_index].name;
+                const auto& tname = tracer_config[tracer_index].name;
                 const auto concentration = rst_well.tracer_concentration_injection[tracer_index];
-                tracer->setConcentration(name, concentration);
+                tracer->setConcentration(tname, concentration);
             }
             this->updateTracer(tracer);
         }
@@ -481,7 +482,7 @@ Well::Well(const std::string& wname_arg,
     automatic_shutin(auto_shutin),
     pvt_table(pvt_table_),
     gas_inflow(inflow_eq),
-    unit_system(unit_system_arg),
+    unit_system(&unit_system_arg),
     udq_undefined(udq_undefined_arg),
     wtype(wtype_arg),
     guide_rate({true, -1, Well::GuideRateTarget::UNDEFINED,ParserKeywords::WGRUPCON::SCALING_FACTOR::defaultValue}),
@@ -495,16 +496,16 @@ Well::Well(const std::string& wname_arg,
     brine_properties(std::make_shared<WellBrineProperties>()),
     tracer_properties(std::make_shared<WellTracerProperties>()),
     connections(std::make_shared<WellConnections>(ordering_arg, headI, headJ)),
-    production(std::make_shared<WellProductionProperties>(unit_system, wname)),
-    injection(std::make_shared<WellInjectionProperties>(unit_system, wname)),
+    production(std::make_shared<WellProductionProperties>(*unit_system, wname)),
+    injection(std::make_shared<WellInjectionProperties>(*unit_system, wname)),
     wvfpdp(std::make_shared<WVFPDP>()),
     wvfpexp(std::make_shared<WVFPEXP>()),
     wdfac(std::make_shared<WDFAC>()),
     status(Status::SHUT),
-    well_temperature(Metric::TemperatureOffset + ParserKeywords::STCOND::TEMPERATURE::defaultValue),
+    well_inj_temperature(std::nullopt),
     well_inj_mult(std::nullopt)
 {
-    auto p = std::make_shared<WellProductionProperties>(this->unit_system, this->wname);
+    auto p = std::make_shared<WellProductionProperties>(*this->unit_system, this->wname);
     p->whistctl_cmode = whistctl_cmode;
     this->updateProduction(p);
 }
@@ -519,7 +520,6 @@ Well Well::serializationTestObject()
     result.headI = 3;
     result.headJ = 4;
     result.ref_depth = 5;
-    result.unit_system = UnitSystem::serializationTestObject();
     result.udq_undefined = 6.0;
     result.status = Status::AUTO;
     result.drainage_radius = 7.0;
@@ -543,10 +543,11 @@ Well Well::serializationTestObject()
     result.production = std::make_shared<Well::WellProductionProperties>(Well::WellProductionProperties::serializationTestObject());
     result.injection = std::make_shared<Well::WellInjectionProperties>(Well::WellInjectionProperties::serializationTestObject());
     result.segments = std::make_shared<WellSegments>(WellSegments::serializationTestObject());
+    result.wvfpdp = std::make_shared<WVFPDP>(WVFPDP::serializationTestObject());
     result.wvfpexp = std::make_shared<WVFPEXP>(WVFPEXP::serializationTestObject());
     result.wdfac = std::make_shared<WDFAC>(WDFAC::serializationTestObject());
     result.m_pavg = PAvg();
-    result.well_temperature = 10.0;
+    result.well_inj_temperature = 10.0;
     result.well_inj_mult = InjMult::serializationTestObject();
     result.m_filter_concentration = UDAValue::serializationTestObject();
 
@@ -1130,11 +1131,11 @@ double Well::convertDeckPI(double deckPI) const {
     //      not provide that enumerator.
     switch (this->getPreferredPhase()) {
     case Phase::GAS:
-        return this->unit_system.to_si(M::gas_productivity_index, deckPI);
+        return this->unit_system->to_si(M::gas_productivity_index, deckPI);
 
     case Phase::OIL:
     case Phase::WATER:
-        return this->unit_system.to_si(M::liquid_productivity_index, deckPI);
+        return this->unit_system->to_si(M::liquid_productivity_index, deckPI);
 
     default:
         throw std::invalid_argument {
@@ -1257,12 +1258,13 @@ std::map<int, std::vector<Connection>> Well::getCompletions() const {
     return completions;
 }
 
-bool Well::hasCompletion(int completion) const {
-    for (const auto& conn : *this->connections) {
-        if (conn.complnum() == completion)
-            return true;
-    }
-    return false;
+bool Well::hasCompletion(int completion) const
+{
+    return std::any_of(this->connections->begin(), this->connections->end(),
+                       [completion](const auto& conn)
+                       {
+                           return conn.complnum() == completion;
+                       });
 }
 
 
@@ -1694,7 +1696,6 @@ bool Well::wellNameInWellNamePattern(const std::string& wellName, const std::str
 Well::ProductionControls Well::productionControls(const SummaryState& st) const {
     if (this->isProducer()) {
         auto controls = this->production->controls(st, this->udq_undefined);
-        controls.prediction_mode = this->predictionMode();
         return controls;
     } else
         throw std::logic_error("Trying to get production data from an injector");
@@ -1702,8 +1703,7 @@ Well::ProductionControls Well::productionControls(const SummaryState& st) const
 
 Well::InjectionControls Well::injectionControls(const SummaryState& st) const {
     if (!this->isProducer()) {
-        auto controls = this->injection->controls(this->unit_system, st, this->udq_undefined);
-        controls.prediction_mode = this->predictionMode();
+        auto controls = this->injection->controls(*this->unit_system, st, this->udq_undefined);
         return controls;
     } else
         throw std::logic_error("Trying to get injection data from a producer");
@@ -1734,19 +1734,25 @@ int Well::vfp_table_number() const {
         return this->injection->VFPTableNumber;
 }
 
-double Well::temperature() const {
-    if (!this->wtype.producer())
-        return this->well_temperature;
+double Well::inj_temperature() const {
+    if (this->wtype.injector() && this->well_inj_temperature)
+        return *this->well_inj_temperature;
 
-    throw std::runtime_error("Can only ask for temperature in an injector");
+    throw std::runtime_error("Can only ask for well temperature for" 
+                    "injectors with non-default temperature.");
 }
-void Well::setWellTemperature(const double temp) {
-    this->well_temperature = temp;
+bool Well::hasInjTemperature() const {
+    return this->well_inj_temperature.has_value(); 
+}
+void Well::setWellInjTemperature(const double temp) {
+    this->well_inj_temperature = temp;
 }
 
 bool Well::cmp_structure(const Well& other) const {
     if ((this->segments && !other.segments) ||
-        (!this->segments && other.segments))
+        (!this->segments && other.segments) ||
+        (this->unit_system && !other.unit_system) ||
+        (!this->unit_system && other.unit_system))
     {
         return false;
     }
@@ -1755,6 +1761,10 @@ bool Well::cmp_structure(const Well& other) const {
         return false;
     }
 
+    if (this->unit_system && *this->unit_system != *other.unit_system) {
+        return false;
+    }
+
     return (this->name() == other.name())
         && (this->groupName() == other.groupName())
         && (this->firstTimeStep() == other.firstTimeStep())
@@ -1763,38 +1773,43 @@ bool Well::cmp_structure(const Well& other) const {
         && (this->getHeadJ() == other.getHeadJ())
         && (this->hasRefDepth() == other.hasRefDepth())
         && (!this->hasRefDepth() || (this->getRefDepth() == other.getRefDepth()))
-        && (this->getPreferredPhase() == other.getPreferredPhase())
-        && (this->unit_system == other.unit_system)
-        && (this->udq_undefined == other.udq_undefined)
-        && (this->getConnections() == other.getConnections())
         && (this->getDrainageRadius() == other.getDrainageRadius())
         && (this->getAllowCrossFlow() == other.getAllowCrossFlow())
         && (this->getAutomaticShutIn() == other.getAutomaticShutIn())
+        && (this->udq_undefined == other.udq_undefined)
+        && (this->getPreferredPhase() == other.getPreferredPhase()) // wellType()
         && (this->getEfficiencyFactor() == other.getEfficiencyFactor())
+        && (this->getConnections() == other.getConnections())
         ;
 }
 
 
 bool Well::operator==(const Well& data) const {
     return this->cmp_structure(data)
-        && (this->getSolventFraction() == data.getSolventFraction())
-        && (this->getEconLimits() == data.getEconLimits())
-        && (this->isProducer() == data.isProducer())
-        && (this->getFoamProperties() == data.getFoamProperties())
-        && (this->getStatus() == data.getStatus())
+        && (this->wpave_ref_depth == data.wpave_ref_depth)
+        && (this->gas_inflow == data.gas_inflow)
+        && (this->pvt_table == data.pvt_table)
+        && (this->isProducer() == data.isProducer()) // wellType()
         && (this->guide_rate == data.guide_rate)
-        && (this->solvent_fraction == data.solvent_fraction)
         && (this->hasProduced() == data.hasProduced())
         && (this->hasInjected() == data.hasInjected())
         && (this->predictionMode() == data.predictionMode())
+        && (this->getSolventFraction() == data.getSolventFraction())
         && (this->derive_refdepth_from_conns_ == data.derive_refdepth_from_conns_)
+        && (this->getEconLimits() == data.getEconLimits())
+        && (this->getFoamProperties() == data.getFoamProperties())
+        && (this->getPolymerProperties() == data.getPolymerProperties())
+        && (this->getMICPProperties() == data.getMICPProperties())
+        && (this->getBrineProperties() == data.getBrineProperties())
         && (this->getTracerProperties() == data.getTracerProperties())
+        && (this->getProductionProperties() == data.getProductionProperties())
+        && (this->getInjectionProperties() == data.getInjectionProperties())
+        && (this->getWVFPDP() == data.getWVFPDP())
         && (this->getWVFPEXP() == data.getWVFPEXP())
         && (this->getWDFAC() == data.getWDFAC())
-        && (this->getProductionProperties() == data.getProductionProperties())
+        && (this->getStatus() == data.getStatus())
         && (this->m_pavg == data.m_pavg)
-        && (this->getInjectionProperties() == data.getInjectionProperties())
-        && (this->well_temperature == data.well_temperature)
+        && (this->well_inj_temperature == data.well_inj_temperature)
         && (this->inj_mult_mode == data.inj_mult_mode)
         && (this->well_inj_mult == data.well_inj_mult)
         && (this->m_filter_concentration == data.m_filter_concentration)
