@@ -184,11 +184,13 @@ public:
 
         //! \brief Handle a WCONINJH keyword.
         //! \param record The deck record to use
+        //! \param vfp_table_nr The vfp table number
         //! \param bhp_def The default BHP limit in SI units
         //! \param is_producer True if well is a producer
         //! \param well_name Name of well
         //! \param loc Location of keyword for logging purpose
         void handleWCONINJH(const DeckRecord& record,
+                            const int vfp_table_nr,
                             const double bhp_def,
                             const bool is_producer,
                             const std::string& well_name,
@@ -301,12 +303,14 @@ public:
 
         //! \brief Handle WCONPROD keyword.
         //! \param alq_type ALQ type
+        //! \param vfp_table_nr The vfp table number
         //! \param bhp_def Default BHP target in SI units
         //! \param unit_system Unit system to use
         //! \param well Well name
         //! \param record Deck record to use
         //! \param location Location of keyword for logging purpose
         void handleWCONPROD(const std::optional<VFPProdTable::ALQ_TYPE>& alq_type,
+                            const int vfp_table_nr,
                             const double bhp_def,
                             const UnitSystem& unit_system,
                             const std::string& well,
@@ -315,10 +319,12 @@ public:
 
         //! \brief Handle WCONHIST keyword.
         //! \param alq_type ALQ type
+        //! \param vfp_table_nr The vfp table number
         //! \param bhp_def Default BHP limit in SI units
         //! \param unit_system Unit system to use
         //! \param record Deck record to use
         void handleWCONHIST(const std::optional<VFPProdTable::ALQ_TYPE>& alq_type,
+                            const int vfp_table_nr,
                             const double bhp_def,
                             const UnitSystem& unit_system,
                             const DeckRecord& record);
@@ -362,7 +368,7 @@ public:
         void init_rates( const DeckRecord& record );
 
         void init_history(const DeckRecord& record);
-        void init_vfp(const std::optional<VFPProdTable::ALQ_TYPE>& alq_type, const UnitSystem& unit_system, const DeckRecord& record);
+        void init_vfp(const std::optional<VFPProdTable::ALQ_TYPE>& alq_type, const int vfp_table_nr, const UnitSystem& unit_system, const DeckRecord& record);
 
         WellProductionProperties(const DeckRecord& record);
 
@@ -377,7 +383,6 @@ public:
     static int eclipseControlMode(const Well&         well,
                                   const SummaryState& st);
 
-
     Well() = default;
     Well(const std::string& wname,
          const std::string& gname,
@@ -553,8 +558,9 @@ public:
     GasInflowEquation gas_inflow_equation() const;
     bool segmented_density_calculation() const { return true; }
     double alq_value(const SummaryState& st) const;
-    double temperature() const;
-    void setWellTemperature(const double temp);
+    double inj_temperature() const;
+    bool hasInjTemperature() const;
+    void setWellInjTemperature(const double temp);
     bool hasInjected( ) const;
     bool hasProduced( ) const;
     bool updateHasInjected( );
@@ -568,6 +574,9 @@ public:
                                    std::vector<bool>& scalingApplicable);
     const PAvg& pavg() const;
 
+    //! \brief Used by schedule deserialization.
+    void updateUnitSystem(const UnitSystem* usys) { unit_system = usys; }
+
     template<class Serializer>
     void serializeOp(Serializer& serializer)
     {
@@ -579,7 +588,6 @@ public:
         serializer(headJ);
         serializer(ref_depth);
         serializer(wpave_ref_depth);
-        serializer(unit_system);
         serializer(udq_undefined);
         serializer(status);
         serializer(drainage_radius);
@@ -609,7 +617,7 @@ public:
         serializer(wdfac);
         serializer(wvfpexp);
         serializer(m_pavg);
-        serializer(well_temperature);
+        serializer(well_inj_temperature);
         serializer(inj_mult_mode);
         serializer(well_inj_mult);
         serializer(m_filter_concentration);
@@ -621,50 +629,50 @@ private:
 
     GuideRateTarget preferredPhaseAsGuideRatePhase() const;
 
-    std::string wname;
-    std::string group_name;
-    std::size_t init_step;
-    std::size_t insert_index;
-    int headI;
-    int headJ;
-    std::optional<double> ref_depth;
-    std::optional<double> wpave_ref_depth;
-    double drainage_radius;
-    bool allow_cross_flow;
-    bool automatic_shutin;
-    int pvt_table;
+    std::string wname{};
+    std::string group_name{};
+    std::size_t init_step{};
+    std::size_t insert_index{};
+    int headI{};
+    int headJ{};
+    std::optional<double> ref_depth{};
+    std::optional<double> wpave_ref_depth{};
+    double drainage_radius{};
+    bool allow_cross_flow{false};
+    bool automatic_shutin{false};
+    int pvt_table{};
     GasInflowEquation gas_inflow = GasInflowEquation::STD;  // Will NOT be loaded/assigned from restart file
-    UnitSystem unit_system;
-    double udq_undefined;
-    WellType wtype;
-    WellGuideRate guide_rate;
-    double efficiency_factor;
-    double solvent_fraction;
+    const UnitSystem* unit_system{nullptr};
+    double udq_undefined{};
+    WellType wtype{};
+    WellGuideRate guide_rate{};
+    double efficiency_factor{};
+    double solvent_fraction{};
     bool has_produced = false;
     bool has_injected = false;
     bool prediction_mode = true;
     bool derive_refdepth_from_conns_ { true };
 
-    std::shared_ptr<WellEconProductionLimits> econ_limits;
-    std::shared_ptr<WellFoamProperties> foam_properties;
-    std::shared_ptr<WellPolymerProperties> polymer_properties;
-    std::shared_ptr<WellMICPProperties> micp_properties;
-    std::shared_ptr<WellBrineProperties> brine_properties;
-    std::shared_ptr<WellTracerProperties> tracer_properties;
-    std::shared_ptr<WellConnections> connections; // The WellConnections object cannot be const because of WELPI and the filterConnections method
-    std::shared_ptr<WellProductionProperties> production;
-    std::shared_ptr<WellInjectionProperties> injection;
-    std::shared_ptr<WellSegments> segments;
-    std::shared_ptr<WVFPDP> wvfpdp;
-    std::shared_ptr<WVFPEXP> wvfpexp;
-    std::shared_ptr<WDFAC> wdfac;
+    std::shared_ptr<WellEconProductionLimits> econ_limits{};
+    std::shared_ptr<WellFoamProperties> foam_properties{};
+    std::shared_ptr<WellPolymerProperties> polymer_properties{};
+    std::shared_ptr<WellMICPProperties> micp_properties{};
+    std::shared_ptr<WellBrineProperties> brine_properties{};
+    std::shared_ptr<WellTracerProperties> tracer_properties{};
+    std::shared_ptr<WellConnections> connections{}; // The WellConnections object cannot be const because of WELPI and the filterConnections method
+    std::shared_ptr<WellProductionProperties> production{};
+    std::shared_ptr<WellInjectionProperties> injection{};
+    std::shared_ptr<WellSegments> segments{};
+    std::shared_ptr<WVFPDP> wvfpdp{};
+    std::shared_ptr<WVFPEXP> wvfpexp{};
+    std::shared_ptr<WDFAC> wdfac{};
 
-    Status status;
-    PAvg m_pavg;
-    double well_temperature;
+    Status status{Status::AUTO};
+    PAvg m_pavg{};
+    std::optional<double> well_inj_temperature{};
     InjMultMode inj_mult_mode = InjMultMode::NONE;
-    std::optional<InjMult> well_inj_mult;
-    UDAValue m_filter_concentration;
+    std::optional<InjMult> well_inj_mult{};
+    UDAValue m_filter_concentration{};
 };
 
 std::ostream& operator<<( std::ostream&, const Well::WellInjectionProperties& );
