@@ -103,7 +103,7 @@ void handleWCONHIST(HandlerContext& handlerContext)
             bool update_well = false;
 
             auto table_nr = record.getItem("VFP_TABLE").get< int >(0);
-            if (record.getItem("VFP_TABLE").defaultApplied(0)) {
+            if (record.getItem("VFP_TABLE").defaultApplied(0)) { // Default 1* use the privious set vfp table
                 table_nr = properties->VFPTableNumber;
             }
 
@@ -124,7 +124,14 @@ void handleWCONHIST(HandlerContext& handlerContext)
                                                             ParserKeywords::FBHPDEF::TARGET_BHP::defaultValue);
             }
 
+            // Injectors at a restart time will not have any WellProductionProperties with the
+            // proper whistctl_cmode, so this needs to be set before the call to handleWCONHIST
+            if (switching_from_injector) {
+                properties->whistctl_cmode = handlerContext.state().whistctl();
+            }
+
             properties->handleWCONHIST(alq_type,
+                                       table_nr,
                                        default_bhp,
                                        handlerContext.static_schedule().m_unit_system, record);
 
@@ -198,9 +205,6 @@ void handleWCONINJE(HandlerContext& handlerContext)
             }
 
             auto table_nr = record.getItem("VFP_TABLE").get< int >(0);
-            if (record.getItem("VFP_TABLE").defaultApplied(0)) {
-                table_nr = injection->VFPTableNumber;
-            }
 
             if (table_nr != 0) {
                 const auto& vfpinj = handlerContext.state().vfpinj;
@@ -277,10 +281,9 @@ void handleWCONINJH(HandlerContext& handlerContext)
             }
 
             auto table_nr = record.getItem("VFP_TABLE").get< int >(0);
-            if (record.getItem("VFP_TABLE").defaultApplied(0)) {
+            if (record.getItem("VFP_TABLE").defaultApplied(0)) { // Default 1* use the privious set vfp table
                 table_nr = injection->VFPTableNumber;
             }
-
             if (table_nr != 0) {
                 const auto& vfpinj = handlerContext.state().vfpinj;
                 if (!vfpinj.has(table_nr)) {
@@ -289,7 +292,7 @@ void handleWCONINJH(HandlerContext& handlerContext)
                 }
             }
 
-            injection->handleWCONINJH(record, default_bhp_limit,
+            injection->handleWCONINJH(record, table_nr, default_bhp_limit,
                                       well2.isProducer(), well_name,
                                       handlerContext.keyword.location());
 
@@ -349,10 +352,6 @@ void handleWCONPROD(HandlerContext& handlerContext)
             }
 
             auto table_nr = record.getItem("VFP_TABLE").get< int >(0);
-            if (record.getItem("VFP_TABLE").defaultApplied(0)) {
-                table_nr = properties->VFPTableNumber;
-            }
-
             if (table_nr != 0) {
                 const auto& vfpprod = handlerContext.state().vfpprod;
                 if (vfpprod.has(table_nr)) {
@@ -371,7 +370,7 @@ void handleWCONPROD(HandlerContext& handlerContext)
                                                                    ParserKeywords::WCONPROD::BHP::defaultValue.get<double>());
             }
 
-            properties->handleWCONPROD(alq_type, default_bhp_target,
+            properties->handleWCONPROD(alq_type, table_nr, default_bhp_target,
                                        handlerContext.static_schedule().m_unit_system,
                                        well_name, record, handlerContext.keyword.location());
 
@@ -558,9 +557,7 @@ void handleWELSPECS(HandlerContext& handlerContext)
                 fieldWells.push_back(wellName);
             }
             else {
-                for (const auto& existingWell : existingWells) {
-                    fieldWells.push_back(existingWell);
-                }
+                fieldWells.insert(fieldWells.end(), existingWells.begin(), existingWells.end());
             }
         }
 
@@ -712,15 +709,12 @@ void handleWHISTCTL(HandlerContext& handlerContext)
     const std::string& cmodeString = record.getItem("CMODE").getTrimmedString(0);
     const auto controlMode = WellProducerCModeFromString(cmodeString);
 
-    if (controlMode != Well::ProducerCMode::NONE) {
-        if (!Well::WellProductionProperties::effectiveHistoryProductionControl(controlMode) ) {
-            std::string msg = "The WHISTCTL keyword specifies an un-supported control mode " + cmodeString
-                + ", which makes WHISTCTL keyword not affect the simulation at all";
-            OpmLog::warning(msg);
-        } else {
-            handlerContext.state().update_whistctl( controlMode );
-        }
+    if (controlMode != Well::ProducerCMode::NONE && !Well::WellProductionProperties::effectiveHistoryProductionControl(controlMode) ) {
+        std::string msg = "The WHISTCTL keyword specifies an un-supported control mode " + cmodeString
+            + ", which makes WHISTCTL keyword not affect the simulation at all";
+        OpmLog::warning(msg);
     }
+    handlerContext.state().update_whistctl( controlMode );
 
     const std::string bhp_terminate = record.getItem("BPH_TERMINATE").getTrimmedString(0);
     if (bhp_terminate == "YES") {
