@@ -93,10 +93,9 @@ namespace Opm {
     }
 
 
-    void Well::WellProductionProperties::init_vfp(const std::optional<VFPProdTable::ALQ_TYPE>& alq_type, const UnitSystem& unit_system_arg, const DeckRecord& record) {
+    void Well::WellProductionProperties::init_vfp(const std::optional<VFPProdTable::ALQ_TYPE>& alq_type, const int vfp_table_nr, const UnitSystem& unit_system_arg, const DeckRecord& record) {
+        this->VFPTableNumber = vfp_table_nr;
         if (alq_type) {
-            if (!record.getItem("VFP_TABLE").defaultApplied(0))
-                this->VFPTableNumber = record.getItem("VFP_TABLE").get< int >(0);
             const auto alq_dim = VFPProdTable::ALQDimension(*alq_type, unit_system_arg);
             const auto& alq_input = record.getItem("ALQ").get<UDAValue>(0);
             if (alq_input.is<double>())
@@ -108,8 +107,7 @@ namespace Opm {
                 this->ALQValue = UDAValue(alq_input.get<std::string>(), alq_dim);
             }
         } else {
-            const auto table_nr = record.getItem("VFP_TABLE").get< int >(0);
-            if (table_nr != 0)
+            if (this->VFPTableNumber != 0)
                 throw std::logic_error("VFP table inconsistency - BUG");
         }
     }
@@ -166,6 +164,7 @@ namespace Opm {
 
 
     void Well::WellProductionProperties::handleWCONPROD(const std::optional<VFPProdTable::ALQ_TYPE>& alq_type,
+                                                        const int vfp_table_nr,
                                                         const double bhp_def,
                                                         const UnitSystem& unit_system_arg,
                                                         const std::string& well_name,
@@ -173,7 +172,7 @@ namespace Opm {
                                                         const KeywordLocation& location)
     {
         this->predictionMode = true;
-        this->init_vfp(alq_type, unit_system_arg, record);
+        this->init_vfp(alq_type, vfp_table_nr, unit_system_arg, record);
         this->init_rates(record);
 
         if (record.getItem("BHP").defaultApplied(0)) {
@@ -234,12 +233,13 @@ namespace Opm {
       default constructor and the handleWCONPROD() method.
     */
 void Well::WellProductionProperties::handleWCONHIST(const std::optional<VFPProdTable::ALQ_TYPE>& alq_type,
+                                                    const int vfp_table_nr,
                                                     const double bhp_def,
                                                     const UnitSystem& unit_system_arg,
                                                     const DeckRecord& record)
     {
         this->init_rates(record);
-        this->init_vfp(alq_type, unit_system_arg, record);
+        this->init_vfp(alq_type, vfp_table_nr, unit_system_arg, record);
         this->LiquidRate.update(0);
         this->ResVRate.update(0);
 
@@ -417,6 +417,7 @@ void Well::WellProductionProperties::handleWCONHIST(const std::optional<VFPProdT
         controls.bhp_history = this->BHPH;
         controls.thp_history = this->THPH;
         controls.vfp_table_number = this->VFPTableNumber;
+        controls.prediction_mode = this->predictionMode;
         controls.cmode = this->controlMode;
 
         return controls;
