@@ -20,14 +20,15 @@
 #ifndef UNITSYSTEM_H
 #define UNITSYSTEM_H
 
-#include <string>
-#include <map>
-#include <vector>
-#include <memory>
-
 #include <opm/input/eclipse/Units/Dimension.hpp>
+
 #include <opm/input/eclipse/Schedule/UDQ/UDQEnums.hpp>
 
+#include <map>
+#include <memory>
+#include <string>
+#include <vector>
+
 namespace Opm {
 
     class UnitSystem {
@@ -86,6 +87,7 @@ namespace Opm {
             moles,
             ppm,
             ymodule,
+            dfactor,
             _count // New entries must be added *before* this
         };
 
@@ -115,8 +117,8 @@ namespace Opm {
 
         Dimension parse(const std::string& dimension) const;
 
-        double from_si( const std::string& dimension, double );
-        double to_si( const std::string& dimension, double );
+        double from_si( const std::string& dimension, double ) const;
+        double to_si( const std::string& dimension, double ) const;
         double from_si( measure, double ) const;
         double to_si( measure, double ) const;
         void from_si( measure, std::vector<double>& ) const;
