@@ -28,7 +28,6 @@
 #include <filesystem>
 #include <fstream>
 #include <iterator>
-#include <iomanip>
 #include <numeric>
 #include <string>
 #include <stdexcept>
@@ -38,8 +37,8 @@ namespace Opm { namespace EclIO {
 
 using NNCentry = std::tuple<int, int, int, int, int,int, float>;
 
-EGrid::EGrid(const std::string &filename, std::string grid_name) :
-    EclFile(filename), inputFileName { filename }, m_grid_name {grid_name}
+EGrid::EGrid(const std::string& filename, const std::string& grid_name)
+    : EclFile(filename), inputFileName { filename }, m_grid_name {grid_name}
 {
     initFileName = inputFileName.parent_path() / inputFileName.stem();
 
@@ -165,8 +164,12 @@ EGrid::EGrid(const std::string &filename, std::string grid_name) :
         auto hostnum = getImpl(hostnum_index, INTE, inte_array, "integer");
         host_cells.reserve(hostnum.size());
 
-        for (auto val : hostnum)
-            host_cells.push_back(val -1);
+        std::transform(hostnum.begin(), hostnum.end(),
+                       std::back_inserter(host_cells),
+                       [](const auto& val)
+                       {
+                           return val - 1;
+                       });
     }
 }
 
@@ -372,11 +375,17 @@ void EGrid::getCellCorners(const std::array<int, 3>& ijk,
             yb = coord_array[pind[n] + 4];
         }
 
-        X[n] = xt + (xb-xt) / (zt-zb) * (zt - Z[n]);
-        X[n+4] = xt + (xb-xt) / (zt-zb) * (zt-Z[n+4]);
-
-        Y[n] = yt+(yb-yt)/(zt-zb)*(zt-Z[n]);
-        Y[n+4] = yt+(yb-yt)/(zt-zb)*(zt-Z[n+4]);
+        if (zt == zb) {
+            X[n] = xt;
+            X[n+4] = xt;
+            Y[n] = yt;
+            Y[n+4] = yt;
+        } else {
+            X[n] = xt + (xb-xt) / (zt-zb) * (zt - Z[n]);
+            X[n+4] = xt + (xb-xt) / (zt-zb) * (zt-Z[n+4]);
+            Y[n] = yt+(yb-yt)/(zt-zb)*(zt-Z[n]);
+            Y[n+4] = yt+(yb-yt)/(zt-zb)*(zt-Z[n+4]);
+        }
     }
 }
 
