@@ -417,7 +417,7 @@ ESmry::ESmry(const std::string &filename, bool loadBaseRunData) :
     std::map<std::string, int> keyIndex;
     {
         size_t m = 0;
-        for (auto key : keywList)
+        for (const auto& key : keywList)
             if (!key.empty())
                 keyIndex[key] = m++;
     }
@@ -636,7 +636,6 @@ void ESmry::read_ministeps_from_disk()
 {
     auto specInd = std::get<0>(miniStepList[0]);
     auto dataFileIndex = std::get<1>(miniStepList[0]);
-    uint64_t stepFilePos;
 
     std::fstream fileH;
 
@@ -660,7 +659,7 @@ void ESmry::read_ministeps_from_disk()
                 fileH.open(dataFileList[dataFileIndex], std::ios::in |  std::ios::binary);
         }
 
-        stepFilePos = std::get<2>(miniStepList[n]);
+        uint64_t stepFilePos = std::get<2>(miniStepList[n]);
 
         fileH.seekg (stepFilePos , fileH.beg);
 
@@ -885,15 +884,23 @@ void ESmry::loadData() const
 
             const auto fileStr = std::string_view(buffer.data(), size);
             std::size_t p = 0;
-            std::int64_t p1= 0;
+            std::size_t p1= 0;
 
             for (int i=0; i< nParamsSpecFile[specInd]; ++i, ++p) {
                 p1 = fileStr.find_first_not_of(' ',p1);
-                const std::int64_t p2 = fileStr.find_first_of(' ', p1);
+                const std::size_t p2 = fileStr.find_first_of(' ', p1);
 
-                if ((keywpos[p] > -1) && !vectorLoaded[keywpos[p]]) {
-                    const auto dtmpv = std::strtof(fileStr.substr(p1, p2-p1).data(), nullptr);
-                    vectorData[keywpos[p]].push_back(dtmpv);
+                if (p1 == std::string::npos) {
+                    // File possibly corrupted. Adding an obviously invalid value.
+                    if ((keywpos[p] > -1) && !vectorLoaded[keywpos[p]]) {
+                        const float invalid_value = -1e20f;
+                        vectorData[keywpos[p]].push_back(invalid_value);
+                    }
+                } else {
+                    if ((keywpos[p] > -1) && !vectorLoaded[keywpos[p]]) {
+                        const auto dtmpv = std::strtof(fileStr.substr(p1, p2-p1).data(), nullptr);
+                        vectorData[keywpos[p]].push_back(dtmpv);
+                    }
                 }
 
                 p1 = fileStr.find_first_not_of(' ',p2);
@@ -944,7 +951,7 @@ void ESmry::loadData() const
 
 
 std::vector<std::tuple <std::string, uint64_t>>
-ESmry::getListOfArrays(std::string filename, bool formatted)
+ESmry::getListOfArrays(const std::string& filename, bool formatted)
 {
     std::vector<std::tuple <std::string, uint64_t>> resultVect;
 
@@ -1059,11 +1066,10 @@ bool ESmry::make_esmry_file()
     if (mini_steps.size() == 0)
         this->read_ministeps_from_disk();
 
-    std::filesystem::path path = inputFileName.parent_path();
-    std::filesystem::path rootName = inputFileName.stem();
-    std::filesystem::path smryDataFile;
-
-    smryDataFile = path / rootName += ".ESMRY";
+    const std::filesystem::path path = inputFileName.parent_path();
+    const std::filesystem::path rootName = inputFileName.stem();
+    std::filesystem::path smryDataFile = path / rootName;
+    smryDataFile.replace_extension(".ESMRY");
 
     if (Opm::EclIO::fileExists(smryDataFile))
     {
@@ -1097,9 +1103,8 @@ bool ESmry::make_esmry_file()
             std::vector<std::string> units;
             units.reserve(keyword.size());
 
-            for (const auto& key : keyword)
-                units.push_back(kwunits.at(key));
-
+            std::transform(keyword.begin(), keyword.end(), std::back_inserter(units),
+                           [this](const auto& key) { return kwunits.at(key); });
 
             Opm::EclIO::EclOutput outFile(smryDataFile, false, std::ios::out);
 
@@ -1399,10 +1404,8 @@ std::vector<std::string> ESmry::keywordList(const std::string& pattern) const
 {
     std::vector<std::string> list;
 
-    for (auto key : keyword)
-        if (shmatch(pattern, key))
-            list.push_back(key);
-
+    std::copy_if(keyword.begin(), keyword.end(), std::back_inserter(list),
+                 [&pattern](const auto& key) { return shmatch(pattern, key); });
     return list;
 }
 
@@ -1412,12 +1415,20 @@ const std::vector<SummaryNode>& ESmry::summaryNodeList() const {
     return summaryNodes;
 }
 
-std::vector<Opm::time_point> ESmry::dates() const {
+std::vector<Opm::time_point> ESmry::dates() const
+{
     double time_unit = 24 * 3600;
     std::vector<Opm::time_point> d;
 
-    for (const auto& t : this->get("TIME"))
-        d.push_back( this->tp_startdat + std::chrono::duration_cast<std::chrono::seconds>( std::chrono::duration<double, std::chrono::seconds::period>( t * time_unit)));
+    std::transform(this->get("TIME").begin(),
+                   this->get("TIME").end(),
+                   std::back_inserter(d),
+                   [this, time_unit](const auto& t)
+                   {
+                       using Seconds = std::chrono::duration<double, std::chrono::seconds::period>;
+                       return this->tp_startdat +
+                              std::chrono::duration_cast<std::chrono::seconds>(Seconds{t * time_unit});
+                   });
 
     return d;
 }
