@@ -20,18 +20,16 @@
 #include <opm/io/eclipse/EclUtil.hpp>
 #include <opm/common/ErrorMacros.hpp>
 
-#include <fmt/format.h>
 #include <algorithm>
-#include <array>
 #include <cstring>
-#include <functional>
+#include <cstddef>
 #include <fstream>
-#include <iomanip>
-#include <iterator>
 #include <string>
 #include <numeric>
 #include <cmath>
 
+#include <fmt/format.h>
+
 namespace Opm { namespace EclIO {
 
 void EclFile::load(bool preload) {
@@ -203,7 +201,7 @@ void EclFile::loadData()
             OPM_THROW(std::runtime_error, message);
         }
 
-        for (size_t i = 0; i < array_name.size(); i++) {
+        for (std::size_t i = 0; i < array_name.size(); i++) {
             loadBinaryArray(fileH, i);
         }
 
@@ -225,11 +223,13 @@ void EclFile::loadData(const std::string& name)
 
                 inFile.seekg(ifStreamPos[arrIndex]);
 
-                size_t size = sizeOnDiskFormatted(array_size[arrIndex], array_type[arrIndex], array_element_size[arrIndex])+1;
-                std::vector<char> buffer(size);
-                inFile.read (buffer.data(), size);
+                std::size_t disk_size = sizeOnDiskFormatted(array_size[arrIndex],
+                                                            array_type[arrIndex],
+                                                            array_element_size[arrIndex]) + 1;
+                std::vector<char> buffer(disk_size);
+                inFile.read (buffer.data(), disk_size);
 
-                std::string fileStr = std::string(buffer.data(), size);
+                std::string fileStr = std::string(buffer.data(), disk_size);
 
                 loadFormattedArray(fileStr, arrIndex, 0);
             }
@@ -267,11 +267,13 @@ void EclFile::loadData(const std::vector<int>& arrIndex)
 
             inFile.seekg(ifStreamPos[ind]);
 
-            size_t size = sizeOnDiskFormatted(array_size[ind], array_type[ind], array_element_size[ind])+1;
-            std::vector<char> buffer(size);
-            inFile.read (buffer.data(), size);
+            std::size_t disk_size = sizeOnDiskFormatted(array_size[ind],
+                                                        array_type[ind],
+                                                        array_element_size[ind]) + 1;
+            std::vector<char> buffer(disk_size);
+            inFile.read (buffer.data(), disk_size);
 
-            std::string fileStr = std::string(buffer.data(), size);
+            std::string fileStr = std::string(buffer.data(), disk_size);
 
             loadFormattedArray(fileStr, ind, 0);
         }
@@ -302,11 +304,13 @@ void EclFile::loadData(int arrIndex)
 
             inFile.seekg(ifStreamPos[arrIndex]);
 
-            size_t size = sizeOnDiskFormatted(array_size[arrIndex], array_type[arrIndex], array_element_size[arrIndex])+1;
-            std::vector<char> buffer(size);
-            inFile.read (buffer.data(), size);
+            std::size_t disk_size = sizeOnDiskFormatted(array_size[arrIndex],
+                                                        array_type[arrIndex],
+                                                        array_element_size[arrIndex]) + 1;
+            std::vector<char> buffer(disk_size);
+            inFile.read (buffer.data(), disk_size);
 
-            std::string fileStr = std::string(buffer.data(), size);
+            std::string fileStr = std::string(buffer.data(), disk_size);
 
             loadFormattedArray(fileStr, arrIndex, 0);
 
@@ -343,7 +347,7 @@ bool EclFile::is_ix() const
                 auto realStr = get_fmt_real_raw_str_values(n);
                 int p, first;
 
-                for (auto val : realStr) {
+                for (const auto& val : realStr) {
                     double dtmpv = fabs(std::stod(val));
 
                     if (dtmpv > 0.0) {
@@ -364,9 +368,10 @@ bool EclFile::is_ix() const
                 return true;
             } else if (array_type[n] == Opm::EclIO::LOGI) {
                 auto raw_logi_values = get_bin_logi_raw_values(n);
-                for (unsigned int val : raw_logi_values) {
-                    if (val == Opm::EclIO::true_value_ix)
-                        return true;
+                const auto it = std::find(raw_logi_values.begin(), raw_logi_values.end(),
+                                          Opm::EclIO::true_value_ix);
+                if (it != raw_logi_values.end()) {
+                    return true;
                 }
             }
         }
@@ -399,8 +404,6 @@ std::vector<unsigned int> EclFile::get_bin_logi_raw_values(int arrIndex) const
 
 std::vector<std::string> EclFile::get_fmt_real_raw_str_values(int arrIndex) const
 {
-    std::vector<std::string> real_vect;
-
     if (array_type[arrIndex] != Opm::EclIO::REAL)
         OPM_THROW(std::runtime_error, "Error, selected array is not of type REAL");
 
@@ -413,12 +416,14 @@ std::vector<std::string> EclFile::get_fmt_real_raw_str_values(int arrIndex) cons
 
     inFile.seekg(ifStreamPos[arrIndex]);
 
-    size_t size = sizeOnDiskFormatted(array_size[arrIndex], array_type[arrIndex], array_element_size[arrIndex])+1;
+    std::size_t disk_size = sizeOnDiskFormatted(array_size[arrIndex],
+                                                array_type[arrIndex],
+                                                array_element_size[arrIndex]) + 1;
 
-    std::vector<char> buffer(size);
-    inFile.read (buffer.data(), size);
+    std::vector<char> buffer(disk_size);
+    inFile.read (buffer.data(), disk_size);
 
-    std::string fileStr = std::string(buffer.data(), size);
+    std::string fileStr = std::string(buffer.data(), disk_size);
 
     std::vector<std::string> real_vect_str;
     real_vect_str = readFormattedRealRawStrings(fileStr, array_size[arrIndex], 0);
