@@ -59,7 +59,7 @@ namespace {
                const std::size_t               udq_index)
     {
         auto zudl_begin = zudl.begin();
-        auto zudl_end = zudl.begin();
+        auto zudl_end = zudl_begin;
         std::advance( zudl_begin, (udq_index + 0) * Opm::UDQDims::entriesPerZUDL() );
         std::advance( zudl_end  , (udq_index + 1) * Opm::UDQDims::entriesPerZUDL() );
 
@@ -90,10 +90,21 @@ RstState::RstState(std::shared_ptr<EclIO::RestartFileView> rstView,
     , aquifers(rstView, grid, unit_system)
     , netbalan(rstView->intehead(), rstView->doubhead(), unit_system)
     , network(rstView, unit_system)
+    , oilvap(runspec.tabdims().getNumPVTTables())
 {
     this->load_tuning(rstView->intehead(), rstView->doubhead());
+    this->load_oil_vaporization(rstView->intehead(), rstView->doubhead());
 }
 
+void RstState::load_oil_vaporization(const std::vector<int>& intehead,
+                                     const std::vector<double>& doubhead)
+{
+    const std::size_t numPvtRegions = this->oilvap.numPvtRegions();
+    const auto tconv = this->unit_system.to_si(::Opm::UnitSystem::measure::time, 1.0);
+    std::vector<double> maximums(numPvtRegions, doubhead[VI::doubhead::dRsDt]/tconv);
+    std::vector<std::string> options(numPvtRegions, intehead[VI::intehead::DRSDT_FREE]==1 ? "FREE" : "ALL");
+    OilVaporizationProperties::updateDRSDT(this->oilvap, maximums, options);
+}
 
 void RstState::load_tuning(const std::vector<int>& intehead,
                            const std::vector<double>& doubhead)
