@@ -27,7 +27,7 @@
 #ifndef OPM_CHECK_FLUIDSYSTEM_HPP
 #define OPM_CHECK_FLUIDSYSTEM_HPP
 
-#include <opm/common/utility/Demangle.hpp>
+#include <opm/common/utility/DemangledType.hpp>
 
 // include all fluid systems in opm-material
 #include <opm/material/fluidsystems/SinglePhaseFluidSystem.hpp>
@@ -49,8 +49,7 @@
 
 #include <iostream>
 #include <string>
-#include <string_view>
-#include <typeinfo>
+#include <tuple>
 
 /*!
  * \brief This is a fluid state which makes sure that only the quantities
@@ -222,12 +221,12 @@ public:
     }
 
 private:
-    bool allowSaturation_;
-    bool allowTemperature_;
-    bool allowPressure_;
-    bool allowComposition_;
-    bool allowDensity_;
-    int restrictPhaseIdx_;
+    bool allowSaturation_{false};
+    bool allowTemperature_{false};
+    bool allowPressure_{false};
+    bool allowComposition_{false};
+    bool allowDensity_{false};
+    int restrictPhaseIdx_{};
 };
 
 template <class Scalar, class BaseFluidState>
@@ -247,25 +246,21 @@ void checkFluidState(const BaseFluidState& fs)
 
     // make sure the fluid state provides all mandatory methods
     while (false) {
-        Scalar val = 1.0;
-
-        val = 2*val; // get rid of GCC warning (only occurs with paranoid warning flags)
-
-        val = fs.temperature(/*phaseIdx=*/0);
-        val = fs.pressure(/*phaseIdx=*/0);
-        val = fs.moleFraction(/*phaseIdx=*/0, /*compIdx=*/0);
-        val = fs.massFraction(/*phaseIdx=*/0, /*compIdx=*/0);
-        val = fs.averageMolarMass(/*phaseIdx=*/0);
-        val = fs.molarity(/*phaseIdx=*/0, /*compIdx=*/0);
-        val = fs.molarDensity(/*phaseIdx=*/0);
-        val = fs.molarVolume(/*phaseIdx=*/0);
-        val = fs.density(/*phaseIdx=*/0);
-        val = fs.saturation(/*phaseIdx=*/0);
-        val = fs.fugacity(/*phaseIdx=*/0, /*compIdx=*/0);
-        val = fs.fugacityCoefficient(/*phaseIdx=*/0, /*compIdx=*/0);
-        val = fs.enthalpy(/*phaseIdx=*/0);
-        val = fs.internalEnergy(/*phaseIdx=*/0);
-        val = fs.viscosity(/*phaseIdx=*/0);
+        std::ignore = fs.temperature(/*phaseIdx=*/0);
+        std::ignore = fs.pressure(/*phaseIdx=*/0);
+        std::ignore = fs.moleFraction(/*phaseIdx=*/0, /*compIdx=*/0);
+        std::ignore = fs.massFraction(/*phaseIdx=*/0, /*compIdx=*/0);
+        std::ignore = fs.averageMolarMass(/*phaseIdx=*/0);
+        std::ignore = fs.molarity(/*phaseIdx=*/0, /*compIdx=*/0);
+        std::ignore = fs.molarDensity(/*phaseIdx=*/0);
+        std::ignore = fs.molarVolume(/*phaseIdx=*/0);
+        std::ignore = fs.density(/*phaseIdx=*/0);
+        std::ignore = fs.saturation(/*phaseIdx=*/0);
+        std::ignore = fs.fugacity(/*phaseIdx=*/0, /*compIdx=*/0);
+        std::ignore = fs.fugacityCoefficient(/*phaseIdx=*/0, /*compIdx=*/0);
+        std::ignore = fs.enthalpy(/*phaseIdx=*/0);
+        std::ignore = fs.internalEnergy(/*phaseIdx=*/0);
+        std::ignore = fs.viscosity(/*phaseIdx=*/0);
     };
 }
 
@@ -276,9 +271,9 @@ template <class Scalar, class FluidSystem, class RhsEval, class LhsEval>
 void checkFluidSystem()
 {
     std::cout << "Testing fluid system '"
-              << Opm::demangle(typeid(FluidSystem).name())
-              << ", RhsEval = " << Opm::demangle(typeid(RhsEval).name())
-              << ", LhsEval = " << Opm::demangle(typeid(LhsEval).name()) << "'\n";
+              << Opm::getDemangledType<FluidSystem>()
+              << ", RhsEval = " << Opm::getDemangledType<RhsEval>()
+              << ", LhsEval = " << Opm::getDemangledType<LhsEval>() << "'\n";
 
     // make sure the fluid system provides the number of phases and
     // the number of components
@@ -375,14 +370,13 @@ void checkFluidSystem()
     // test for phaseName(), isLiquid() and isIdealGas()
     for (unsigned phaseIdx = 0; phaseIdx < numPhases; ++ phaseIdx) {
         [[maybe_unused]] std::string name{FluidSystem::phaseName(phaseIdx)};
-        bool bVal = FluidSystem::isLiquid(phaseIdx);
-        bVal = FluidSystem::isIdealGas(phaseIdx);
-        bVal = !bVal; // get rid of GCC warning (only occurs with paranoid warning flags)
+        std::ignore = FluidSystem::isLiquid(phaseIdx);
+        std::ignore = FluidSystem::isIdealGas(phaseIdx);
     }
 
     // test for molarMass() and componentName()
     for (unsigned compIdx = 0; compIdx < numComponents; ++ compIdx) {
-        val = FluidSystem::molarMass(compIdx);
+        std::ignore = FluidSystem::molarMass(compIdx);
         std::string{FluidSystem::componentName(compIdx)};
     }
 }
