@@ -27,13 +27,13 @@
 #ifndef OPM_ECL_TWO_PHASE_MATERIAL_HPP
 #define OPM_ECL_TWO_PHASE_MATERIAL_HPP
 
-#include "EclTwoPhaseMaterialParams.hpp"
+#include <opm/material/fluidmatrixinteractions/EclTwoPhaseMaterialParams.hpp>
+
+#include <opm/common/TimingMacros.hpp>
 
 #include <opm/material/common/Valgrind.hpp>
 #include <opm/material/common/MathToolbox.hpp>
 
-#include <algorithm>
-
 namespace Opm {
 
 /*!
@@ -182,83 +182,107 @@ public:
 
     /*
      * Hysteresis parameters for oil-water
-     * @see EclHysteresisTwoPhaseLawParams::pcSwMdc(...)
-     * @see EclHysteresisTwoPhaseLawParams::krnSwMdc(...)
+     * @see EclHysteresisTwoPhaseLawParams::soMax(...)
+     * @see EclHysteresisTwoPhaseLawParams::swMax(...)
+     * @see EclHysteresisTwoPhaseLawParams::swMin(...)
      * \param params Parameters
      */
-    static void oilWaterHysteresisParams(Scalar& pcSwMdc,
-                                         Scalar& krnSwMdc,
+    static void oilWaterHysteresisParams(Scalar& soMax,
+                                         Scalar& swMax,
+                                         Scalar& swMin,
                                          const Params& params)
     {
-        OPM_TIMEFUNCTION_LOCAL();
-        pcSwMdc = params.oilWaterParams().pcSwMdc();
-        krnSwMdc = params.oilWaterParams().krnSwMdc();
-
-        Valgrind::CheckDefined(pcSwMdc);
-        Valgrind::CheckDefined(krnSwMdc);
+        soMax = 1.0 - params.oilWaterParams().krnSwMdc();
+        swMax = params.oilWaterParams().krwSwMdc();
+        swMin = params.oilWaterParams().pcSwMdc();
+        Valgrind::CheckDefined(soMax);
+        Valgrind::CheckDefined(swMax);
+        Valgrind::CheckDefined(swMin);
     }
 
     /*
      * Hysteresis parameters for oil-water
-     * @see EclHysteresisTwoPhaseLawParams::pcSwMdc(...)
-     * @see EclHysteresisTwoPhaseLawParams::krnSwMdc(...)
+     * @see EclHysteresisTwoPhaseLawParams::soMax(...)
+     * @see EclHysteresisTwoPhaseLawParams::swMax(...)
+     * @see EclHysteresisTwoPhaseLawParams::swMin(...)
      * \param params Parameters
      */
-    static void setOilWaterHysteresisParams(const Scalar& pcSwMdc,
-                                            const Scalar& krnSwMdc,
+    static void setOilWaterHysteresisParams(const Scalar& soMax,
+                                            const Scalar& swMax,
+                                            const Scalar& swMin,
                                             Params& params)
     {
-        OPM_TIMEFUNCTION_LOCAL();
-        constexpr const Scalar krwSw = 2.0; //Should not be used
-        params.oilWaterParams().update(pcSwMdc, krwSw, krnSwMdc);
+        params.oilWaterParams().update(swMin, swMax, 1.0 - soMax);
     }
 
+
     /*
      * Hysteresis parameters for gas-oil
-     * @see EclHysteresisTwoPhaseLawParams::pcSwMdc(...)
-     * @see EclHysteresisTwoPhaseLawParams::krnSwMdc(...)
+     * @see EclHysteresisTwoPhaseLawParams::sgmax(...)
+     * @see EclHysteresisTwoPhaseLawParams::shmax(...)
+     * @see EclHysteresisTwoPhaseLawParams::somin(...)
      * \param params Parameters
      */
-    static void gasOilHysteresisParams(Scalar& pcSwMdc,
-            Scalar& krnSwMdc,
+    static void gasOilHysteresisParams(Scalar& sgmax,
+                                       Scalar& shmax,
+                                       Scalar& somin,
                                        const Params& params)
     {
-        pcSwMdc = params.gasOilParams().pcSwMdc();
-        krnSwMdc = params.gasOilParams().krnSwMdc();
-
-        Valgrind::CheckDefined(pcSwMdc);
-        Valgrind::CheckDefined(krnSwMdc);
+        sgmax = 1.0 - params.gasOilParams().krnSwMdc();
+        shmax = params.gasOilParams().krwSwMdc();
+        somin = params.gasOilParams().pcSwMdc();
+        Valgrind::CheckDefined(sgmax);
+        Valgrind::CheckDefined(shmax);
+        Valgrind::CheckDefined(somin);
     }
 
     /*
      * Hysteresis parameters for gas-oil
-     * @see EclHysteresisTwoPhaseLawParams::pcSwMdc(...)
-     * @see EclHysteresisTwoPhaseLawParams::krnSwMdc(...)
+     * @see EclHysteresisTwoPhaseLawParams::sgmax(...)
+     * @see EclHysteresisTwoPhaseLawParams::shmax(...)
      * \param params Parameters
      */
-    static void setGasOilHysteresisParams(const Scalar& pcSwMdc,
-                                          const Scalar& krnSwMdc,
+    static void setGasOilHysteresisParams(const Scalar& sgmax,
+                                          const Scalar& shmax,
+                                          const Scalar& somin,
                                           Params& params)
     {
-        constexpr const Scalar krwSw = 2.0; //Should not be used
-        params.gasOilParams().update(pcSwMdc, krwSw, krnSwMdc);
+        params.gasOilParams().update(somin , shmax, 1.0 - sgmax);
     }
 
-    static Scalar trappedGasSaturation(const Params& params){
+    static Scalar trappedGasSaturation(const Params& params, bool maximumTrapping){
         if(params.approach() == EclTwoPhaseApproach::GasOil)
-            return params.gasOilParams().SnTrapped();
+            return params.gasOilParams().SnTrapped(maximumTrapping);
         if(params.approach() == EclTwoPhaseApproach::GasWater)
-            return params.gasWaterParams().SnTrapped();
+            return params.gasWaterParams().SnTrapped(maximumTrapping);
         return 0.0; // oil-water case
     }
 
-    static Scalar trappedOilSaturation(const Params& params){
+    static Scalar strandedGasSaturation(const Params& params, Scalar Sg, Scalar Kg){
+        if(params.approach() == EclTwoPhaseApproach::GasOil)
+            return params.gasOilParams().SnStranded(Sg, Kg);
+        if(params.approach() == EclTwoPhaseApproach::GasWater)
+            return params.gasWaterParams().SnStranded(Sg, Kg);
+        return 0.0; // oil-water case
+    }
+
+    static Scalar trappedOilSaturation(const Params& params, bool maximumTrapping){
         if(params.approach() == EclTwoPhaseApproach::GasOil)
             return params.gasOilParams().SwTrapped();
         if(params.approach() == EclTwoPhaseApproach::OilWater)
-            return params.oilWaterParams().SnTrapped();
+            return params.oilWaterParams().SnTrapped(maximumTrapping);
         return 0.0; // gas-water case
     }
+
+    static Scalar trappedWaterSaturation(const Params& params){
+        if(params.approach() == EclTwoPhaseApproach::GasWater)
+            return params.gasWaterParams().SwTrapped();
+        if(params.approach() == EclTwoPhaseApproach::OilWater)
+            return params.oilWaterParams().SwTrapped();
+        return 0.0; // gas-oil case
+    }
+
+
     /*!
      * \brief Capillary pressure between the gas and the non-wetting
      *        liquid (i.e., oil) phase.
@@ -433,21 +457,18 @@ public:
             Scalar So = scalarValue(fluidState.saturation(oilPhaseIdx));
 
             return params.gasOilParams().update(/*pcSw=*/So, /*krwSw=*/So, /*krnSw=*/So);
-            break;
         }
 
         case EclTwoPhaseApproach::OilWater: {
             Scalar Sw = scalarValue(fluidState.saturation(waterPhaseIdx));
 
             return params.oilWaterParams().update(/*pcSw=*/Sw, /*krwSw=*/Sw, /*krnSw=*/Sw);
-            break;
         }
 
         case EclTwoPhaseApproach::GasWater: {
             Scalar Sw = scalarValue(fluidState.saturation(waterPhaseIdx));
 
             return params.gasWaterParams().update(/*pcSw=*/Sw, /*krwSw=*/Sw, /*krnSw=*/Sw);
-            break;
         }
         }
 
