@@ -510,7 +510,12 @@ public:
      * exception!
      */
     Scalar internalEnergy(unsigned phaseIdx) const
-    { return (*enthalpy_)[canonicalToStoragePhaseIndex_(phaseIdx)] - pressure(phaseIdx)/density(phaseIdx); }
+    {   auto energy = (*enthalpy_)[canonicalToStoragePhaseIndex_(phaseIdx)];
+        if(!FluidSystem::enthalpyEqualEnergy()){
+            energy -= pressure(phaseIdx)/density(phaseIdx);
+        }
+        return energy;
+    }
 
     //////
     // slow methods
@@ -675,21 +680,22 @@ private:
             return FluidSystem::canonicalToActivePhaseIdx(canonicalPhaseIdx);
     }
 
-    ConditionalStorage<enableTemperature || enableEnergy, Scalar> temperature_;
-    ConditionalStorage<enableEnergy, std::array<Scalar, numStoragePhases> > enthalpy_;
-    Scalar totalSaturation_;
-    std::array<Scalar, numStoragePhases> pressure_;
-    std::array<Scalar, numStoragePhases> pc_;
-    std::array<Scalar, numStoragePhases> saturation_;
-    std::array<Scalar, numStoragePhases> invB_;
-    std::array<Scalar, numStoragePhases> density_;
-    ConditionalStorage<enableDissolution,Scalar> Rs_;
-    ConditionalStorage<enableDissolution, Scalar> Rv_;
-    ConditionalStorage<enableVapwat,Scalar> Rvw_;
-    ConditionalStorage<enableDissolutionInWater,Scalar> Rsw_;
-    ConditionalStorage<enableBrine, Scalar> saltConcentration_;
-    ConditionalStorage<enableSaltPrecipitation, Scalar> saltSaturation_;
-    unsigned short pvtRegionIdx_;
+    ConditionalStorage<enableTemperature || enableEnergy, Scalar> temperature_{};
+    ConditionalStorage<enableEnergy, std::array<Scalar, numStoragePhases> > enthalpy_{};
+    Scalar totalSaturation_{};
+    std::array<Scalar, numStoragePhases> pressure_{};
+    std::array<Scalar, numStoragePhases> pc_{};
+    std::array<Scalar, numStoragePhases> saturation_{};
+    std::array<Scalar, numStoragePhases> invB_{};
+    std::array<Scalar, numStoragePhases> density_{};
+    ConditionalStorage<enableDissolution,Scalar> Rs_{};
+    ConditionalStorage<enableDissolution, Scalar> Rv_{};
+    ConditionalStorage<enableVapwat,Scalar> Rvw_{};
+    ConditionalStorage<enableDissolutionInWater,Scalar> Rsw_{};
+    ConditionalStorage<enableBrine, Scalar> saltConcentration_{};
+    ConditionalStorage<enableSaltPrecipitation, Scalar> saltSaturation_{};
+
+    unsigned short pvtRegionIdx_{};
 };
 
 } // namespace Opm
