@@ -43,6 +43,11 @@ template <class Scalar, class IndexTraits>
 void BlackOilFluidSystem<Scalar,IndexTraits>::
 initFromState(const EclipseState& eclState, const Schedule& schedule)
 {
+    if(eclState.getSimulationConfig().useEnthalpy()){
+        enthalpy_eq_energy_ = false;
+    }else{
+        enthalpy_eq_energy_ = true;
+    }
     std::size_t numRegions = eclState.runspec().tabdims().getNumPVTTables();
     initBegin(numRegions);
 
@@ -90,7 +95,7 @@ initFromState(const EclipseState& eclState, const Schedule& schedule)
             // For CO2SOL and H2SOL the dissolved gas in water is added in the solvent model
             // The HC gas is not allowed to dissolved into water.
             // For most HC gasses this is a resonable assumption.
-            OpmLog::info("CO2SOL/H2SOL is activated together with DISGASW. \n" 
+            OpmLog::info("CO2SOL/H2SOL is activated together with DISGASW. \n"
                          "Only CO2/H2 is allowed to dissolve into water");
         } else
             OPM_THROW(std::runtime_error,
@@ -126,10 +131,7 @@ initFromState(const EclipseState& eclState, const Schedule& schedule)
     // use molarMass of CO2 and Brine as default
     // when we are using the the CO2STORE option
     if (eclState.runspec().co2Storage()) {
-        const Scalar molality = eclState.getTableManager().salinity(); // mol/kg
-        const Scalar MmNaCl = 58.44e-3; // molar mass of NaCl [kg/mol]
-        // convert to mass fraction
-        const Scalar salinity = 1 / ( 1 + 1 / (molality*MmNaCl));
+        const Scalar salinity = eclState.getCo2StoreConfig().salinity();  // mass fraction
         for (unsigned regionIdx = 0; regionIdx < numRegions; ++regionIdx) {
             if (phaseIsActive(oilPhaseIdx)) // The oil component is used for the brine if OIL is active
                 molarMass_[regionIdx][oilCompIdx] = BrineCo2Pvt<Scalar>::Brine::molarMass(salinity);
@@ -162,6 +164,10 @@ initFromState(const EclipseState& eclState, const Schedule& schedule)
         }
     }
 
+    // For co2storage and h2storage we dont have a concept of tables and should not spend time on
+    // checking if we are at the saturated front
+    setUseSaturatedTables(!(eclState.runspec().h2Storage() || eclState.runspec().co2Storage()));
+
     setEnableDiffusion(eclState.getSimulationConfig().isDiffusive());
     if (enableDiffusion()) {
         const auto& diffCoeffTables = eclState.getTableManager().getDiffusionCoefficientTable();
@@ -223,6 +229,7 @@ void BlackOilFluidSystem<Scalar,IndexTraits>::
 initBegin(std::size_t numPvtRegions)
 {
     isInitialized_ = false;
+    useSaturatedTables_ = true;
 
     enableDissolvedGas_ = true;
     enableDissolvedGasInWater_ = false;
@@ -392,6 +399,72 @@ resizeArrays_(std::size_t numRegions)
     referenceDensity_.resize(numRegions);
 }
 
+template<> unsigned char BlackOilFluidSystem<double, BlackOilDefaultIndexTraits>::numActivePhases_ = 0;
+template<> std::array<bool, BlackOilFluidSystem<double, BlackOilDefaultIndexTraits>::numPhases>
+BlackOilFluidSystem<double, BlackOilDefaultIndexTraits>::phaseIsActive_ = {false, false, false};
+
+template<> unsigned char BlackOilFluidSystem<float, BlackOilDefaultIndexTraits>::numActivePhases_ = 0;
+template<> std::array<bool, BlackOilFluidSystem<float, BlackOilDefaultIndexTraits>::numPhases>
+BlackOilFluidSystem<float, BlackOilDefaultIndexTraits>::phaseIsActive_ = {false, false, false};
+
+template<> std::array<short, BlackOilFluidSystem<double, BlackOilDefaultIndexTraits>::numPhases> BlackOilFluidSystem<double, BlackOilDefaultIndexTraits>::activeToCanonicalPhaseIdx_ = {0, 1, 2};
+
+template<> std::array<short, BlackOilFluidSystem<float, BlackOilDefaultIndexTraits>::numPhases> BlackOilFluidSystem<float, BlackOilDefaultIndexTraits>::activeToCanonicalPhaseIdx_ = {0, 1, 2};
+
+template<> std::array<short, BlackOilFluidSystem<double, BlackOilDefaultIndexTraits>::numPhases> BlackOilFluidSystem<double, BlackOilDefaultIndexTraits>::canonicalToActivePhaseIdx_ = {0, 1, 2};
+
+template<> std::array<short, BlackOilFluidSystem<float, BlackOilDefaultIndexTraits>::numPhases> BlackOilFluidSystem<float, BlackOilDefaultIndexTraits>::canonicalToActivePhaseIdx_ = {0, 1, 2};
+
+template<> double BlackOilFluidSystem<double, BlackOilDefaultIndexTraits>::surfaceTemperature = 0.0;
+template<> float BlackOilFluidSystem<float, BlackOilDefaultIndexTraits>::surfaceTemperature = 0.0;
+
+template<> double BlackOilFluidSystem<double, BlackOilDefaultIndexTraits>::surfacePressure = 0.0;
+template<> float BlackOilFluidSystem<float, BlackOilDefaultIndexTraits>::surfacePressure = 0.0;
+
+template<> double BlackOilFluidSystem<double, BlackOilDefaultIndexTraits>::reservoirTemperature_ = 0.0;
+template<> float BlackOilFluidSystem<float, BlackOilDefaultIndexTraits>::reservoirTemperature_ = 0.0;
+
+template<> bool BlackOilFluidSystem<double, BlackOilDefaultIndexTraits>::enableDissolvedGas_ = true;
+template<> bool BlackOilFluidSystem<float, BlackOilDefaultIndexTraits>::enableDissolvedGas_ = true;
+
+template <> bool BlackOilFluidSystem<double, BlackOilDefaultIndexTraits>::enableDissolvedGasInWater_ = false;
+template <> bool BlackOilFluidSystem<float, BlackOilDefaultIndexTraits>::enableDissolvedGasInWater_ = false;
+
+template <> bool BlackOilFluidSystem<double, BlackOilDefaultIndexTraits>::enableVaporizedOil_ = false;
+template <> bool BlackOilFluidSystem<float, BlackOilDefaultIndexTraits>::enableVaporizedOil_ = false;
+
+template <> bool BlackOilFluidSystem<double, BlackOilDefaultIndexTraits>::enableVaporizedWater_ = false;
+template <> bool BlackOilFluidSystem<float, BlackOilDefaultIndexTraits>::enableVaporizedWater_ = false;
+
+template <> bool BlackOilFluidSystem<double, BlackOilDefaultIndexTraits>::enableDiffusion_ = false;
+template <> bool BlackOilFluidSystem<float, BlackOilDefaultIndexTraits>::enableDiffusion_ = false;
+
+template <> std::shared_ptr<OilPvtMultiplexer<double>> BlackOilFluidSystem<double, BlackOilDefaultIndexTraits>::oilPvt_ = nullptr;
+template <> std::shared_ptr<OilPvtMultiplexer<float>> BlackOilFluidSystem<float, BlackOilDefaultIndexTraits>::oilPvt_ = nullptr;
+
+template <> std::shared_ptr<GasPvtMultiplexer<double>> BlackOilFluidSystem<double, BlackOilDefaultIndexTraits>::gasPvt_ = nullptr;
+template <> std::shared_ptr<GasPvtMultiplexer<float>> BlackOilFluidSystem<float, BlackOilDefaultIndexTraits>::gasPvt_ = nullptr;
+
+template <> std::shared_ptr<WaterPvtMultiplexer<double>> BlackOilFluidSystem<double, BlackOilDefaultIndexTraits>::waterPvt_ = nullptr;
+template <> std::shared_ptr<WaterPvtMultiplexer<float>> BlackOilFluidSystem<float, BlackOilDefaultIndexTraits>::waterPvt_ = nullptr;
+
+template <> std::vector<std::array<double, 3>> BlackOilFluidSystem<double, BlackOilDefaultIndexTraits>::referenceDensity_ = {};
+template <> std::vector<std::array<float, 3>> BlackOilFluidSystem<float, BlackOilDefaultIndexTraits>::referenceDensity_ = {};
+
+template <> std::vector<std::array<double, 3>> BlackOilFluidSystem<double, BlackOilDefaultIndexTraits>::molarMass_ = {};
+template <> std::vector<std::array<float, 3>> BlackOilFluidSystem<float, BlackOilDefaultIndexTraits>::molarMass_ = {};
+
+template <> std::vector<std::array<double, 9>> BlackOilFluidSystem<double, BlackOilDefaultIndexTraits>::diffusionCoefficients_ = {};
+template <> std::vector<std::array<float, 9>> BlackOilFluidSystem<float, BlackOilDefaultIndexTraits>::diffusionCoefficients_ = {};
+
+template <> bool BlackOilFluidSystem<double, BlackOilDefaultIndexTraits>::isInitialized_ = false;
+template <> bool BlackOilFluidSystem<float, BlackOilDefaultIndexTraits>::isInitialized_ = false;
+
+template <> bool BlackOilFluidSystem<double, BlackOilDefaultIndexTraits>::useSaturatedTables_ = false;
+template <> bool BlackOilFluidSystem<float, BlackOilDefaultIndexTraits>::useSaturatedTables_ = false;
+
+// IMPORTANT: The following two lines must come after the template specializations above
+//    or else the static variable above will appear as undefined in the generated object file.
 template class BlackOilFluidSystem<double,BlackOilDefaultIndexTraits>;
 template class BlackOilFluidSystem<float,BlackOilDefaultIndexTraits>;
 
