@@ -31,8 +31,6 @@
 #include "blackoilpvt/OilPvtMultiplexer.hpp"
 #include "blackoilpvt/GasPvtMultiplexer.hpp"
 #include "blackoilpvt/WaterPvtMultiplexer.hpp"
-#include "blackoilpvt/BrineCo2Pvt.hpp"
-#include "blackoilpvt/BrineH2Pvt.hpp"
 
 #include <opm/common/TimingMacros.hpp>
 
@@ -42,6 +40,7 @@
 #include <opm/material/common/MathToolbox.hpp>
 #include <opm/material/common/Valgrind.hpp>
 #include <opm/material/common/HasMemberGeneratorMacros.hpp>
+#include <opm/material/fluidsystems/NullParameterCache.hpp>
 
 #include <array>
 #include <cstddef>
@@ -174,10 +173,10 @@ public:
         using Evaluation = EvaluationT;
 
     public:
-        ParameterCache(Scalar maxOilSat = 1.0, unsigned regionIdx=0)
+        explicit ParameterCache(Scalar maxOilSat = 1.0, unsigned regionIdx = 0)
+            : maxOilSat_(maxOilSat)
+            , regionIdx_(regionIdx)
         {
-            maxOilSat_ = maxOilSat;
-            regionIdx_ = regionIdx;
         }
 
         /*!
@@ -288,6 +287,13 @@ public:
     static void setEnableDiffusion(bool yesno)
     { enableDiffusion_ = yesno; }
 
+    /*!
+     * \brief Specify whether the saturated tables should be used
+     *
+     * By default, saturated tables are used
+     */
+    static void setUseSaturatedTables(bool yesno)
+    { useSaturatedTables_ = yesno; }
 
     /*!
      * \brief Set the pressure-volume-saturation (PVT) relations for the gas phase.
@@ -486,6 +492,14 @@ public:
     static bool enableDiffusion()
     { return enableDiffusion_; }
 
+    /*!
+     * \brief Returns whether the saturated tables should be used
+     *
+     * By default, saturated tables are used. If false the unsaturated tables are extrapolated
+     */
+    static bool useSaturatedTables()
+    { return useSaturatedTables_; }
+
     /*!
      * \brief Returns the density of a fluid phase at surface pressure [kg/m^3]
      *
@@ -531,6 +545,12 @@ public:
                             unsigned phaseIdx)
     { return enthalpy<FluidState, LhsEval>(fluidState, phaseIdx, paramCache.regionIndex()); }
 
+    template <class FluidState, class LhsEval = typename FluidState::Scalar, class ParamCacheEval = LhsEval>
+    static LhsEval internalEnergy(const FluidState& fluidState,
+                                  const ParameterCache<ParamCacheEval>& paramCache,
+                                  unsigned phaseIdx)
+    { return internalEnergy<FluidState, LhsEval>(fluidState, phaseIdx, paramCache.regionIndex()); }
+
     /****************************************
      * thermodynamic quantities (black-oil specific version: Note that the PVT region
      * index is explicitly passed instead of a parameter cache object)
@@ -610,7 +630,7 @@ public:
         case waterPhaseIdx:
             if (enableDissolvedGasInWater()) {
                  // gas miscible in water
-                const LhsEval& Rsw = saturatedDissolutionFactor<FluidState, LhsEval>(fluidState, waterPhaseIdx, regionIdx);
+                const LhsEval& Rsw =BlackOil::template getRsw_<ThisType, FluidState, LhsEval>(fluidState, regionIdx);
                 const LhsEval& bw = waterPvt_->inverseFormationVolumeFactor(regionIdx, T, p, Rsw, saltConcentration);
                 return
                     bw*referenceDensity(waterPhaseIdx, regionIdx)
@@ -672,7 +692,7 @@ public:
 
                 return
                     bg*referenceDensity(gasPhaseIdx, regionIdx)
-                    + Rv*bg*referenceDensity(oilPhaseIdx, regionIdx) 
+                    + Rv*bg*referenceDensity(oilPhaseIdx, regionIdx)
                     + Rvw*bg*referenceDensity(waterPhaseIdx, regionIdx) ;
             }
 
@@ -751,7 +771,7 @@ public:
         case oilPhaseIdx: {
             if (enableDissolvedGas()) {
                 const auto& Rs = BlackOil::template getRs_<ThisType, FluidState, LhsEval>(fluidState, regionIdx);
-                if (fluidState.saturation(gasPhaseIdx) > 0.0
+                if (useSaturatedTables() && fluidState.saturation(gasPhaseIdx) > 0.0
                     && Rs >= (1.0 - 1e-10)*oilPvt_->saturatedGasDissolutionFactor(regionIdx, scalarValue(T), scalarValue(p)))
                 {
                     return oilPvt_->saturatedInverseFormationVolumeFactor(regionIdx, T, p);
@@ -767,20 +787,20 @@ public:
             if (enableVaporizedOil() && enableVaporizedWater()) {
                  const auto& Rvw = BlackOil::template getRvw_<ThisType, FluidState, LhsEval>(fluidState, regionIdx);
                  const auto& Rv = BlackOil::template getRv_<ThisType, FluidState, LhsEval>(fluidState, regionIdx);
-                 if (fluidState.saturation(waterPhaseIdx) > 0.0
+                 if (useSaturatedTables() && fluidState.saturation(waterPhaseIdx) > 0.0
                     && Rvw >= (1.0 - 1e-10)*gasPvt_->saturatedWaterVaporizationFactor(regionIdx, scalarValue(T), scalarValue(p))
                     && fluidState.saturation(oilPhaseIdx) > 0.0
                     && Rv >= (1.0 - 1e-10)*gasPvt_->saturatedOilVaporizationFactor(regionIdx, scalarValue(T), scalarValue(p)))
-                 { 
+                 {
                     return gasPvt_->saturatedInverseFormationVolumeFactor(regionIdx, T, p);
                  } else {
-                     return gasPvt_->inverseFormationVolumeFactor(regionIdx, T, p, Rv, Rvw);
+                    return gasPvt_->inverseFormationVolumeFactor(regionIdx, T, p, Rv, Rvw);
                  }
             }
 
             if (enableVaporizedOil()) {
                 const auto& Rv = BlackOil::template getRv_<ThisType, FluidState, LhsEval>(fluidState, regionIdx);
-                if (fluidState.saturation(oilPhaseIdx) > 0.0
+                if (useSaturatedTables() && fluidState.saturation(oilPhaseIdx) > 0.0
                     && Rv >= (1.0 - 1e-10)*gasPvt_->saturatedOilVaporizationFactor(regionIdx, scalarValue(T), scalarValue(p)))
                 {
                     return gasPvt_->saturatedInverseFormationVolumeFactor(regionIdx, T, p);
@@ -790,9 +810,9 @@ public:
                 }
             }
 
-            if (enableVaporizedWater()) { 
+            if (enableVaporizedWater()) {
                 const auto& Rvw = BlackOil::template getRvw_<ThisType, FluidState, LhsEval>(fluidState, regionIdx);
-                if (fluidState.saturation(waterPhaseIdx) > 0.0
+                if (useSaturatedTables() && fluidState.saturation(waterPhaseIdx) > 0.0
                     && Rvw >= (1.0 - 1e-10)*gasPvt_->saturatedWaterVaporizationFactor(regionIdx, scalarValue(T), scalarValue(p)))
                 {
                     return gasPvt_->saturatedInverseFormationVolumeFactor(regionIdx, T, p);
@@ -801,7 +821,7 @@ public:
                     return gasPvt_->inverseFormationVolumeFactor(regionIdx, T, p, Rv, Rvw);
                 }
             }
-            
+
             const LhsEval Rv(0.0);
             const LhsEval Rvw(0.0);
             return gasPvt_->inverseFormationVolumeFactor(regionIdx, T, p, Rv, Rvw);
@@ -811,7 +831,7 @@ public:
             const auto& saltConcentration = BlackOil::template getSaltConcentration_<ThisType, FluidState, LhsEval>(fluidState, regionIdx);
             if (enableDissolvedGasInWater()) {
                 const auto& Rsw = BlackOil::template getRsw_<ThisType, FluidState, LhsEval>(fluidState, regionIdx);
-                if (fluidState.saturation(gasPhaseIdx) > 0.0
+                if (useSaturatedTables() && fluidState.saturation(gasPhaseIdx) > 0.0
                     && Rsw >= (1.0 - 1e-10)*waterPvt_->saturatedGasDissolutionFactor(regionIdx, scalarValue(T), scalarValue(p), scalarValue(saltConcentration)))
                 {
                     return waterPvt_->saturatedInverseFormationVolumeFactor(regionIdx, T, p, saltConcentration);
@@ -995,7 +1015,7 @@ public:
         case oilPhaseIdx: {
             if (enableDissolvedGas()) {
                 const auto& Rs = BlackOil::template getRs_<ThisType, FluidState, LhsEval>(fluidState, regionIdx);
-                if (fluidState.saturation(gasPhaseIdx) > 0.0
+                if (useSaturatedTables() && fluidState.saturation(gasPhaseIdx) > 0.0
                     && Rs >= (1.0 - 1e-10)*oilPvt_->saturatedGasDissolutionFactor(regionIdx, scalarValue(T), scalarValue(p)))
                 {
                     return oilPvt_->saturatedViscosity(regionIdx, T, p);
@@ -1012,11 +1032,11 @@ public:
              if (enableVaporizedOil() && enableVaporizedWater()) {
                  const auto& Rvw = BlackOil::template getRvw_<ThisType, FluidState, LhsEval>(fluidState, regionIdx);
                  const auto& Rv = BlackOil::template getRv_<ThisType, FluidState, LhsEval>(fluidState, regionIdx);
-                 if (fluidState.saturation(waterPhaseIdx) > 0.0
+                 if (useSaturatedTables() && fluidState.saturation(waterPhaseIdx) > 0.0
                     && Rvw >= (1.0 - 1e-10)*gasPvt_->saturatedWaterVaporizationFactor(regionIdx, scalarValue(T), scalarValue(p))
                     && fluidState.saturation(oilPhaseIdx) > 0.0
                     && Rv >= (1.0 - 1e-10)*gasPvt_->saturatedOilVaporizationFactor(regionIdx, scalarValue(T), scalarValue(p)))
-                 { 
+                 {
                      return gasPvt_->saturatedViscosity(regionIdx, T, p);
                  } else {
                      return gasPvt_->viscosity(regionIdx, T, p, Rv, Rvw);
@@ -1024,7 +1044,7 @@ public:
             }
             if (enableVaporizedOil()) {
                 const auto& Rv = BlackOil::template getRv_<ThisType, FluidState, LhsEval>(fluidState, regionIdx);
-                if (fluidState.saturation(oilPhaseIdx) > 0.0
+                if (useSaturatedTables() && fluidState.saturation(oilPhaseIdx) > 0.0
                     && Rv >= (1.0 - 1e-10)*gasPvt_->saturatedOilVaporizationFactor(regionIdx, scalarValue(T), scalarValue(p)))
                 {
                     return gasPvt_->saturatedViscosity(regionIdx, T, p);
@@ -1035,10 +1055,10 @@ public:
             }
             if (enableVaporizedWater()) {
                 const auto& Rvw = BlackOil::template getRvw_<ThisType, FluidState, LhsEval>(fluidState, regionIdx);
-                if (fluidState.saturation(waterPhaseIdx) > 0.0
+                if (useSaturatedTables() && fluidState.saturation(waterPhaseIdx) > 0.0
                     && Rvw >= (1.0 - 1e-10)*gasPvt_->saturatedWaterVaporizationFactor(regionIdx, scalarValue(T), scalarValue(p)))
                 {
-                    return gasPvt_->saturatedViscosity(regionIdx, T, p); 
+                    return gasPvt_->saturatedViscosity(regionIdx, T, p);
                 } else {
                     const LhsEval Rv(0.0);
                     return gasPvt_->viscosity(regionIdx, T, p, Rv, Rvw);
@@ -1055,7 +1075,7 @@ public:
             const LhsEval& saltConcentration = BlackOil::template getSaltConcentration_<ThisType, FluidState, LhsEval>(fluidState, regionIdx);
             if (enableDissolvedGasInWater()) {
                 const auto& Rsw = BlackOil::template getRsw_<ThisType, FluidState, LhsEval>(fluidState, regionIdx);
-                if (fluidState.saturation(gasPhaseIdx) > 0.0
+                if (useSaturatedTables() && fluidState.saturation(gasPhaseIdx) > 0.0
                     && Rsw >= (1.0 - 1e-10)*waterPvt_->saturatedGasDissolutionFactor(regionIdx, scalarValue(T), scalarValue(p), scalarValue(saltConcentration)))
                 {
                     return waterPvt_->saturatedViscosity(regionIdx, T, p, saltConcentration);
@@ -1071,42 +1091,198 @@ public:
         throw std::logic_error("Unhandled phase index "+std::to_string(phaseIdx));
     }
 
+    template <class FluidState, class LhsEval = typename FluidState::Scalar>
+    static LhsEval internalEnergy(const FluidState& fluidState,
+                                  unsigned phaseIdx,
+                                  unsigned regionIdx){
+        bool is_mixing = false;
+        const LhsEval& p = decay<LhsEval>(fluidState.pressure(phaseIdx));
+        const LhsEval& T = decay<LhsEval>(fluidState.temperature(phaseIdx));
+
+        switch (phaseIdx) {
+        case oilPhaseIdx: {
+            if(!oilPvt_->mixingEnergy()){
+                const auto& oilEnergy =
+                    oilPvt_->internalEnergy(regionIdx, T, p,
+                                            BlackOil::template getRs_<ThisType, FluidState, LhsEval>(fluidState, regionIdx));
+
+                return oilEnergy;
+            }
+            is_mixing = true;
+            break;
+        }
+        case waterPhaseIdx: {
+            if(!waterPvt_->mixingEnergy()){
+                const auto waterEnergy =
+                    waterPvt_->internalEnergy(regionIdx, T, p,
+                                              BlackOil::template getRsw_<ThisType, FluidState, LhsEval>(fluidState, regionIdx),
+                                              BlackOil::template getSaltConcentration_<ThisType, FluidState, LhsEval>(fluidState, regionIdx));
+
+                return waterEnergy;
+            }
+            is_mixing = true;
+            break;
+        }
+        case gasPhaseIdx: {
+            if(!gasPvt_->mixingEnergy()){
+                const auto gasEnergy =
+                    gasPvt_->internalEnergy(regionIdx, T, p,
+                                            BlackOil::template getRv_<ThisType, FluidState, LhsEval>(fluidState, regionIdx),
+                                            BlackOil::template getRvw_<ThisType, FluidState, LhsEval>(fluidState, regionIdx));
+
+                return gasEnergy;
+            }
+            is_mixing = true;
+            break;
+        }
+        }
+        assert(is_mixing==true);
+        const auto& energy = internalMixingTotalEnergy<FluidState,LhsEval>(fluidState, phaseIdx, regionIdx);
+        const auto& phase_density = density<FluidState,LhsEval>(fluidState, phaseIdx, regionIdx);
+        return energy/phase_density;
+    }
+
+
+    template <class FluidState, class LhsEval = typename FluidState::Scalar>
+    static LhsEval internalMixingTotalEnergy(const FluidState& fluidState,
+                                             unsigned phaseIdx,
+                                             unsigned regionIdx)
+    {
+        assert(phaseIdx <= numPhases);
+        assert(regionIdx <= numRegions());
+        const LhsEval& p = decay<LhsEval>(fluidState.pressure(phaseIdx));
+        const LhsEval& T = decay<LhsEval>(fluidState.temperature(phaseIdx));
+        const LhsEval& saltConcentration = BlackOil::template getSaltConcentration_<ThisType, FluidState, LhsEval>(fluidState, regionIdx);
+        // to avoid putting all thermal into the interface of the multiplexer
+        switch (phaseIdx) {
+        case oilPhaseIdx: {
+            auto oilEnergy = oilPvt_->internalEnergy(regionIdx, T, p,
+                                                     BlackOil::template getRs_<ThisType, FluidState, LhsEval>(fluidState, regionIdx));
+            assert(oilPvt_->mixingEnergy());
+            //mixing energy adsed
+            if (enableDissolvedGas()) {
+                // miscible oil
+                const LhsEval& Rs = BlackOil::template getRs_<ThisType, FluidState, LhsEval>(fluidState, regionIdx);
+                const LhsEval& bo = oilPvt_->inverseFormationVolumeFactor(regionIdx, T, p, Rs);
+                const auto& gasEnergy =
+                    gasPvt_->internalEnergy(regionIdx, T, p,
+                                            BlackOil::template getRv_<ThisType, FluidState, LhsEval>(fluidState, regionIdx),
+                                            BlackOil::template getRvw_<ThisType, FluidState, LhsEval>(fluidState, regionIdx));
+                const auto hVapG = gasPvt_->hVap(regionIdx);// pressure correction ? assume equal to energy change
+                return
+                    oilEnergy*bo*referenceDensity(oilPhaseIdx, regionIdx)
+                    + (gasEnergy-hVapG)*Rs*bo*referenceDensity(gasPhaseIdx, regionIdx);
+            }
+
+            // immiscible oil
+            const LhsEval Rs(0.0);
+            const auto& bo = oilPvt_->inverseFormationVolumeFactor(regionIdx, T, p, Rs);
+
+            return oilEnergy*referenceDensity(phaseIdx, regionIdx)*bo;
+        }
+
+        case gasPhaseIdx: {
+            const auto& gasEnergy =
+                gasPvt_->internalEnergy(regionIdx, T, p,
+                                        BlackOil::template getRv_<ThisType, FluidState, LhsEval>(fluidState, regionIdx),
+                                        BlackOil::template getRvw_<ThisType, FluidState, LhsEval>(fluidState, regionIdx));
+            assert(gasPvt_->mixingEnergy());
+            if (enableVaporizedOil() && enableVaporizedWater()) {
+                const auto& oilEnergy =
+                    oilPvt_->internalEnergy(regionIdx, T, p,
+                                            BlackOil::template getRs_<ThisType, FluidState, LhsEval>(fluidState, regionIdx));
+                const auto waterEnergy =
+                    waterPvt_->internalEnergy(regionIdx, T, p,
+                                              BlackOil::template getRsw_<ThisType, FluidState, LhsEval>(fluidState, regionIdx),
+                                              BlackOil::template getSaltConcentration_<ThisType, FluidState, LhsEval>(fluidState, regionIdx));
+                // gas containing vaporized oil and vaporized water
+                const LhsEval& Rv = BlackOil::template getRv_<ThisType, FluidState, LhsEval>(fluidState, regionIdx);
+                const LhsEval& Rvw = BlackOil::template getRvw_<ThisType, FluidState, LhsEval>(fluidState, regionIdx);
+                const LhsEval& bg = gasPvt_->inverseFormationVolumeFactor(regionIdx, T, p, Rv, Rvw);
+                const auto hVapO = oilPvt_->hVap(regionIdx);
+                const auto hVapW = waterPvt_->hVap(regionIdx);
+                return
+                    gasEnergy*bg*referenceDensity(gasPhaseIdx, regionIdx)
+                    + (oilEnergy+hVapO)*Rv*bg*referenceDensity(oilPhaseIdx, regionIdx)
+                    + (waterEnergy+hVapW)*Rvw*bg*referenceDensity(waterPhaseIdx, regionIdx);
+            }
+            if (enableVaporizedOil()) {
+                const auto& oilEnergy =
+                    oilPvt_->internalEnergy(regionIdx, T, p,
+                                            BlackOil::template getRs_<ThisType, FluidState, LhsEval>(fluidState, regionIdx));
+                // miscible gas
+                const LhsEval Rvw(0.0);
+                const LhsEval& Rv = BlackOil::template getRv_<ThisType, FluidState, LhsEval>(fluidState, regionIdx);
+                const LhsEval& bg = gasPvt_->inverseFormationVolumeFactor(regionIdx, T, p, Rv, Rvw);
+                const auto hVapO = oilPvt_->hVap(regionIdx);
+                return
+                    gasEnergy*bg*referenceDensity(gasPhaseIdx, regionIdx)
+                    + (oilEnergy+hVapO)*Rv*bg*referenceDensity(oilPhaseIdx, regionIdx);
+            }
+            if (enableVaporizedWater()) {
+                // gas containing vaporized water
+                const LhsEval Rv(0.0);
+                const LhsEval& Rvw = BlackOil::template getRvw_<ThisType, FluidState, LhsEval>(fluidState, regionIdx);
+                const LhsEval& bg = gasPvt_->inverseFormationVolumeFactor(regionIdx, T, p, Rv, Rvw);
+                const auto waterEnergy =
+                    waterPvt_->internalEnergy(regionIdx, T, p,
+                                              BlackOil::template getRsw_<ThisType, FluidState, LhsEval>(fluidState, regionIdx),
+                                              BlackOil::template getSaltConcentration_<ThisType, FluidState, LhsEval>(fluidState, regionIdx));
+                const auto hVapW = waterPvt_->hVap(regionIdx);
+                return
+                    gasEnergy*bg*referenceDensity(gasPhaseIdx, regionIdx)
+                    + (waterEnergy+hVapW)*Rvw*bg*referenceDensity(waterPhaseIdx, regionIdx);
+            }
+
+            // immiscible gas
+            const LhsEval Rv(0.0);
+            const LhsEval Rvw(0.0);
+            const auto& bg = gasPvt_->inverseFormationVolumeFactor(regionIdx, T, p, Rv, Rvw);
+            return gasEnergy*bg*referenceDensity(phaseIdx, regionIdx);
+        }
+
+        case waterPhaseIdx:
+            const auto waterEnergy =
+                waterPvt_->internalEnergy(regionIdx, T, p,
+                                          BlackOil::template getRsw_<ThisType, FluidState, LhsEval>(fluidState, regionIdx),
+                                          BlackOil::template getSaltConcentration_<ThisType, FluidState, LhsEval>(fluidState, regionIdx));
+            assert(waterPvt_->mixingEnergy());
+            if (enableDissolvedGasInWater()) {
+                const auto& gasEnergy =
+                    gasPvt_->internalEnergy(regionIdx, T, p,
+                                            BlackOil::template getRv_<ThisType, FluidState, LhsEval>(fluidState, regionIdx),
+                                            BlackOil::template getRvw_<ThisType, FluidState, LhsEval>(fluidState, regionIdx));
+                // gas miscible in water
+                const LhsEval& Rsw = saturatedDissolutionFactor<FluidState, LhsEval>(fluidState, waterPhaseIdx, regionIdx);
+                const LhsEval& bw = waterPvt_->inverseFormationVolumeFactor(regionIdx, T, p, Rsw, saltConcentration);
+                return
+                    waterEnergy*bw*referenceDensity(waterPhaseIdx, regionIdx)
+                    + gasEnergy*Rsw*bw*referenceDensity(gasPhaseIdx, regionIdx);
+            }
+            const LhsEval Rsw(0.0);
+            return
+                waterEnergy*referenceDensity(waterPhaseIdx, regionIdx)
+                * waterPvt_->inverseFormationVolumeFactor(regionIdx, T, p, Rsw, saltConcentration);
+        }
+        throw std::logic_error("Unhandled phase index " + std::to_string(phaseIdx));
+    }
+
+
+
     //! \copydoc BaseFluidSystem::enthalpy
     template <class FluidState, class LhsEval = typename FluidState::Scalar>
     static LhsEval enthalpy(const FluidState& fluidState,
                             unsigned phaseIdx,
                             unsigned regionIdx)
     {
-        assert(phaseIdx <= numPhases);
-        assert(regionIdx <= numRegions());
-
+        // should preferably not be used values should be taken from intensive quantities fluid state.
         const auto& p = decay<LhsEval>(fluidState.pressure(phaseIdx));
-        const auto& T = decay<LhsEval>(fluidState.temperature(phaseIdx));
-
-        switch (phaseIdx) {
-        case oilPhaseIdx:
-            return
-                oilPvt_->internalEnergy(regionIdx, T, p, BlackOil::template getRs_<ThisType, FluidState, LhsEval>(fluidState, regionIdx))
-                + p/density<FluidState, LhsEval>(fluidState, phaseIdx, regionIdx);
-
-        case gasPhaseIdx:
-            return
-                 gasPvt_->internalEnergy(regionIdx, T, p,
-                 BlackOil::template getRv_<ThisType, FluidState, LhsEval>(fluidState, regionIdx),
-                  BlackOil::template getRvw_<ThisType, FluidState, LhsEval>(fluidState, regionIdx))
-                  + p/density<FluidState, LhsEval>(fluidState, phaseIdx, regionIdx);
-
-        case waterPhaseIdx:
-            return
-                waterPvt_->internalEnergy(regionIdx, T, p,
-                                          BlackOil::template getRsw_<ThisType, FluidState, LhsEval>(fluidState, regionIdx),
-                                          BlackOil::template getSaltConcentration_<ThisType, FluidState, LhsEval>(fluidState, regionIdx))
-                + p/density<FluidState, LhsEval>(fluidState, phaseIdx, regionIdx);
-
-        default: throw std::logic_error("Unhandled phase index "+std::to_string(phaseIdx));
+        auto energy = internalEnergy<FluidState, LhsEval>(fluidState, phaseIdx, regionIdx);
+        if(!enthalpy_eq_energy_){
+            // used for simplified models
+            energy += p/density<FluidState, LhsEval>(fluidState, phaseIdx, regionIdx);
         }
-
-        throw std::logic_error("Unhandled phase index "+std::to_string(phaseIdx));
+        return energy;
     }
 
     /*!
@@ -1153,7 +1329,7 @@ public:
 
         const auto& p = decay<LhsEval>(fluidState.pressure(phaseIdx));
         const auto& T = decay<LhsEval>(fluidState.temperature(phaseIdx));
-        const auto& So = decay<LhsEval>(fluidState.saturation(oilPhaseIdx));
+        const auto& So = (phaseIdx == waterPhaseIdx) ? 0 : decay<LhsEval>(fluidState.saturation(oilPhaseIdx));
 
         switch (phaseIdx) {
         case oilPhaseIdx: return oilPvt_->saturatedGasDissolutionFactor(regionIdx, T, p, So, maxOilSaturation);
@@ -1514,6 +1690,13 @@ public:
         default: throw std::logic_error("Unhandled phase index "+std::to_string(phaseIdx));
         }
     }
+    static void setEnergyEqualEnthalpy(bool enthalpy_eq_energy){
+        enthalpy_eq_energy_ = enthalpy_eq_energy;
+    }
+
+    static bool enthalpyEqualEnergy(){
+        return enthalpy_eq_energy_;
+    }
 
 private:
     static void resizeArrays_(std::size_t numRegions);
@@ -1541,74 +1724,38 @@ private:
     static std::array<short, numPhases> canonicalToActivePhaseIdx_;
 
     static bool isInitialized_;
+    static bool useSaturatedTables_;
+    inline static bool enthalpy_eq_energy_ = false;
 };
 
-template <class Scalar, class IndexTraits>
-unsigned char BlackOilFluidSystem<Scalar, IndexTraits>::numActivePhases_;
+template <typename T> using BOFS = BlackOilFluidSystem<T, BlackOilDefaultIndexTraits>;
 
-template <class Scalar, class IndexTraits>
-std::array<bool, BlackOilFluidSystem<Scalar, IndexTraits>::numPhases> BlackOilFluidSystem<Scalar, IndexTraits>::phaseIsActive_;
+#define DECLARE_INSTANCE(T) \
+template<> unsigned char BOFS<T>::numActivePhases_; \
+template<> std::array<bool, BOFS<T>::numPhases> BOFS<T>::phaseIsActive_; \
+template<> std::array<short, BOFS<T>::numPhases> BOFS<T>::activeToCanonicalPhaseIdx_; \
+template<> std::array<short, BOFS<T>::numPhases> BOFS<T>::canonicalToActivePhaseIdx_; \
+template<> T BOFS<T>::surfaceTemperature; \
+template<> T BOFS<T>::surfacePressure; \
+template<> T BOFS<T>::reservoirTemperature_; \
+template<> bool BOFS<T>::enableDissolvedGas_; \
+template<> bool BOFS<T>::enableDissolvedGasInWater_; \
+template<> bool BOFS<T>::enableVaporizedOil_; \
+template<> bool BOFS<T>::enableVaporizedWater_; \
+template<> bool BOFS<T>::enableDiffusion_; \
+template<> std::shared_ptr<OilPvtMultiplexer<T>> BOFS<T>::oilPvt_; \
+template<> std::shared_ptr<GasPvtMultiplexer<T>> BOFS<T>::gasPvt_; \
+template<> std::shared_ptr<WaterPvtMultiplexer<T>> BOFS<T>::waterPvt_; \
+template<> std::vector<std::array<T, 3>> BOFS<T>::referenceDensity_; \
+template<> std::vector<std::array<T, 3>> BOFS<T>::molarMass_; \
+template<> std::vector<std::array<T, 9>> BOFS<T>::diffusionCoefficients_; \
+template<> bool BOFS<T>::isInitialized_; \
+template<> bool BOFS<T>::useSaturatedTables_;
 
-template <class Scalar, class IndexTraits>
-std::array<short, BlackOilFluidSystem<Scalar, IndexTraits>::numPhases> BlackOilFluidSystem<Scalar, IndexTraits>::activeToCanonicalPhaseIdx_;
+DECLARE_INSTANCE(float)
+DECLARE_INSTANCE(double)
 
-template <class Scalar, class IndexTraits>
-std::array<short, BlackOilFluidSystem<Scalar, IndexTraits>::numPhases> BlackOilFluidSystem<Scalar, IndexTraits>::canonicalToActivePhaseIdx_;
-
-template <class Scalar, class IndexTraits>
-Scalar
-BlackOilFluidSystem<Scalar, IndexTraits>::surfaceTemperature; // [K]
-
-template <class Scalar, class IndexTraits>
-Scalar
-BlackOilFluidSystem<Scalar, IndexTraits>::surfacePressure; // [Pa]
-
-template <class Scalar, class IndexTraits>
-Scalar
-BlackOilFluidSystem<Scalar, IndexTraits>::reservoirTemperature_;
-
-template <class Scalar, class IndexTraits>
-bool BlackOilFluidSystem<Scalar, IndexTraits>::enableDissolvedGas_;
-
-template <class Scalar, class IndexTraits>
-bool BlackOilFluidSystem<Scalar, IndexTraits>::enableDissolvedGasInWater_;
-
-
-template <class Scalar, class IndexTraits>
-bool BlackOilFluidSystem<Scalar, IndexTraits>::enableVaporizedOil_;
-
-template <class Scalar, class IndexTraits>
-bool BlackOilFluidSystem<Scalar, IndexTraits>::enableVaporizedWater_;
-
-template <class Scalar, class IndexTraits>
-bool BlackOilFluidSystem<Scalar, IndexTraits>::enableDiffusion_;
-
-template <class Scalar, class IndexTraits>
-std::shared_ptr<OilPvtMultiplexer<Scalar> >
-BlackOilFluidSystem<Scalar, IndexTraits>::oilPvt_;
-
-template <class Scalar, class IndexTraits>
-std::shared_ptr<GasPvtMultiplexer<Scalar> >
-BlackOilFluidSystem<Scalar, IndexTraits>::gasPvt_;
-
-template <class Scalar, class IndexTraits>
-std::shared_ptr<WaterPvtMultiplexer<Scalar> >
-BlackOilFluidSystem<Scalar, IndexTraits>::waterPvt_;
-
-template <class Scalar, class IndexTraits>
-std::vector<std::array<Scalar, 3> >
-BlackOilFluidSystem<Scalar, IndexTraits>::referenceDensity_;
-
-template <class Scalar, class IndexTraits>
-std::vector<std::array<Scalar, 3> >
-BlackOilFluidSystem<Scalar, IndexTraits>::molarMass_;
-
-template <class Scalar, class IndexTraits>
-std::vector<std::array<Scalar, 9> >
-BlackOilFluidSystem<Scalar, IndexTraits>::diffusionCoefficients_;
-
-template <class Scalar, class IndexTraits>
-bool BlackOilFluidSystem<Scalar, IndexTraits>::isInitialized_ = false;
+#undef DECLARE_INSTANCE
 
 } // namespace Opm
 
