@@ -39,6 +39,7 @@
 #include <opm/material/common/Valgrind.hpp>
 
 #include <cassert>
+#include <string_view>
 
 namespace Opm {
 
@@ -78,9 +79,9 @@ public:
     static const int gasPhaseIdx = 1;
 
     //! \copydoc BaseFluidSystem::phaseName
-    static const char* phaseName(unsigned phaseIdx)
+    static std::string_view phaseName(unsigned phaseIdx)
     {
-        static const char* name[] = {
+        static const std::string_view name[] = {
             "liquid",
             "gas"
         };
@@ -149,9 +150,9 @@ public:
     typedef SimpleN2 N2;
 
     //! \copydoc BaseFluidSystem::componentName
-    static const char* componentName(unsigned compIdx)
+    static std::string_view componentName(unsigned compIdx)
     {
-        static const char* name[] = {
+        static const std::string_view name[] = {
             H2O::name(),
             N2::name()
         };
