@@ -27,12 +27,13 @@
 #ifndef OPM_GAS_PVT_MULTIPLEXER_HPP
 #define OPM_GAS_PVT_MULTIPLEXER_HPP
 
-#include "DryGasPvt.hpp"
-#include "DryHumidGasPvt.hpp"
-#include "WetHumidGasPvt.hpp"
-#include "WetGasPvt.hpp"
-#include "GasPvtThermal.hpp"
-#include "Co2GasPvt.hpp"
+#include <opm/material/fluidsystems/blackoilpvt/Co2GasPvt.hpp>
+#include <opm/material/fluidsystems/blackoilpvt/DryGasPvt.hpp>
+#include <opm/material/fluidsystems/blackoilpvt/DryHumidGasPvt.hpp>
+#include <opm/material/fluidsystems/blackoilpvt/GasPvtThermal.hpp>
+#include <opm/material/fluidsystems/blackoilpvt/H2GasPvt.hpp>
+#include <opm/material/fluidsystems/blackoilpvt/WetGasPvt.hpp>
+#include <opm/material/fluidsystems/blackoilpvt/WetHumidGasPvt.hpp>
 
 namespace Opm {
 
@@ -41,38 +42,44 @@ class EclipseState;
 class Schedule;
 #endif
 
-#define OPM_GAS_PVT_MULTIPLEXER_CALL(codeToCall)                          \
+#define OPM_GAS_PVT_MULTIPLEXER_CALL(codeToCall, ...)                     \
     switch (gasPvtApproach_) {                                            \
     case GasPvtApproach::DryGas: {                                        \
         auto& pvtImpl = getRealPvt<GasPvtApproach::DryGas>();             \
         codeToCall;                                                       \
-        break;                                                            \
+        __VA_ARGS__;                                                      \
     }                                                                     \
     case GasPvtApproach::DryHumidGas: {                                   \
         auto& pvtImpl = getRealPvt<GasPvtApproach::DryHumidGas>();        \
         codeToCall;                                                       \
-        break;                                                            \
+        __VA_ARGS__;                                                      \
     }                                                                     \
     case GasPvtApproach::WetHumidGas: {                                   \
         auto& pvtImpl = getRealPvt<GasPvtApproach::WetHumidGas>();        \
         codeToCall;                                                       \
-        break;                                                            \
+        __VA_ARGS__;                                                      \
     }                                                                     \
     case GasPvtApproach::WetGas: {                                        \
         auto& pvtImpl = getRealPvt<GasPvtApproach::WetGas>();             \
         codeToCall;                                                       \
-        break;                                                            \
+        __VA_ARGS__;                                                      \
     }                                                                     \
     case GasPvtApproach::ThermalGas: {                                    \
         auto& pvtImpl = getRealPvt<GasPvtApproach::ThermalGas>();         \
         codeToCall;                                                       \
-        break;                                                            \
+        __VA_ARGS__;                                                      \
     }                                                                     \
     case GasPvtApproach::Co2Gas: {                                        \
         auto& pvtImpl = getRealPvt<GasPvtApproach::Co2Gas>();             \
         codeToCall;                                                       \
-        break;                                                            \
+        __VA_ARGS__;                                                      \
     }                                                                     \
+    case GasPvtApproach::H2Gas: {                                         \
+        auto& pvtImpl = getRealPvt<GasPvtApproach::H2Gas>();              \
+        codeToCall;                                                       \
+        __VA_ARGS__;                                                      \
+    }                                                                     \
+    default:                                                              \
     case GasPvtApproach::NoGas:                                           \
         throw std::logic_error("Not implemented: Gas PVT of this deck!"); \
     }
@@ -84,7 +91,8 @@ enum class GasPvtApproach {
     WetHumidGas,
     WetGas,
     ThermalGas,
-    Co2Gas
+    Co2Gas,
+    H2Gas
 };
 
 /*!
@@ -102,9 +110,9 @@ class GasPvtMultiplexer
 {
 public:
     GasPvtMultiplexer()
+        : gasPvtApproach_(GasPvtApproach::NoGas)
+        , realGasPvt_(nullptr)
     {
-        gasPvtApproach_ = GasPvtApproach::NoGas;
-        realGasPvt_ = nullptr;
     }
 
     GasPvtMultiplexer(GasPvtApproach approach, void* realGasPvt)
@@ -117,36 +125,11 @@ public:
         *this = data;
     }
 
-    ~GasPvtMultiplexer()
+    ~GasPvtMultiplexer();
+
+    bool mixingEnergy() const
     {
-        switch (gasPvtApproach_) {
-        case GasPvtApproach::DryGas: {
-            delete &getRealPvt<GasPvtApproach::DryGas>();
-            break;
-        }
-        case GasPvtApproach::DryHumidGas: {
-            delete &getRealPvt<GasPvtApproach::DryHumidGas>();
-            break;
-        }
-        case GasPvtApproach::WetHumidGas: {
-            delete &getRealPvt<GasPvtApproach::WetHumidGas>();
-            break;
-        }
-        case GasPvtApproach::WetGas: {
-            delete &getRealPvt<GasPvtApproach::WetGas>();
-            break;
-        }
-        case GasPvtApproach::ThermalGas: {
-            delete &getRealPvt<GasPvtApproach::ThermalGas>();
-            break;
-        }
-        case GasPvtApproach::Co2Gas: {
-            delete &getRealPvt<GasPvtApproach::Co2Gas>();
-            break;
-        }
-        case GasPvtApproach::NoGas:
-            break;
-        }
+        return gasPvtApproach_ == GasPvtApproach::ThermalGas;
     }
 
 #if HAVE_ECL_INPUT
@@ -158,54 +141,21 @@ public:
     void initFromState(const EclipseState& eclState, const Schedule& schedule);
 #endif // HAVE_ECL_INPUT
 
-    void setApproach(GasPvtApproach gasPvtAppr)
-    {
-        switch (gasPvtAppr) {
-        case GasPvtApproach::DryGas:
-            realGasPvt_ = new DryGasPvt<Scalar>;
-            break;
+    void setApproach(GasPvtApproach gasPvtAppr);
 
-        case GasPvtApproach::DryHumidGas:
-            realGasPvt_ = new DryHumidGasPvt<Scalar>;
-            break;
-        
-        case GasPvtApproach::WetHumidGas:
-            realGasPvt_ = new WetHumidGasPvt<Scalar>;
-            break;
-
-        case GasPvtApproach::WetGas:
-            realGasPvt_ = new WetGasPvt<Scalar>;
-            break;
-
-        case GasPvtApproach::ThermalGas:
-            realGasPvt_ = new GasPvtThermal<Scalar>;
-            break;
-
-        case GasPvtApproach::Co2Gas:
-            realGasPvt_ = new Co2GasPvt<Scalar>;
-            break;
-
-        case GasPvtApproach::NoGas:
-            throw std::logic_error("Not implemented: Gas PVT of this deck!");
-        }
-
-        gasPvtApproach_ = gasPvtAppr;
-    }
-
-    void initEnd()
-    { OPM_GAS_PVT_MULTIPLEXER_CALL(pvtImpl.initEnd()); }
+    void initEnd();
 
     /*!
      * \brief Return the number of PVT regions which are considered by this PVT-object.
      */
-    unsigned numRegions() const
-    { OPM_GAS_PVT_MULTIPLEXER_CALL(return pvtImpl.numRegions()); return 1; }
+    unsigned numRegions() const;
+
+    void setVapPars(const Scalar par1, const Scalar par2);
 
     /*!
      * \brief Return the reference density which are considered by this PVT-object.
      */
-    const Scalar gasReferenceDensity(unsigned regionIdx)
-    { OPM_GAS_PVT_MULTIPLEXER_CALL(return pvtImpl.gasReferenceDensity(regionIdx)); return 2.; }
+    Scalar gasReferenceDensity(unsigned regionIdx);
 
     /*!
      * \brief Returns the specific enthalpy [J/kg] of gas given a set of parameters.
@@ -216,7 +166,9 @@ public:
                         const Evaluation& pressure,
                         const Evaluation& Rv,
                         const Evaluation& Rvw) const
-    { OPM_GAS_PVT_MULTIPLEXER_CALL(return pvtImpl.internalEnergy(regionIdx, temperature, pressure, Rv, Rvw)); return 0; }
+    { OPM_GAS_PVT_MULTIPLEXER_CALL(return pvtImpl.internalEnergy(regionIdx, temperature, pressure, Rv, Rvw)); }
+
+    Scalar hVap(unsigned regionIdx) const;
 
     /*!
      * \brief Returns the dynamic viscosity [Pa s] of the fluid phase given a set of parameters.
@@ -227,7 +179,7 @@ public:
                          const Evaluation& pressure,
                          const Evaluation& Rv,
                          const Evaluation& Rvw ) const
-    { OPM_GAS_PVT_MULTIPLEXER_CALL(return pvtImpl.viscosity(regionIdx, temperature, pressure, Rv, Rvw)); return 0; }
+    { OPM_GAS_PVT_MULTIPLEXER_CALL(return pvtImpl.viscosity(regionIdx, temperature, pressure, Rv, Rvw)); }
 
     /*!
      * \brief Returns the dynamic viscosity [Pa s] of oil saturated gas given a set of parameters.
@@ -236,7 +188,7 @@ public:
     Evaluation saturatedViscosity(unsigned regionIdx,
                                   const Evaluation& temperature,
                                   const Evaluation& pressure) const
-    { OPM_GAS_PVT_MULTIPLEXER_CALL(return pvtImpl.saturatedViscosity(regionIdx, temperature, pressure)); return 0; }
+    { OPM_GAS_PVT_MULTIPLEXER_CALL(return pvtImpl.saturatedViscosity(regionIdx, temperature, pressure)); }
 
     /*!
      * \brief Returns the formation volume factor [-] of the fluid phase.
@@ -247,7 +199,7 @@ public:
                                             const Evaluation& pressure,
                                             const Evaluation& Rv,
                                             const Evaluation& Rvw) const
-    { OPM_GAS_PVT_MULTIPLEXER_CALL(return pvtImpl.inverseFormationVolumeFactor(regionIdx, temperature, pressure, Rv, Rvw)); return 0; }
+    { OPM_GAS_PVT_MULTIPLEXER_CALL(return pvtImpl.inverseFormationVolumeFactor(regionIdx, temperature, pressure, Rv, Rvw)); }
 
     /*!
      * \brief Returns the formation volume factor [-] of oil saturated gas given a set of parameters.
@@ -256,7 +208,7 @@ public:
     Evaluation saturatedInverseFormationVolumeFactor(unsigned regionIdx,
                                                      const Evaluation& temperature,
                                                      const Evaluation& pressure) const
-    { OPM_GAS_PVT_MULTIPLEXER_CALL(return pvtImpl.saturatedInverseFormationVolumeFactor(regionIdx, temperature, pressure)); return 0; }
+    { OPM_GAS_PVT_MULTIPLEXER_CALL(return pvtImpl.saturatedInverseFormationVolumeFactor(regionIdx, temperature, pressure)); }
 
     /*!
      * \brief Returns the oil vaporization factor \f$R_v\f$ [m^3/m^3] of oil saturated gas.
@@ -265,7 +217,7 @@ public:
     Evaluation saturatedOilVaporizationFactor(unsigned regionIdx,
                                               const Evaluation& temperature,
                                               const Evaluation& pressure) const
-    { OPM_GAS_PVT_MULTIPLEXER_CALL(return pvtImpl.saturatedOilVaporizationFactor(regionIdx, temperature, pressure)); return 0; }
+    { OPM_GAS_PVT_MULTIPLEXER_CALL(return pvtImpl.saturatedOilVaporizationFactor(regionIdx, temperature, pressure)); }
 
     /*!
      * \brief Returns the oil vaporization factor \f$R_v\f$ [m^3/m^3] of oil saturated gas.
@@ -276,7 +228,7 @@ public:
                                               const Evaluation& pressure,
                                               const Evaluation& oilSaturation,
                                               const Evaluation& maxOilSaturation) const
-    { OPM_GAS_PVT_MULTIPLEXER_CALL(return pvtImpl.saturatedOilVaporizationFactor(regionIdx, temperature, pressure, oilSaturation, maxOilSaturation)); return 0; }
+    { OPM_GAS_PVT_MULTIPLEXER_CALL(return pvtImpl.saturatedOilVaporizationFactor(regionIdx, temperature, pressure, oilSaturation, maxOilSaturation)); }
 
     /*!
      * \brief Returns the water vaporization factor \f$R_vw\f$ [m^3/m^3] of water saturated gas.
@@ -285,17 +237,17 @@ public:
     Evaluation saturatedWaterVaporizationFactor(unsigned regionIdx,
                                               const Evaluation& temperature,
                                               const Evaluation& pressure) const
-    { OPM_GAS_PVT_MULTIPLEXER_CALL(return pvtImpl.saturatedWaterVaporizationFactor(regionIdx, temperature, pressure)); return 0; }
-    
+    { OPM_GAS_PVT_MULTIPLEXER_CALL(return pvtImpl.saturatedWaterVaporizationFactor(regionIdx, temperature, pressure)); }
+
     /*!
      * \brief Returns the water vaporization factor \f$R_vw\f$ [m^3/m^3] of water saturated gas.
      */
     template <class Evaluation = Scalar>
     Evaluation saturatedWaterVaporizationFactor(unsigned regionIdx,
                                               const Evaluation& temperature,
-                                              const Evaluation& pressure, 
+                                              const Evaluation& pressure,
                                               const Evaluation& saltConcentration) const
-    { OPM_GAS_PVT_MULTIPLEXER_CALL(return pvtImpl.saturatedWaterVaporizationFactor(regionIdx, temperature, pressure, saltConcentration)); return 0; }
+    { OPM_GAS_PVT_MULTIPLEXER_CALL(return pvtImpl.saturatedWaterVaporizationFactor(regionIdx, temperature, pressure, saltConcentration)); }
 
     /*!
      * \brief Returns the saturation pressure of the gas phase [Pa]
@@ -307,7 +259,7 @@ public:
     Evaluation saturationPressure(unsigned regionIdx,
                                   const Evaluation& temperature,
                                   const Evaluation& Rv) const
-    { OPM_GAS_PVT_MULTIPLEXER_CALL(return pvtImpl.saturationPressure(regionIdx, temperature, Rv)); return 0; }
+    { OPM_GAS_PVT_MULTIPLEXER_CALL(return pvtImpl.saturationPressure(regionIdx, temperature, Rv)); }
 
     /*!
      * \copydoc BaseFluidSystem::diffusionCoefficient
@@ -317,7 +269,7 @@ public:
                                     const Evaluation& pressure,
                                     unsigned compIdx) const
     {
-      OPM_GAS_PVT_MULTIPLEXER_CALL(return pvtImpl.diffusionCoefficient(temperature, pressure, compIdx)); return 0;
+      OPM_GAS_PVT_MULTIPLEXER_CALL(return pvtImpl.diffusionCoefficient(temperature, pressure, compIdx));
     }
 
     /*!
@@ -416,40 +368,28 @@ public:
         return *static_cast<const Co2GasPvt<Scalar>* >(realGasPvt_);
     }
 
-    const void* realGasPvt() const { return realGasPvt_; }
-
-    GasPvtMultiplexer<Scalar,enableThermal>& operator=(const GasPvtMultiplexer<Scalar,enableThermal>& data)
+    template <GasPvtApproach approachV>
+    typename std::enable_if<approachV == GasPvtApproach::H2Gas, H2GasPvt<Scalar> >::type& getRealPvt()
     {
-        gasPvtApproach_ = data.gasPvtApproach_;
-        switch (gasPvtApproach_) {
-        case GasPvtApproach::DryGas:
-            realGasPvt_ = new DryGasPvt<Scalar>(*static_cast<const DryGasPvt<Scalar>*>(data.realGasPvt_));
-            break;
-        case GasPvtApproach::DryHumidGas:
-            realGasPvt_ = new DryHumidGasPvt<Scalar>(*static_cast<const DryHumidGasPvt<Scalar>*>(data.realGasPvt_));
-            break;
-        case GasPvtApproach::WetHumidGas:
-            realGasPvt_ = new WetHumidGasPvt<Scalar>(*static_cast<const WetHumidGasPvt<Scalar>*>(data.realGasPvt_));
-            break;
-        case GasPvtApproach::WetGas:
-            realGasPvt_ = new WetGasPvt<Scalar>(*static_cast<const WetGasPvt<Scalar>*>(data.realGasPvt_));
-            break;
-        case GasPvtApproach::ThermalGas:
-            realGasPvt_ = new GasPvtThermal<Scalar>(*static_cast<const GasPvtThermal<Scalar>*>(data.realGasPvt_));
-            break;
-        case GasPvtApproach::Co2Gas:
-            realGasPvt_ = new Co2GasPvt<Scalar>(*static_cast<const Co2GasPvt<Scalar>*>(data.realGasPvt_));
-            break;
-        default:
-            break;
-        }
-
-        return *this;
+        assert(gasPvtApproach() == approachV);
+        return *static_cast<H2GasPvt<Scalar>* >(realGasPvt_);
     }
 
+    template <GasPvtApproach approachV>
+    typename std::enable_if<approachV == GasPvtApproach::H2Gas, const H2GasPvt<Scalar> >::type& getRealPvt() const
+    {
+        assert(gasPvtApproach() == approachV);
+        return *static_cast<const H2GasPvt<Scalar>* >(realGasPvt_);
+    }
+
+    const void* realGasPvt() const { return realGasPvt_; }
+
+    GasPvtMultiplexer<Scalar,enableThermal>&
+    operator=(const GasPvtMultiplexer<Scalar,enableThermal>& data);
+
 private:
-    GasPvtApproach gasPvtApproach_;
-    void* realGasPvt_;
+    GasPvtApproach gasPvtApproach_{GasPvtApproach::NoGas};
+    void* realGasPvt_{nullptr};
 };
 
 } // namespace Opm
