@@ -27,12 +27,12 @@
 #ifndef OPM_OIL_PVT_MULTIPLEXER_HPP
 #define OPM_OIL_PVT_MULTIPLEXER_HPP
 
-#include "ConstantCompressibilityOilPvt.hpp"
-#include "DeadOilPvt.hpp"
-#include "LiveOilPvt.hpp"
-#include "OilPvtThermal.hpp"
-#include "BrineCo2Pvt.hpp"
-#include "BrineH2Pvt.hpp"
+#include <opm/material/fluidsystems/blackoilpvt/BrineCo2Pvt.hpp>
+#include <opm/material/fluidsystems/blackoilpvt/BrineH2Pvt.hpp>
+#include <opm/material/fluidsystems/blackoilpvt/ConstantCompressibilityOilPvt.hpp>
+#include <opm/material/fluidsystems/blackoilpvt/DeadOilPvt.hpp>
+#include <opm/material/fluidsystems/blackoilpvt/LiveOilPvt.hpp>
+#include <opm/material/fluidsystems/blackoilpvt/OilPvtThermal.hpp>
 
 namespace Opm {
 
@@ -41,38 +41,39 @@ class EclipseState;
 class Schedule;
 #endif
 
-#define OPM_OIL_PVT_MULTIPLEXER_CALL(codeToCall)                                  \
+#define OPM_OIL_PVT_MULTIPLEXER_CALL(codeToCall, ...)                             \
     switch (approach_) {                                                          \
     case OilPvtApproach::ConstantCompressibilityOil: {                            \
         auto& pvtImpl = getRealPvt<OilPvtApproach::ConstantCompressibilityOil>(); \
         codeToCall;                                                               \
-        break;                                                                    \
+        __VA_ARGS__;                                                              \
     }                                                                             \
     case OilPvtApproach::DeadOil: {                                               \
         auto& pvtImpl = getRealPvt<OilPvtApproach::DeadOil>();                    \
         codeToCall;                                                               \
-        break;                                                                    \
+        __VA_ARGS__;                                                              \
     }                                                                             \
     case OilPvtApproach::LiveOil: {                                               \
         auto& pvtImpl = getRealPvt<OilPvtApproach::LiveOil>();                    \
         codeToCall;                                                               \
-        break;                                                                    \
+        __VA_ARGS__;                                                              \
     }                                                                             \
     case OilPvtApproach::ThermalOil: {                                            \
         auto& pvtImpl = getRealPvt<OilPvtApproach::ThermalOil>();                 \
         codeToCall;                                                               \
-        break;                                                                    \
+        __VA_ARGS__;                                                              \
     }                                                                             \
     case OilPvtApproach::BrineCo2: {                                              \
         auto& pvtImpl = getRealPvt<OilPvtApproach::BrineCo2>();                   \
         codeToCall;                                                               \
-        break;                                                                    \
+        __VA_ARGS__;                                                              \
     }                                                                             \
     case OilPvtApproach::BrineH2: {                                               \
         auto& pvtImpl = getRealPvt<OilPvtApproach::BrineH2>();                    \
         codeToCall;                                                               \
-        break;                                                                    \
+        __VA_ARGS__;                                                              \
     }                                                                             \
+    default:                                                                      \
     case OilPvtApproach::NoOil:                                                   \
         throw std::logic_error("Not implemented: Oil PVT of this deck!");         \
     }
@@ -104,9 +105,9 @@ class OilPvtMultiplexer
 {
 public:
     OilPvtMultiplexer()
+        : approach_(OilPvtApproach::NoOil)
+        , realOilPvt_(nullptr)
     {
-        approach_ = OilPvtApproach::NoOil;
-        realOilPvt_ = nullptr;
     }
 
     OilPvtMultiplexer(OilPvtApproach approach, void* realOilPvt)
@@ -119,38 +120,12 @@ public:
         *this = data;
     }
 
-    ~OilPvtMultiplexer()
-    {
-        switch (approach_) {
-        case OilPvtApproach::LiveOil: {
-            delete &getRealPvt<OilPvtApproach::LiveOil>();
-            break;
-        }
-        case OilPvtApproach::DeadOil: {
-            delete &getRealPvt<OilPvtApproach::DeadOil>();
-            break;
-        }
-        case OilPvtApproach::ConstantCompressibilityOil: {
-            delete &getRealPvt<OilPvtApproach::ConstantCompressibilityOil>();
-            break;
-        }
-        case OilPvtApproach::ThermalOil: {
-            delete &getRealPvt<OilPvtApproach::ThermalOil>();
-            break;
-        }
-        case OilPvtApproach::BrineCo2: {
-            delete &getRealPvt<OilPvtApproach::BrineCo2>();
-            break;
-        }
-	    case OilPvtApproach::BrineH2: {
-            delete &getRealPvt<OilPvtApproach::BrineH2>();
-            break;
-        }
-        case OilPvtApproach::NoOil:
-            break;
-        }
-    }
+    ~OilPvtMultiplexer();
 
+    bool mixingEnergy() const
+    {
+        return approach_ == OilPvtApproach::ThermalOil;
+    }
 #if HAVE_ECL_INPUT
     /*!
      * \brief Initialize the parameters for water using an ECL state.
@@ -161,25 +136,19 @@ public:
 #endif // HAVE_ECL_INPUT
 
 
-    void initEnd()
-    { OPM_OIL_PVT_MULTIPLEXER_CALL(pvtImpl.initEnd()); }
+    void initEnd();
 
     /*!
      * \brief Return the number of PVT regions which are considered by this PVT-object.
      */
-    unsigned numRegions() const
-    { OPM_OIL_PVT_MULTIPLEXER_CALL(return pvtImpl.numRegions()); return 1; }
+    unsigned numRegions() const;
 
-    void setVapPars(const Scalar par1, const Scalar par2)
-    {
-        OPM_OIL_PVT_MULTIPLEXER_CALL(pvtImpl.setVapPars(par1, par2));
-    }
+    void setVapPars(const Scalar par1, const Scalar par2);
 
     /*!
      * \brief Return the reference density which are considered by this PVT-object.
      */
-    const Scalar oilReferenceDensity(unsigned regionIdx) const
-    { OPM_OIL_PVT_MULTIPLEXER_CALL(return pvtImpl.oilReferenceDensity(regionIdx)); return 700.; }
+    Scalar oilReferenceDensity(unsigned regionIdx) const;
 
     /*!
      * \brief Returns the specific enthalpy [J/kg] oil given a set of parameters.
@@ -189,8 +158,10 @@ public:
                         const Evaluation& temperature,
                         const Evaluation& pressure,
                         const Evaluation& Rs) const
-    { OPM_OIL_PVT_MULTIPLEXER_CALL(return pvtImpl.internalEnergy(regionIdx, temperature, pressure, Rs)); return 0; }
+    { OPM_OIL_PVT_MULTIPLEXER_CALL(return pvtImpl.internalEnergy(regionIdx, temperature, pressure, Rs)); }
 
+    Scalar hVap(unsigned regionIdx) const
+    { OPM_OIL_PVT_MULTIPLEXER_CALL(return pvtImpl.hVap(regionIdx)); }
     /*!
      * \brief Returns the dynamic viscosity [Pa s] of the fluid phase given a set of parameters.
      */
@@ -199,7 +170,7 @@ public:
                          const Evaluation& temperature,
                          const Evaluation& pressure,
                          const Evaluation& Rs) const
-    { OPM_OIL_PVT_MULTIPLEXER_CALL(return pvtImpl.viscosity(regionIdx, temperature, pressure, Rs)); return 0; }
+    { OPM_OIL_PVT_MULTIPLEXER_CALL(return pvtImpl.viscosity(regionIdx, temperature, pressure, Rs)); }
 
     /*!
      * \brief Returns the dynamic viscosity [Pa s] of the fluid phase given a set of parameters.
@@ -208,7 +179,7 @@ public:
     Evaluation saturatedViscosity(unsigned regionIdx,
                                   const Evaluation& temperature,
                                   const Evaluation& pressure) const
-    { OPM_OIL_PVT_MULTIPLEXER_CALL(return pvtImpl.saturatedViscosity(regionIdx, temperature, pressure)); return 0; }
+    { OPM_OIL_PVT_MULTIPLEXER_CALL(return pvtImpl.saturatedViscosity(regionIdx, temperature, pressure)); }
 
     /*!
      * \brief Returns the formation volume factor [-] of the fluid phase.
@@ -218,7 +189,7 @@ public:
                                             const Evaluation& temperature,
                                             const Evaluation& pressure,
                                             const Evaluation& Rs) const
-    { OPM_OIL_PVT_MULTIPLEXER_CALL(return pvtImpl.inverseFormationVolumeFactor(regionIdx, temperature, pressure, Rs)); return 0; }
+    { OPM_OIL_PVT_MULTIPLEXER_CALL(return pvtImpl.inverseFormationVolumeFactor(regionIdx, temperature, pressure, Rs)); }
 
     /*!
      * \brief Returns the formation volume factor [-] of the fluid phase.
@@ -227,7 +198,7 @@ public:
     Evaluation saturatedInverseFormationVolumeFactor(unsigned regionIdx,
                                                      const Evaluation& temperature,
                                                      const Evaluation& pressure) const
-    { OPM_OIL_PVT_MULTIPLEXER_CALL(return pvtImpl.saturatedInverseFormationVolumeFactor(regionIdx, temperature, pressure)); return 0; }
+    { OPM_OIL_PVT_MULTIPLEXER_CALL(return pvtImpl.saturatedInverseFormationVolumeFactor(regionIdx, temperature, pressure)); }
 
     /*!
      * \brief Returns the gas dissolution factor \f$R_s\f$ [m^3/m^3] of saturated oil.
@@ -236,7 +207,7 @@ public:
     Evaluation saturatedGasDissolutionFactor(unsigned regionIdx,
                                              const Evaluation& temperature,
                                              const Evaluation& pressure) const
-    { OPM_OIL_PVT_MULTIPLEXER_CALL(return pvtImpl.saturatedGasDissolutionFactor(regionIdx, temperature, pressure)); return 0; }
+    { OPM_OIL_PVT_MULTIPLEXER_CALL(return pvtImpl.saturatedGasDissolutionFactor(regionIdx, temperature, pressure)); }
 
     /*!
      * \brief Returns the gas dissolution factor \f$R_s\f$ [m^3/m^3] of saturated oil.
@@ -247,7 +218,7 @@ public:
                                              const Evaluation& pressure,
                                              const Evaluation& oilSaturation,
                                              const Evaluation& maxOilSaturation) const
-    { OPM_OIL_PVT_MULTIPLEXER_CALL(return pvtImpl.saturatedGasDissolutionFactor(regionIdx, temperature, pressure, oilSaturation, maxOilSaturation)); return 0; }
+    { OPM_OIL_PVT_MULTIPLEXER_CALL(return pvtImpl.saturatedGasDissolutionFactor(regionIdx, temperature, pressure, oilSaturation, maxOilSaturation)); }
 
     /*!
      * \brief Returns the saturation pressure [Pa] of oil given the mass fraction of the
@@ -260,7 +231,7 @@ public:
     Evaluation saturationPressure(unsigned regionIdx,
                                   const Evaluation& temperature,
                                   const Evaluation& Rs) const
-    { OPM_OIL_PVT_MULTIPLEXER_CALL(return pvtImpl.saturationPressure(regionIdx, temperature, Rs)); return 0; }
+    { OPM_OIL_PVT_MULTIPLEXER_CALL(return pvtImpl.saturationPressure(regionIdx, temperature, Rs)); }
 
     /*!
      * \copydoc BaseFluidSystem::diffusionCoefficient
@@ -270,42 +241,10 @@ public:
                                     const Evaluation& pressure,
                                     unsigned compIdx) const
     {
-      OPM_OIL_PVT_MULTIPLEXER_CALL(return pvtImpl.diffusionCoefficient(temperature, pressure, compIdx)); return 0;
+      OPM_OIL_PVT_MULTIPLEXER_CALL(return pvtImpl.diffusionCoefficient(temperature, pressure, compIdx));
     }
 
-    void setApproach(OilPvtApproach appr)
-    {
-        switch (appr) {
-        case OilPvtApproach::LiveOil:
-            realOilPvt_ = new LiveOilPvt<Scalar>;
-            break;
-
-        case OilPvtApproach::DeadOil:
-            realOilPvt_ = new DeadOilPvt<Scalar>;
-            break;
-
-        case OilPvtApproach::ConstantCompressibilityOil:
-            realOilPvt_ = new ConstantCompressibilityOilPvt<Scalar>;
-            break;
-
-        case OilPvtApproach::ThermalOil:
-            realOilPvt_ = new OilPvtThermal<Scalar>;
-            break;
-
-        case OilPvtApproach::BrineCo2:
-            realOilPvt_ = new BrineCo2Pvt<Scalar>;
-            break;
-
-        case OilPvtApproach::BrineH2:
-            realOilPvt_ = new BrineH2Pvt<Scalar>;
-            break;
-
-        case OilPvtApproach::NoOil:
-            throw std::logic_error("Not implemented: Oil PVT of this deck!");
-        }
-
-        approach_ = appr;
-    }
+    void setApproach(OilPvtApproach appr);
 
     /*!
      * \brief Returns the concrete approach for calculating the PVT relations.
@@ -402,38 +341,12 @@ public:
         return *static_cast<const BrineH2Pvt<Scalar>* >(realOilPvt_);
     }
 
-    OilPvtMultiplexer<Scalar,enableThermal>& operator=(const OilPvtMultiplexer<Scalar,enableThermal>& data)
-    {
-        approach_ = data.approach_;
-        switch (approach_) {
-        case OilPvtApproach::ConstantCompressibilityOil:
-            realOilPvt_ = new ConstantCompressibilityOilPvt<Scalar>(*static_cast<const ConstantCompressibilityOilPvt<Scalar>*>(data.realOilPvt_));
-            break;
-        case OilPvtApproach::DeadOil:
-            realOilPvt_ = new DeadOilPvt<Scalar>(*static_cast<const DeadOilPvt<Scalar>*>(data.realOilPvt_));
-            break;
-        case OilPvtApproach::LiveOil:
-            realOilPvt_ = new LiveOilPvt<Scalar>(*static_cast<const LiveOilPvt<Scalar>*>(data.realOilPvt_));
-            break;
-        case OilPvtApproach::ThermalOil:
-            realOilPvt_ = new OilPvtThermal<Scalar>(*static_cast<const OilPvtThermal<Scalar>*>(data.realOilPvt_));
-            break;
-        case OilPvtApproach::BrineCo2:
-            realOilPvt_ = new BrineCo2Pvt<Scalar>(*static_cast<const BrineCo2Pvt<Scalar>*>(data.realOilPvt_));
-            break;
-	    case OilPvtApproach::BrineH2:
-            realOilPvt_ = new BrineH2Pvt<Scalar>(*static_cast<const BrineH2Pvt<Scalar>*>(data.realOilPvt_));
-            break;
-        default:
-            break;
-        }
-
-        return *this;
-    }
+    OilPvtMultiplexer<Scalar,enableThermal>&
+    operator=(const OilPvtMultiplexer<Scalar,enableThermal>& data);
 
 private:
-    OilPvtApproach approach_;
-    void* realOilPvt_;
+    OilPvtApproach approach_{OilPvtApproach::NoOil};
+    void* realOilPvt_{nullptr};
 };
 
 } // namespace Opm
