@@ -15,7 +15,7 @@
 
   You should have received a copy of the GNU General Public License
   along with OPM.  If not, see <http://www.gnu.org/licenses/>.
- */
+*/
 
 #ifndef OPM_OUTPUT_WELLS_HPP
 #define OPM_OUTPUT_WELLS_HPP
@@ -27,6 +27,7 @@
 
 #include <algorithm>
 #include <array>
+#include <climits>
 #include <cstddef>
 #include <map>
 #include <stdexcept>
@@ -71,7 +72,8 @@ namespace Opm { namespace data {
                 alq              = (1 << 18),
                 tracer           = (1 << 19),
                 micp             = (1 << 20),
-                vaporized_water     = (1 << 21)
+                vaporized_water  = (1 << 21),
+                mass_gas         = (1 << 22)
             };
 
             using enum_size = std::underlying_type< opt >::type;
@@ -130,6 +132,7 @@ namespace Opm { namespace data {
                 serializer(tracer);
                 serializer(micp);
                 serializer(vaporized_water);
+                serializer(mass_gas);
             }
 
             static Rates serializationTestObject()
@@ -156,6 +159,7 @@ namespace Opm { namespace data {
                 rat1.set(opt::alq, 19.0);
                 rat1.set(opt::micp, 21.0);
                 rat1.set(opt::vaporized_water, 22.0);
+                rat1.set(opt::mass_gas, 23.0);
                 rat1.tracer.insert({"test_tracer", 1.0});
 
                 return rat1;
@@ -188,12 +192,14 @@ namespace Opm { namespace data {
             double well_potential_gas = 0.0;
             double brine = 0.0;
             double alq = 0.0;
-            std::map<std::string, double> tracer;
+            std::map<std::string, double> tracer{};
             double micp = 0.0;
             double vaporized_water = 0.0;
+            double mass_gas = 0.0;
     };
 
-    struct ConnectionFiltrate {
+    struct ConnectionFiltrate
+    {
         double rate;
         double total;
         double skin_factor;
@@ -240,20 +246,21 @@ namespace Opm { namespace data {
         void read(MessageBufferType& buffer);
     };
 
-    struct Connection {
-        using global_index = size_t;
+    struct Connection
+    {
+        using global_index = std::size_t;
         static const constexpr int restart_size = 6;
 
-        global_index index;
-        Rates rates;
-        double pressure;
-        double reservoir_rate;
-        double cell_pressure;
-        double cell_saturation_water;
-        double cell_saturation_gas;
-        double effective_Kh;
-        double trans_factor;
-        double d_factor;
+        global_index index{};
+        Rates rates{};
+        double pressure{};
+        double reservoir_rate{};
+        double cell_pressure{};
+        double cell_saturation_water{};
+        double cell_saturation_gas{};
+        double effective_Kh{};
+        double trans_factor{};
+        double d_factor{};
         double compact_mult{1.0}; // Rock compaction transmissibility multiplier (ROCKTAB)
 
         ConnectionFiltrate filtrate;
@@ -310,7 +317,8 @@ namespace Opm { namespace data {
         }
     };
 
-    class SegmentPressures {
+    class SegmentPressures
+    {
     public:
         enum class Value : std::size_t {
             Pressure, PDrop, PDropHydrostatic, PDropAccel, PDropFriction,
@@ -377,7 +385,7 @@ namespace Opm { namespace data {
     };
 
     template <typename Items>
-    class SegmentQuantity
+    class QuantityCollection
     {
     public:
         using Item = typename Items::Item;
@@ -395,10 +403,10 @@ namespace Opm { namespace data {
             return (i < Size) && this->hasItem(i);
         }
 
-        bool operator==(const SegmentQuantity& vec) const
+        bool operator==(const QuantityCollection& that) const
         {
-            return (this->has_   == vec.has_)
-                && (this->value_ == vec.value_);
+            return (this->has_   == that.has_)
+                && (this->value_ == that.value_);
         }
 
         double get(const Item p) const
@@ -412,7 +420,7 @@ namespace Opm { namespace data {
             return this->value_[ this->index(p) ];
         }
 
-        SegmentQuantity& set(const Item p, const double value)
+        QuantityCollection& set(const Item p, const double value)
         {
             const auto i = this->index(p);
 
@@ -457,9 +465,9 @@ namespace Opm { namespace data {
             serializer(this->value_);
         }
 
-        static SegmentQuantity serializationTestObject()
+        static QuantityCollection serializationTestObject()
         {
-            auto quant = SegmentQuantity{};
+            auto quant = QuantityCollection{};
 
             for (const auto& [item, value] : Items::serializationTestItems()) {
                 quant.set(item, value);
@@ -471,6 +479,9 @@ namespace Opm { namespace data {
     private:
         enum { Size = static_cast<std::size_t>(Item::NumItems) };
 
+        static_assert(Size <= static_cast<std::size_t>(CHAR_BIT),
+                      "Number of items must not exceed CHAR_BIT");
+
         /// Whether or not item has a defined value.  We use the bottom
         /// 'Size' bits.
         unsigned char has_{};
@@ -559,8 +570,8 @@ namespace Opm { namespace data {
         }
     };
 
-    using SegmentPhaseQuantity = SegmentQuantity<PhaseItems>;
-    using SegmentPhaseDensity = SegmentQuantity<DensityItems>;
+    using SegmentPhaseQuantity = QuantityCollection<PhaseItems>;
+    using SegmentPhaseDensity = QuantityCollection<DensityItems>;
 
     struct Segment
     {
@@ -615,7 +626,8 @@ namespace Opm { namespace data {
         }
     };
 
-    struct CurrentControl {
+    struct CurrentControl
+    {
         bool isProducer{true};
 
         ::Opm::WellProducerCMode prod {
@@ -720,7 +732,8 @@ namespace Opm { namespace data {
         std::array<double, NumQuantities> wbp_{};
     };
 
-    struct WellFiltrate {
+    struct WellFiltrate
+    {
         double rate{0.};
         double total{0.};
         double concentration{0.};
@@ -753,8 +766,51 @@ namespace Opm { namespace data {
         void read(MessageBufferType& buffer);
     };
 
-    struct Well {
+    struct WellControlLimitItems
+    {
+        enum class Item {
+            Bhp, OilRate, WaterRate, GasRate, ResVRate, LiquidRate,
+
+            // -- Must be last enumerator --
+            NumItems,
+        };
+
+        static std::string itemName(const Item p)
+        {
+            switch (p) {
+            case Item::Bhp:        return "Bhp";
+            case Item::OilRate:    return "OilRate";
+            case Item::WaterRate:  return "WaterRate";
+            case Item::GasRate:    return "GasRate";
+            case Item::ResVRate:   return "ResVRate";
+            case Item::LiquidRate: return "LiquidRate";
+
+            case Item::NumItems:
+                return "Out of bounds (NumItems)";
+            }
+
+            return "Unknown (" + std::to_string(static_cast<int>(p)) + ')';
+        }
+
+        static auto serializationTestItems()
+        {
+            return std::vector {
+                std::pair { Item::Bhp       , 321.09 },
+                std::pair { Item::OilRate   , 987.65 },
+                std::pair { Item::WaterRate , 975.31 },
+                std::pair { Item::GasRate   , 765.43 },
+                std::pair { Item::ResVRate  , 876.54 },
+                std::pair { Item::LiquidRate,  54.32 },
+            };
+        }
+    };
+
+    using WellControlLimits = QuantityCollection<WellControlLimitItems>;
+
+    struct Well
+    {
         Rates rates{};
+
         double bhp{0.0};
         double thp{0.0};
         double temperature{0.0};
@@ -764,39 +820,48 @@ namespace Opm { namespace data {
 
         ::Opm::WellStatus dynamicStatus { Opm::WellStatus::OPEN };
 
-        std::vector< Connection > connections{};
+        std::vector<Connection> connections{};
         std::unordered_map<std::size_t, Segment> segments{};
         CurrentControl current_control{};
         GuideRateValue guide_rates{};
+        WellControlLimits limits{};
 
         inline bool flowing() const noexcept;
+
         template <class MessageBufferType>
         void write(MessageBufferType& buffer) const;
+
         template <class MessageBufferType>
         void read(MessageBufferType& buffer);
 
         inline void init_json(Json::JsonObject& json_data) const;
 
-        const Connection* find_connection(Connection::global_index connection_grid_index) const {
-            const auto connection = std::find_if( this->connections.begin() ,
-                                                  this->connections.end() ,
-                                                  [=]( const Connection& c ) {
-                                                      return c.index == connection_grid_index; });
+        const Connection*
+        find_connection(const Connection::global_index connection_grid_index) const
+        {
+            auto connection = std::find_if(this->connections.begin(),
+                                           this->connections.end(),
+                                           [connection_grid_index](const Connection& c)
+                                           { return c.index == connection_grid_index; });
 
-            if( connection == this->connections.end() )
+            if (connection == this->connections.end()) {
                 return nullptr;
+            }
 
             return &*connection;
         }
 
-        Connection* find_connection(Connection::global_index connection_grid_index) {
-            auto connection = std::find_if( this->connections.begin() ,
-                                            this->connections.end() ,
-                                            [=]( const Connection& c ) {
-                                                return c.index == connection_grid_index; });
+        Connection*
+        find_connection(const Connection::global_index connection_grid_index)
+        {
+            auto connection = std::find_if(this->connections.begin(),
+                                           this->connections.end(),
+                                           [connection_grid_index](const Connection& c)
+                                           { return c.index == connection_grid_index; });
 
-            if( connection == this->connections.end() )
+            if (connection == this->connections.end()) {
                 return nullptr;
+            }
 
             return &*connection;
         }
@@ -814,6 +879,7 @@ namespace Opm { namespace data {
                 && (this->segments == well2.segments)
                 && (this->current_control == well2.current_control)
                 && (this->guide_rates == well2.guide_rates)
+                && (this->limits == well2.limits)
                 ;
         }
 
@@ -831,6 +897,7 @@ namespace Opm { namespace data {
             serializer(segments);
             serializer(current_control);
             serializer(guide_rates);
+            serializer(limits);
         }
 
         static Well serializationTestObject()
@@ -846,7 +913,8 @@ namespace Opm { namespace data {
                 {Connection::serializationTestObject()},
                 {{0, Segment::serializationTestObject()}},
                 CurrentControl::serializationTestObject(),
-                GuideRateValue::serializationTestObject()
+                GuideRateValue::serializationTestObject(),
+                WellControlLimits::serializationTestObject()
             };
         }
     };
@@ -1045,7 +1113,8 @@ namespace Opm { namespace data {
              alq == rate.alq &&
              tracer == rate.tracer &&
              micp == rate.micp &&
-             vaporized_water == rate.vaporized_water;
+             vaporized_water == rate.vaporized_water &&
+             mass_gas == rate.mass_gas;
     }
 
 
@@ -1083,6 +1152,7 @@ namespace Opm { namespace data {
                 break;
             case opt::micp: return this->micp;
             case opt::vaporized_water: return this->vaporized_water;
+            case opt::mass_gas: return this->mass_gas;
         }
 
         throw std::invalid_argument(
@@ -1155,6 +1225,7 @@ namespace Opm { namespace data {
             buffer.write(this->well_potential_gas);
             buffer.write(this->brine);
             buffer.write(this->alq);
+
             //tracer:
             unsigned int size = this->tracer.size();
             buffer.write(size);
@@ -1162,8 +1233,10 @@ namespace Opm { namespace data {
                 buffer.write(name);
                 buffer.write(rate);
             }
+
             buffer.write(this->micp);
             buffer.write(this->vaporized_water);
+            buffer.write(this->mass_gas);
     }
 
     template <class MessageBufferType>
@@ -1251,12 +1324,15 @@ namespace Opm { namespace data {
     }
 
     template <class MessageBufferType>
-    void Well::write(MessageBufferType& buffer) const {
+    void Well::write(MessageBufferType& buffer) const
+    {
         this->rates.write(buffer);
+
         buffer.write(this->bhp);
         buffer.write(this->thp);
         buffer.write(this->temperature);
         buffer.write(this->control);
+
         this->filtrate.write(buffer);
 
         {
@@ -1264,10 +1340,14 @@ namespace Opm { namespace data {
             buffer.write(status);
         }
 
-        unsigned int size = this->connections.size();
-        buffer.write(size);
-        for (const Connection& comp : this->connections)
-            comp.write(buffer);
+        {
+            const unsigned int size = this->connections.size();
+            buffer.write(size);
+
+            for (const Connection& comp : this->connections) {
+                comp.write(buffer);
+            }
+        }
 
         {
             const auto nSeg =
@@ -1281,6 +1361,7 @@ namespace Opm { namespace data {
 
         this->current_control.write(buffer);
         this->guide_rates.write(buffer);
+        this->limits.write(buffer);
     }
 
     template <class MessageBufferType>
@@ -1316,6 +1397,7 @@ namespace Opm { namespace data {
             buffer.read(this->well_potential_gas);
             buffer.read(this->brine);
             buffer.read(this->alq);
+
             //tracer:
             unsigned int size;
             buffer.read(size);
@@ -1326,8 +1408,10 @@ namespace Opm { namespace data {
                 buffer.read(tracer_rate);
                 this->tracer.emplace(tracer_name, tracer_rate);
             }
+
             buffer.read(this->micp);
             buffer.read(this->vaporized_water);
+            buffer.read(this->mass_gas);
     }
 
     template <class MessageBufferType>
@@ -1399,12 +1483,15 @@ namespace Opm { namespace data {
     }
 
     template <class MessageBufferType>
-    void Well::read(MessageBufferType& buffer) {
+    void Well::read(MessageBufferType& buffer)
+    {
         this->rates.read(buffer);
+
         buffer.read(this->bhp);
         buffer.read(this->thp);
         buffer.read(this->temperature);
         buffer.read(this->control);
+
         this->filtrate.read(buffer);
 
         {
@@ -1414,13 +1501,14 @@ namespace Opm { namespace data {
         }
 
         // Connection information
-        unsigned int size = 0.0; //this->connections.size();
-        buffer.read(size);
-        this->connections.resize(size);
-        for (size_t i = 0;  i < size; ++i)
         {
-            auto& comp = this->connections[ i ];
-            comp.read(buffer);
+            unsigned int size = 0;
+            buffer.read(size);
+
+            this->connections.resize(size);
+            for (auto& connection : this->connections) {
+                connection.read(buffer);
+            }
         }
 
         // Segment information (if applicable)
@@ -1442,6 +1530,7 @@ namespace Opm { namespace data {
 
         this->current_control.read(buffer);
         this->guide_rates.read(buffer);
+        this->limits.read(buffer);
     }
 
     template <class MessageBufferType>
