@@ -852,8 +852,9 @@ namespace {
 
             sWell[Ix::HistBHPTarget] = sWell[Ix::BHPTarget];
 
-            if (pc.alq_value != 0.0) {
-                const auto alqType = sched[sim_step].vfpprod(pc.vfp_table_number).getALQType();
+            const auto& vfpprod = sched[sim_step].vfpprod;
+            if (pc.alq_value != 0.0 && vfpprod.has(pc.vfp_table_number)) {
+                const auto alqType = vfpprod(pc.vfp_table_number).getALQType();
                 assignALQProd(alqType, pc.alq_value, std::forward<SWProp>(swprop), sWell);
             }
 
@@ -1077,8 +1078,10 @@ namespace {
             auto output_index = static_cast<std::size_t>(VI::SWell::index::TracerOffset);
 
             for (const auto& tracer : tracers) {
-                sWell[output_index++] =
-                    smry.get_well_var(wname, fmt::format("WTIC{}", tracer.name), 0.0);
+                if (tracer.phase == Opm::Phase::WATER) {
+                    sWell[output_index++] =
+                        smry.get_well_var(wname, fmt::format("WTIC{}", tracer.name), 0.0);
+                }
             }
         }
 
@@ -1332,7 +1335,7 @@ namespace {
                               const Opm::Well& well,
                               XWellArray& xWell)
         {
-            if (tracers.empty())
+            if (tracers.empty() || tracer_dims.water_tracers() == 0)
                 return;
 
             using Ix = ::Opm::RestartIO::Helpers::VectorItems::XWell::index;
