@@ -46,7 +46,6 @@
 
 #include <algorithm>
 #include <cstddef>
-#include <exception>
 #include <iterator>
 #include <stdexcept>
 #include <string>
@@ -146,14 +145,12 @@ namespace {
         }
 
         const auto& udqCfg = sched.getUDQConfig(simStep);
-        std::size_t i_wudq = 0;
-        for (const auto& udq_input : udqCfg.input()) {
-            if (udq_input.var_type() ==  Opm::UDQVarType::WELL_VAR) {
-                i_wudq++;
-            }
-        }
-
-        return i_wudq;
+        const auto input = udqCfg.input();
+        return std::count_if(input.begin(), input.end(),
+                             [](const auto& udq_input)
+                             {
+                                 return udq_input.var_type() == Opm::UDQVarType::WELL_VAR;
+                             });
     }
 
     int noGroupUdqs(const Opm::Schedule& sched,
@@ -181,9 +178,13 @@ namespace {
         const auto& udqAct = sched[simStep].udq_active.get();
         const auto& iuad = udqAct.iuad();
 
-        return std::count_if(iuad.begin(), iuad.end(), [](const Opm::UDQActive::OutputRecord rec) {
-            return (!(((Opm::UDQ::keyword(rec.control) == Opm::UDAKeyword::GCONPROD) || (Opm::UDQ::keyword(rec.control) == Opm::UDAKeyword::GCONINJE))
-            && (rec.wg_name() == "FIELD"))); });
+        return std::count_if(iuad.begin(), iuad.end(),
+                             [](const Opm::UDQActive::OutputRecord& rec)
+                             {
+                                  return (!(((Opm::UDQ::keyword(rec.control) == Opm::UDAKeyword::GCONPROD)
+                                            || (Opm::UDQ::keyword(rec.control) == Opm::UDAKeyword::GCONINJE))
+                                            && (rec.wg_name() == "FIELD")));
+                            });
     }
 
     int noIuaps(const Opm::Schedule& sched,
@@ -197,9 +198,13 @@ namespace {
         const auto& udqAct = sched[simStep].udq_active.get();
         const auto& iuap = udqAct.iuap();
 
-        return std::count_if(iuap.begin(), iuap.end(), [](const Opm::UDQActive::InputRecord rec) {
-            return (!(((Opm::UDQ::keyword(rec.control) == Opm::UDAKeyword::GCONPROD) || (Opm::UDQ::keyword(rec.control) == Opm::UDAKeyword::GCONINJE))
-            && (rec.wgname == "FIELD"))); });
+        return std::count_if(iuap.begin(), iuap.end(),
+                             [](const Opm::UDQActive::InputRecord& rec)
+                             {
+                                return (!(((Opm::UDQ::keyword(rec.control) == Opm::UDAKeyword::GCONPROD)
+                                         || (Opm::UDQ::keyword(rec.control) == Opm::UDAKeyword::GCONINJE))
+                                         && (rec.wgname == "FIELD")));
+                             });
     }
 
 
@@ -212,14 +217,12 @@ namespace {
         }
 
         const auto& udqCfg = sched.getUDQConfig(simStep);
-        std::size_t i_fudq = 0;
-        for (const auto& udq_input : udqCfg.input()) {
-            if (udq_input.var_type() ==  Opm::UDQVarType::FIELD_VAR) {
-                i_fudq++;
-            }
-        }
-
-        return i_fudq;
+        const auto input = udqCfg.input();
+        return std::count_if(input.begin(), input.end(),
+                             [](const auto& udq_input)
+                             {
+                                 return udq_input.var_type() == Opm::UDQVarType::FIELD_VAR;
+                             });
     }
 
     int numMultiSegWells(const ::Opm::Schedule& sched,
@@ -631,6 +634,7 @@ createInteHead(const EclipseState& es,
              // The numbers below have been determined experimentally to work
              // across a range of reference cases, but are not guaranteed to be
              // universally valid.
+        .drsdt(sched, lookup_step)
         .params_NWELZ       (155 + num_water_tracer, 122 + 2*num_water_tracer, 130 + nxwelz_tracer_shift, 3) // n{isxz}welz: number of data elements per well in {ISXZ}WELL
         .params_NCON        (25, 41, 58 + 5*num_water_tracer)       // n{isx}conz: number of data elements per completion in ICON
         .params_GRPZ        (getNGRPZ(nwgmax, ngmax, num_water_tracer, rspec))
