@@ -36,6 +36,7 @@
 #include <opm/output/eclipse/AggregateUDQData.hpp>
 #include <opm/output/eclipse/AggregateActionxData.hpp>
 #include <opm/output/eclipse/RestartValue.hpp>
+#include <opm/output/eclipse/UDQDims.hpp>
 
 #include <opm/output/eclipse/WriteRestartHelpers.hpp>
 
@@ -55,6 +56,7 @@
 #include <opm/input/eclipse/Schedule/ScheduleState.hpp>
 #include <opm/input/eclipse/Schedule/SummaryState.hpp>
 #include <opm/input/eclipse/Schedule/Tuning.hpp>
+#include <opm/input/eclipse/Schedule/UDQ/UDQConfig.hpp>
 #include <opm/input/eclipse/Schedule/Well/Well.hpp>
 #include <opm/input/eclipse/Schedule/Well/WellConnections.hpp>
 
@@ -114,100 +116,6 @@ namespace {
             : 0.0;
     }
 
-    std::vector<int>
-    serialize_OPM_IWEL(const data::Wells&              wells,
-                       const std::vector<std::string>& well_names)
-    {
-      const auto getctrl = [&]( const std::string& wname ) {
-            const auto itr = wells.find( wname );
-            return itr == wells.end() ? 0 : itr->second.control;
-        };
-
-        std::vector<int> iwel(well_names.size(), 0.0);
-        std::transform(well_names.begin(), well_names.end(), iwel.begin(), getctrl);
-
-        return iwel;
-    }
-
-    std::vector<double>
-    serialize_OPM_XWEL(const data::Wells&              wells,
-                       const Schedule&                 schedule,
-                       const std::vector<std::string>& well_names,
-                       const int                       sim_step,
-                       const Phases&                   phase_spec,
-                       const EclipseGrid&              grid)
-    {
-        using rt = data::Rates::opt;
-
-        std::vector<rt> phases;
-        if (phase_spec.active(Phase::WATER)) phases.push_back(rt::wat);
-        if (phase_spec.active(Phase::OIL))   phases.push_back(rt::oil);
-        if (phase_spec.active(Phase::GAS))   phases.push_back(rt::gas);
-
-        std::vector< double > xwel;
-        for (const auto& wellname : well_names) {
-            const auto& sched_well = schedule.getWell(wellname, sim_step);
-            if (wells.count(wellname) == 0 ||
-                sched_well.getStatus() == Opm::Well::Status::SHUT)
-            {
-                const auto elems = (sched_well.getConnections().size()
-                                    * (phases.size() + data::Connection::restart_size))
-                    + 3 /* bhp, thp, temperature */
-                    + phases.size();
-
-                // write zeros if no well data is provided
-                xwel.insert( xwel.end(), elems, 0.0 );
-                continue;
-            }
-
-            const auto& well = wells.at( wellname );
-
-            xwel.push_back( well.bhp );
-            xwel.push_back( well.thp );
-            xwel.push_back( well.temperature );
-
-            for (auto phase : phases)
-                xwel.push_back(well.rates.get(phase));
-
-            for (const auto& sc : sched_well.getConnections()) {
-                const auto i = sc.getI(), j = sc.getJ(), k = sc.getK();
-
-                const auto rs_size = phases.size() + data::Connection::restart_size;
-                if (!grid.cellActive(i, j, k) || sc.state() == Connection::State::SHUT) {
-                    xwel.insert(xwel.end(), rs_size, 0.0);
-                    continue;
-                }
-
-                const auto global_index = grid.getGlobalIndex(i, j, k);
-
-                const auto& connection =
-                    std::find_if(well.connections.begin(),
-                                 well.connections.end(),
-                        [global_index](const data::Connection& c)
-                    {
-                        return c.index == global_index;
-                    });
-
-                if (connection == well.connections.end()) {
-                    xwel.insert( xwel.end(), rs_size, 0.0 );
-                    continue;
-                }
-
-                xwel.push_back(connection->pressure);
-                xwel.push_back(connection->reservoir_rate);
-                xwel.push_back(connection->cell_pressure);
-                xwel.push_back(connection->cell_saturation_water);
-                xwel.push_back(connection->cell_saturation_gas);
-                xwel.push_back(connection->effective_Kh);
-
-                for (auto phase : phases)
-                    xwel.push_back(connection->rates.get(phase));
-            }
-        }
-
-        return xwel;
-    }
-
     void checkSaveArguments(const EclipseState& es,
                             const RestartValue& restart_value,
                             const EclipseGrid&  grid)
@@ -349,28 +257,46 @@ namespace {
                   const std::vector<int>&       ih,
                   EclIO::OutputStream::Restart& rstFile)
     {
-        if (report_step == 0) {
-            // Initial condition.  No UDQs yet.
+        const auto& udqConfig = schedule[sim_step].udq();
+
+        if ((report_step == 0) || (udqConfig.size() == 0)) {
+            // Initial condition or no UDQs in run.
             return;
         }
 
-        // write UDQ - data to restart file
-        const std::size_t simStep = static_cast<size_t> (sim_step);
+        const auto simStep = static_cast<std::size_t>(sim_step);
+        const auto udqDims = UDQDims { udqConfig, ih };
 
-        const auto udqDims = Helpers::createUdqDims(schedule, simStep, ih);
-        auto  udqData = Helpers::AggregateUDQData(udqDims);
+        // UDQs are active in run.  Write UDQ related data to restart file.
+        auto udqData = Helpers::AggregateUDQData(udqDims);
         udqData.captureDeclaredUDQData(schedule, simStep, udq_state, ih);
         
-        if (udqDims[0] >= 1) {
-            rstFile.write("ZUDN", udqData.getZUDN());
-            rstFile.write("ZUDL", udqData.getZUDL());
-            rstFile.write("IUDQ", udqData.getIUDQ());
-            if (udqDims[12] >= 1) rstFile.write("DUDF", udqData.getDUDF());
-            if (udqDims[11] >= 1) rstFile.write("DUDG", udqData.getDUDG());
-            if (udqDims[ 9] >= 1) rstFile.write("DUDW", udqData.getDUDW());
-            if (udqDims[ 2] >= 1) rstFile.write("IUAD", udqData.getIUAD());
-            if (udqDims[ 7] >= 1) rstFile.write("IUAP", udqData.getIUAP());
-            if (udqDims[ 6] >= 1) rstFile.write("IGPH", udqData.getIGPH());
+        rstFile.write("ZUDN", udqData.getZUDN());
+        rstFile.write("ZUDL", udqData.getZUDL());
+        rstFile.write("IUDQ", udqData.getIUDQ());
+
+        if (const auto& dudf = udqData.getDUDF(); dudf.has_value()) {
+            rstFile.write("DUDF", dudf->data());
+        }
+
+        if (const auto& dudg = udqData.getDUDG(); dudg.has_value()) {
+            rstFile.write("DUDG", dudg->data());
+        }
+
+        if (const auto& dudw = udqData.getDUDW(); dudw.has_value()) {
+            rstFile.write("DUDW", dudw->data());
+        }
+
+        if (const auto& iuad = udqData.getIUAD(); iuad.has_value()) {
+            rstFile.write("IUAD", iuad->data());
+        }
+
+        if (const auto& iuap = udqData.getIUAP(); iuap.has_value()) {
+            rstFile.write("IUAP", iuap->data());
+        }
+
+        if (const auto& igph = udqData.getIGPH(); igph.has_value()) {
+            rstFile.write("IGPH", igph->data());
         }
     }
 
@@ -381,14 +307,15 @@ namespace {
                       const SummaryState&           sum_state,
                       EclIO::OutputStream::Restart& rstFile)
     {
-        if (report_step == 0)
+        if ((report_step == 0) || (schedule[sim_step].actions().ecl_size() == 0)) {
             return;
+        }
 
-        if (schedule[sim_step].actions().ecl_size() == 0)
-            return;
+        const auto simStep = static_cast<std::size_t>(sim_step);
 
-        const std::size_t simStep = static_cast<size_t> (sim_step);
-        Opm::RestartIO::Helpers::AggregateActionxData actionxData{schedule, action_state, sum_state, simStep};
+        const auto actionxData = RestartIO::Helpers::AggregateActionxData {
+            schedule, action_state, sum_state, simStep
+        };
 
         rstFile.write("IACT", actionxData.getIACT());
         rstFile.write("SACT", actionxData.getSACT());
@@ -399,20 +326,16 @@ namespace {
         rstFile.write("SACN", actionxData.getSACN());
     }
 
-
-    void writeWell(int                             sim_step,
-                   const bool                      ecl_compatible_rst,
-                   const Phases&                   phases,
-                   const EclipseGrid&              grid,
-                   const Schedule&                 schedule,
-                   const TracerConfig&             tracers,
-                   const std::vector<std::string>& well_names,
-                   const data::Wells&              wells,
-                   const Opm::Action::State&       action_state,
-                   const Opm::WellTestState&       wtest_state,
-                   const Opm::SummaryState&        sumState,
-                   const std::vector<int>&         ih,
-                   EclIO::OutputStream::Restart&   rstFile)
+    void writeWell(int                           sim_step,
+                   const EclipseGrid&            grid,
+                   const Schedule&               schedule,
+                   const TracerConfig&           tracers,
+                   const data::Wells&            wells,
+                   const Opm::Action::State&     action_state,
+                   const Opm::WellTestState&     wtest_state,
+                   const Opm::SummaryState&      sumState,
+                   const std::vector<int>&       ih,
+                   EclIO::OutputStream::Restart& rstFile)
     {
         auto wellData = Helpers::AggregateWellData(ih);
         wellData.captureDeclaredWellData(schedule, tracers, sim_step, action_state, wtest_state, sumState, ih);
@@ -429,19 +352,6 @@ namespace {
         rstFile.write("ZWLS", wListData.getZWls());
         rstFile.write("IWLS", wListData.getIWls());
 
-        // Extended set of OPM well vectors
-        if (!ecl_compatible_rst)
-        {
-            const auto opm_xwel =
-                serialize_OPM_XWEL(wells, schedule, well_names,
-                                   sim_step, phases, grid);
-
-            const auto opm_iwel = serialize_OPM_IWEL(wells, well_names);
-
-            rstFile.write("OPM_IWEL", opm_iwel);
-            rstFile.write("OPM_XWEL", opm_xwel);
-        }
-
         auto connectionData = Helpers::AggregateConnectionData(ih);
         connectionData.captureDeclaredConnData(schedule, grid, schedule.getUnits(),
                                                wells, sumState, sim_step);
@@ -508,7 +418,6 @@ namespace {
     }
 
     void writeDynamicData(const int                                     sim_step,
-                          const bool                                    ecl_compatible_rst,
                           const EclipseGrid&                            grid,
                           const EclipseState&                           es,
                           const Schedule&                               schedule,
@@ -546,10 +455,8 @@ namespace {
                              sumState, wellSol, inteHD, rstFile);
             }
 
-            const auto& phases = es.runspec().phases();
-
-            writeWell(sim_step, ecl_compatible_rst, phases, grid, schedule, es.tracer(),
-                      wells, wellSol, action_state, wtest_state, sumState, inteHD, rstFile);
+            writeWell(sim_step, grid, schedule, es.tracer(), wellSol,
+                      action_state, wtest_state, sumState, inteHD, rstFile);
         }
 
         if (const auto& aqCfg = es.aquifer();
@@ -565,39 +472,6 @@ namespace {
         }
     }
 
-    bool haveHysteresis(const RestartValue& value)
-    {
-        for (const auto* key : { "KRNSW_OW", "PCSWM_OW",
-                                 "KRNSW_GO", "PCSWM_GO", })
-        {
-            if (value.solution.has(key)) { return true; }
-        }
-
-        return false;
-    }
-
-    std::vector<double>
-    convertedHysteresisSat(const RestartValue& value,
-                           const std::string&  primary,
-                           const std::string&  fallback)
-    {
-        auto smax = std::vector<double>{};
-
-        if (value.solution.has(primary)) {
-            smax = value.solution.data<double>(primary);
-        }
-        else if (value.solution.has(fallback)) {
-            smax = value.solution.data<double>(fallback);
-        }
-
-        if (! smax.empty()) {
-            std::transform(std::begin(smax), std::end(smax), std::begin(smax),
-                           [](const double s) { return 1.0 - s; });
-        }
-
-        return smax;
-    }
-
     std::vector<std::string>
     solutionVectorNames(const RestartValue& value)
     {
@@ -762,35 +636,6 @@ namespace {
         }
     }
 
-    template <class OutputVector>
-    void writeEclipseCompatHysteresis(const RestartValue& value,
-                                      OutputVector&       writeVector)
-    {
-        // Convert Flow-specific vectors {KRNSW,PCSWM}_OW to ECLIPSE's
-        // requisite SOMAX vector.  Only partially characterised.
-        // Sufficient for Norne.
-        {
-            const auto somax =
-                convertedHysteresisSat(value, "KRNSW_OW", "PCSWM_OW");
-
-            if (! somax.empty()) {
-                writeVector("SOMAX", somax);
-            }
-        }
-
-        // Convert Flow-specific vectors {KRNSW,PCSWM}_GO to ECLIPSE's
-        // requisite SGMAX vector.  Only partially characterised.
-        // Sufficient for Norne.
-        {
-            const auto sgmax =
-                convertedHysteresisSat(value, "KRNSW_GO", "PCSWM_GO");
-
-            if (! sgmax.empty()) {
-                writeVector("SGMAX", sgmax);
-            }
-        }
-    }
-
     void writeTracerVectors(const UnitSystem&             unit_system,
                             const TracerConfig&           tracer_config,
                             const RestartValue&           value,
@@ -871,10 +716,6 @@ namespace {
 
         writeExtraVectors(value, writeDouble);
 
-        if (ecl_compatible_rst && haveHysteresis(value)) {
-            writeEclipseCompatHysteresis(value, writeDorF);
-        }
-
         if (! ecl_compatible_rst) {
             writeExtendedSolutionVectors(value, writeDorF, writeInt);
         }
@@ -958,9 +799,9 @@ void save(EclIO::OutputStream::Restart&                 rstFile,
                     seconds_elapsed, schedule, grid, es, rstFile);
 
     if (report_step > 0) {
-        writeDynamicData(sim_step, ecl_compatible_rst, grid, es, schedule,
-                         value.wells, action_state, wtest_state,
-                         sumState, inteHD, value.aquifer, aquiferData, rstFile);
+        writeDynamicData(sim_step, grid, es, schedule, value.wells,
+                         action_state, wtest_state, sumState, inteHD,
+                         value.aquifer, aquiferData, rstFile);
     }
 
     writeActionx(report_step, sim_step, schedule, action_state, sumState, rstFile);
