@@ -25,7 +25,6 @@
 
 #include <opm/input/eclipse/Units/UnitSystem.hpp>
 
-#include <map>
 #include <string>
 #include <utility>
 #include <vector>
@@ -34,9 +33,8 @@ namespace Opm {
 
     class RestartKey {
     public:
-
-        std::string key;
-        UnitSystem::measure dim;
+        std::string key{};
+        UnitSystem::measure dim{UnitSystem::measure::_count};
         bool required = false;
 
         RestartKey() = default;
@@ -90,7 +88,9 @@ namespace Opm {
 
         bool hasExtra(const std::string& key) const;
         void addExtra(const std::string& key, UnitSystem::measure dimension, std::vector<double> data);
+        void addExtra(const std::string& key, UnitSystem::measure dimension, std::vector<float> data);
         void addExtra(const std::string& key, std::vector<double> data);
+        void addExtra(const std::string& key, std::vector<float> data);
         const std::vector<double>& getExtra(const std::string& key) const;
 
         void convertFromSI(const UnitSystem& units);
