@@ -59,18 +59,23 @@ public:
 
     int countDev() { return  deviations.size(); }
 
-    // Accept extra keywords: If this switch is set to true the comparison
-    // will ignore extra keywords which are only present
-    // in the new simulation.
 
     void setReportStepOnly(bool reportStepOnlyArg) {
         this->reportStepOnly = reportStepOnlyArg;
     }
 
+    // Accept extra keywords: If this switch is set to true the comparison
+    // will ignore extra keywords which are only present
+    // in the new simulation.
     void setAcceptExtraKeywords(bool acceptExtraKeywordsArg) {
         this->acceptExtraKeywords = acceptExtraKeywordsArg;
     }
 
+    //Accept extra keywords in both simulations and compare only the keywords available in both runs
+    void setAcceptExtraKeywordsBoth(bool acceptExtraKeywordsArg) {
+        this->acceptExtraKeywordsBoth = acceptExtraKeywordsArg;
+    }
+
     void setIntegrationTest(bool inregrationTestArg) {
         this->integrationTest = inregrationTestArg;
     }
@@ -79,7 +84,7 @@ public:
         this->printKeywordOnly = printArg;
     }
 
-    void compareSpesificKeyword(std::string keyword) {
+    void compareSpecificKeyword(std::string keyword) {
         this->specificKeyword = std::move(keyword);
     }
     void compareSpesificRstReportStepNumber(int seqn) {
@@ -121,7 +126,7 @@ private:
                          const std::vector<std::string>& keywords2,
                          const std::string& reference);
 
-    void checkSpesificKeyword(std::vector<std::string>& keywords1,
+    void checkSpecificKeyword(std::vector<std::string>& keywords1,
                               std::vector<std::string>& keywords2,
                               std::vector<EIOD::eclArrType>& arrayType1,
                               std::vector<EIOD::eclArrType>& arrayType2,
@@ -185,6 +190,7 @@ private:
 
     // Accept extra keywords in the restart file of the 'new' simulation.
     bool acceptExtraKeywords = false;
+    bool acceptExtraKeywordsBoth = false;
 
     Opm::EclIO::EGrid* grid1 = nullptr;
     Opm::EclIO::EGrid* grid2 = nullptr;
