@@ -17,11 +17,11 @@
   along with OPM.  If not, see <http://www.gnu.org/licenses/>.
 */
 
+#include <algorithm>
 #include <cmath>
 #include <filesystem>
 #include <iomanip>
 #include <iostream>
-#include <tuple>
 #include <getopt.h>
 #include <sstream>
 
@@ -84,7 +84,7 @@ int main(int argc, char **argv) {
         switch (c) {
         case 'h':
             printHelp();
-            return 0;
+            return EXIT_SUCCESS;
         case 'r':
             reportStepsOnly=true;
             break;
@@ -99,7 +99,13 @@ int main(int argc, char **argv) {
         }
     }
 
-    int argOffset = optind;
+    const int argOffset = optind;
+    if (argOffset > argc - 1) {
+        printHelp();
+        // Returning failure since the user did not
+        // give the correct number of arguments.
+        return EXIT_FAILURE;
+    }
 
     std::unique_ptr<Opm::EclIO::ESmry> esmry;
     std::unique_ptr<Opm::EclIO::ExtESmry> ext_esmry;
@@ -112,15 +118,17 @@ int main(int argc, char **argv) {
 
     smryFileType filetype;
 
-    if (inputFileName.extension()==".SMSPEC"){
+    if (const auto& ext = inputFileName.extension();
+        (ext == ".SMSPEC") || (ext == ".FSMSPEC"))
+    {
         filetype = SMSPEC;
         esmry = std::make_unique<Opm::EclIO::ESmry>(inputFileName);
-    } else if (inputFileName.extension()==".ESMRY"){
+    } else if (ext == ".ESMRY") {
         filetype = ESMRY;
         ext_esmry = std::make_unique<Opm::EclIO::ExtESmry>(inputFileName);
-    } else
+    } else {
         throw std::runtime_error("invalid input file for summary");
-
+    }
 
     if (listKeys){
         std::vector<std::string> list;
@@ -177,8 +185,7 @@ int main(int argc, char **argv) {
                 return EXIT_FAILURE;
             }
 
-            for (auto vect : list)
-                smryList.push_back(vect);
+            smryList.insert(smryList.end(), list.begin(), list.end());
         }
     }
 
@@ -191,10 +198,10 @@ int main(int argc, char **argv) {
     std::vector<std::vector<float>> smryData;
     std::vector<int> width;
 
-    for (auto name : smryList)
-        width.push_back(name.size());
+    std::transform(smryList.begin(), smryList.end(), std::back_inserter(width),
+                   [](const auto& name) { return name.size(); });
 
-    for (auto key : smryList) {
+    for (const auto& key : smryList) {
         std::vector<float> vect;
 
         switch(filetype) {
