@@ -613,6 +613,29 @@ struct Fixture {
     using MaterialLaw = typename MaterialLawManager::MaterialLaw;
 };
 
+namespace Opm
+{
+class FieldPropsManager;
+}
+
+// To support Local Grid Refinement for CpGrid, additional arguments have been added
+// in some EclMaterialLawManager(InitParams) member functions. Therefore, we define
+// some lambda expressions that does not affect this test file.
+std::function<std::vector<int>(const Opm::FieldPropsManager&, const std::string&, bool)> doOldLookup =
+    [](const Opm::FieldPropsManager& fieldPropManager, const std::string& propString, bool needsTranslation)
+    {
+        std::vector<int> dest;
+        const auto& intRawData = fieldPropManager.get_int(propString);
+        unsigned int numElems =  intRawData.size();
+        dest.resize(numElems);
+        for (unsigned elemIdx = 0; elemIdx < numElems; ++elemIdx) {
+            dest[elemIdx] = intRawData[elemIdx] - needsTranslation;
+        }
+        return dest;
+    };
+
+std::function<unsigned(unsigned)> doNothing = [](unsigned elemIdx){ return elemIdx;};
+
 using Types = boost::mpl::list<float,double>;
 BOOST_AUTO_TEST_CASE_TEMPLATE(Fam1Fam2Hysteresis, Scalar, Types)
 {
@@ -630,7 +653,7 @@ BOOST_AUTO_TEST_CASE_TEMPLATE(Fam1Fam2Hysteresis, Scalar, Types)
 
     MaterialLawManager fam1materialLawManager;
     fam1materialLawManager.initFromState(eclState);
-    fam1materialLawManager.initParamsForElements(eclState, n);
+    fam1materialLawManager.initParamsForElements(eclState, n, doOldLookup, doNothing);
 
     BOOST_CHECK(!fam1materialLawManager.enableEndPointScaling());
     BOOST_CHECK(!fam1materialLawManager.enableHysteresis());
@@ -640,7 +663,7 @@ BOOST_AUTO_TEST_CASE_TEMPLATE(Fam1Fam2Hysteresis, Scalar, Types)
 
     MaterialLawManager fam2MaterialLawManager;
     fam2MaterialLawManager.initFromState(fam2EclState);
-    fam2MaterialLawManager.initParamsForElements(fam2EclState, n);
+    fam2MaterialLawManager.initParamsForElements(fam2EclState, n, doOldLookup, doNothing);
 
     BOOST_CHECK(!fam2MaterialLawManager.enableEndPointScaling());
     BOOST_CHECK(!fam2MaterialLawManager.enableHysteresis());
@@ -650,7 +673,7 @@ BOOST_AUTO_TEST_CASE_TEMPLATE(Fam1Fam2Hysteresis, Scalar, Types)
 
     MaterialLawManager hysterMaterialLawManager;
     hysterMaterialLawManager.initFromState(hysterEclState);
-    hysterMaterialLawManager.initParamsForElements(hysterEclState, n);
+    hysterMaterialLawManager.initParamsForElements(hysterEclState, n, doOldLookup, doNothing);
 
     BOOST_CHECK(!hysterMaterialLawManager.enableEndPointScaling());
     BOOST_CHECK(hysterMaterialLawManager.enableHysteresis());
@@ -695,29 +718,31 @@ BOOST_AUTO_TEST_CASE_TEMPLATE(Fam1Fam2Hysteresis, Scalar, Types)
 
                 // This should ideally test each of the materials (stone1, stone2, default, two-phase),
                 // but currently only tests default
-                const std::array<Scalar,2> pcSwMdc_in =  {1.0 / 2.0, 1.0 / 3.0};
-                const std::array<Scalar,2> krnSwMdc_in = {1.0 / 5.0, 1.0 / 7.0};
-                hysterMaterialLawManager.setOilWaterHysteresisParams(pcSwMdc_in[0],
-                                                                     krnSwMdc_in[0],
+                const std::array<Scalar,3> sowmax_in =  {1.0 / 2.0, 1.0 / 3.0, 1.0 / 4.0};
+                const std::array<Scalar,3> sgomax_in = {1.0 / 5.0, 1.0 / 7.0,  1.0 / 9.0};
+                hysterMaterialLawManager.setOilWaterHysteresisParams(sowmax_in[0],
+                                                                     sowmax_in[1],
+                                                                     sowmax_in[2],
                                                                      elemIdx);
-                hysterMaterialLawManager.setGasOilHysteresisParams(pcSwMdc_in[1],
-                                                                   krnSwMdc_in[1],
+                hysterMaterialLawManager.setGasOilHysteresisParams(sgomax_in[0],
+                                                                   sgomax_in[1],
+                                                                   sgomax_in[2],
                                                                    elemIdx);
 
-                std::array<Scalar,2> pcSwMdc_out = {0.0, 0.0};
-                std::array<Scalar,2> krnSwMdc_out = {0.0, 0.0};
-                hysterMaterialLawManager.oilWaterHysteresisParams(pcSwMdc_out[0],
-                                                                  krnSwMdc_out[0],
+                std::array<Scalar,3> sowmax_out = {0.0, 0.0, 0.0};
+                std::array<Scalar,3> sgomax_out = {0.0, 0.0, 0.0};
+                hysterMaterialLawManager.oilWaterHysteresisParams(sowmax_out[0],
+                                                                  sowmax_out[1],
+                                                                  sowmax_out[2],
                                                                   elemIdx);
-                hysterMaterialLawManager.gasOilHysteresisParams(pcSwMdc_out[1],
-                                                                krnSwMdc_out[1],
+                hysterMaterialLawManager.gasOilHysteresisParams(sgomax_out[0],
+                                                                sgomax_out[1],
+                                                                sgomax_out[2],
                                                                 elemIdx);
 
-                for (unsigned phasePairIdx = 0; phasePairIdx < 2; ++phasePairIdx) {
-                    BOOST_CHECK_MESSAGE(pcSwMdc_in[phasePairIdx] == pcSwMdc_out[phasePairIdx],
-                                        "Hysteresis parameters did not propagate correctly");
-                    BOOST_CHECK_MESSAGE(krnSwMdc_in[phasePairIdx] == krnSwMdc_out[phasePairIdx],
-                                        "Hysteresis parameters did not propagate correctly");
+                for (unsigned phasePairIdx = 0; phasePairIdx < 3; ++phasePairIdx) {
+                    BOOST_CHECK_CLOSE(sowmax_in[phasePairIdx], sowmax_out[phasePairIdx], 1e-5);
+                    BOOST_CHECK_CLOSE(sgomax_in[phasePairIdx], sgomax_out[phasePairIdx], 1e-5);     
                 }
             }
         }
@@ -739,14 +764,14 @@ BOOST_AUTO_TEST_CASE_TEMPLATE(GasOil, Scalar, Types)
 
     MaterialLawManager fam1materialLawManager;
     fam1materialLawManager.initFromState(fam1EclState);
-    fam1materialLawManager.initParamsForElements(fam1EclState, n);
+    fam1materialLawManager.initParamsForElements(fam1EclState, n, doOldLookup, doNothing);
 
     const auto fam2Deck = parser.parseString(fam2DeckStringGasOil);
     const Opm::EclipseState fam2EclState(fam2Deck);
 
     MaterialLawManager fam2MaterialLawManager;
     fam2MaterialLawManager.initFromState(fam2EclState);
-    fam2MaterialLawManager.initParamsForElements(fam2EclState, n);
+    fam2MaterialLawManager.initParamsForElements(fam2EclState, n, doOldLookup, doNothing);
 
     for (unsigned elemIdx = 0; elemIdx < n; ++elemIdx) {
         for (int i = 0; i < 100; ++ i) {
@@ -801,14 +826,14 @@ BOOST_AUTO_TEST_CASE_TEMPLATE(GasWater, Scalar, Types)
 
     MaterialLawManager fam2materialLawManager;
     fam2materialLawManager.initFromState(fam2EclState);
-    fam2materialLawManager.initParamsForElements(fam2EclState, n);
+    fam2materialLawManager.initParamsForElements(fam2EclState, n, doOldLookup, doNothing);
 
     const auto fam3Deck = parser.parseString(fam3DeckStringGasWater);
     const Opm::EclipseState fam3EclState(fam3Deck);
 
     MaterialLawManager fam3materialLawManager;
     fam3materialLawManager.initFromState(fam3EclState);
-    fam3materialLawManager.initParamsForElements(fam3EclState, n);
+    fam3materialLawManager.initParamsForElements(fam3EclState, n, doOldLookup, doNothing);
 
     for (unsigned elemIdx = 0; elemIdx < n; ++elemIdx) {
         for (int i = 0; i < 100; ++ i) {
@@ -866,7 +891,7 @@ BOOST_AUTO_TEST_CASE_TEMPLATE(Let, Scalar, Types)
 
     MaterialLawManager letmaterialLawManager;
     letmaterialLawManager.initFromState(letEclState);
-    letmaterialLawManager.initParamsForElements(letEclState, n);
+    letmaterialLawManager.initParamsForElements(letEclState, n, doOldLookup, doNothing);
 
     Scalar Swco = 0.1;
     Scalar psi2Pa = 6894.7573;
