@@ -60,7 +60,7 @@ struct test_data {
     Schedule schedule;
     SummaryConfig summary_config;
 
-    test_data(const Deck& deck_arg) :
+    explicit test_data(const Deck& deck_arg) :
         deck(deck_arg),
         state( this->deck ),
         python( std::make_shared<Python>() ),
@@ -71,7 +71,7 @@ struct test_data {
         ioconfig.setBaseName("MSIM");
     }
 
-    test_data(const std::string& deck_string) :
+    explicit test_data(const std::string& deck_string) :
         test_data( Parser().parseString(deck_string) )
     {}
 
@@ -332,11 +332,11 @@ BOOST_AUTO_TEST_CASE(UDQ_WUWCT) {
                 std::string wuwct_key = std::string("WUWCT:") + well;
                 std::string wopr_key  = std::string("WOPR:") + well;
 
-                if (ecl_sum_get_general_var(ecl_sum, step, wwct_key.c_str()) != 0)
-                    BOOST_CHECK_EQUAL( ecl_sum_get_general_var(ecl_sum, step, wwct_key.c_str()),
-                                       ecl_sum_get_general_var(ecl_sum, step, wuwct_key.c_str()));
+                if (ecl_sum_get_general_var(ecl_sum, step, wwct_key) != 0)
+                    BOOST_CHECK_EQUAL( ecl_sum_get_general_var(ecl_sum, step, wwct_key),
+                                       ecl_sum_get_general_var(ecl_sum, step, wuwct_key));
 
-                wopr_sum += ecl_sum_get_general_var(ecl_sum, step , wopr_key.c_str());
+                wopr_sum += ecl_sum_get_general_var(ecl_sum, step , wopr_key);
             }
             BOOST_CHECK_EQUAL( ecl_sum_get_general_var(ecl_sum, step, "FOPR"),
                                ecl_sum_get_general_var(ecl_sum, step, "FUOPR"));
@@ -451,7 +451,7 @@ BOOST_AUTO_TEST_CASE(UDA) {
                 int prev_tstep = ecl_sum_iget_report_end(ecl_sum, report_step - 1);
                 for (const auto& well : {"P1", "P2", "P3", "P4"}) {
                     std::string wwpr_key  = std::string("WWPR:") + well;
-                    wwpr_sum += ecl_sum_get_general_var(ecl_sum, prev_tstep, wwpr_key.c_str());
+                    wwpr_sum += ecl_sum_get_general_var(ecl_sum, prev_tstep, wwpr_key);
                 }
                 wwpr_sum = 0.90 * wwpr_sum;
                 wwpr_sum = std::max(eps_lim, wwpr_sum);
@@ -481,6 +481,35 @@ BOOST_AUTO_TEST_CASE(COMPDAT) {
 
 #ifdef EMBEDDED_PYTHON
 
+BOOST_AUTO_TEST_CASE(MSIM_EXIT_TEST_PYACTION) {
+    Opm::Parser parser;
+    auto python = std::make_shared<Opm::Python>();
+
+    Opm::Deck deck = parser.parseFile("msim/MSIM_PYACTION_EXIT.DATA");
+    Opm::EclipseState state(deck);
+    Opm::Schedule schedule(deck, state, python);
+    Opm::SummaryConfig summary_config(deck, schedule, state.fieldProps(), state.aquifer());
+
+    {
+        WorkArea work_area("test_msim");
+        Opm::msim msim(state, schedule);
+        Opm::EclipseIO io(state, state.getInputGrid(), schedule, summary_config);
+        msim.well_rate("P1", data::Rates::opt::oil, prod_opr);
+        msim.well_rate("P2", data::Rates::opt::oil, prod_opr);
+        msim.well_rate("P3", data::Rates::opt::oil, prod_opr);
+        msim.well_rate("P4", data::Rates::opt::oil, prod_opr);
+
+        msim.well_rate("P1", data::Rates::opt::wat, prod_wpr_P1);
+        msim.well_rate("P2", data::Rates::opt::wat, prod_wpr_P2);
+        msim.well_rate("P3", data::Rates::opt::wat, prod_wpr_P3);
+        msim.well_rate("P4", data::Rates::opt::wat, prod_wpr_P4);
+        msim.run(io, false);
+
+        auto exit_status = msim.schedule.exitStatus();
+        BOOST_CHECK( exit_status.has_value() );
+        BOOST_CHECK_EQUAL(exit_status.value(), 99);
+    }
+}
 BOOST_AUTO_TEST_CASE(MSIM_PYACTION_INSERT_KEYWORD) {
     const auto& deck = Parser().parseFile("msim/MSIM_PYACTION_INSERT_KEYWORD.DATA");
     test_data td( deck );
