@@ -61,7 +61,7 @@ BOOST_AUTO_TEST_CASE(AICDWellTest)
     const auto dir_x = Opm::Connection::Direction::X;
     const auto kind = Opm::Connection::CTFKind::DeckValue;
 
-    const Opm::EclipseGrid grid { 20,20,20, 1.0, 1.0, 25.0, 2500.0 };
+    Opm::EclipseGrid grid { 20,20,20, 1.0, 1.0, 25.0, 2500.0 };
 
     const auto depth = 0.0;
     const auto state = Opm::Connection::State::OPEN;
@@ -176,8 +176,6 @@ WSEGAICD
     BOOST_CHECK_EQUAL(aicd.widthTransitionRegion(), 0.05);
     BOOST_CHECK_EQUAL(aicd.maxViscosityRatio(), 5.0);
     BOOST_CHECK_EQUAL(aicd.methodFlowScaling(), -1);
-    // the scaling factor has not been updated properly, so it will throw
-    BOOST_CHECK_THROW(aicd.scalingFactor(), std::runtime_error);
 
     const int outlet_segment_number = segment.outletSegment();
     const double outlet_segment_length = segment_set.segmentLength(outlet_segment_number);
@@ -231,7 +229,7 @@ BOOST_AUTO_TEST_CASE(MultisegmentWellTest)
     const auto dir_z = Opm::Connection::Direction::Z;
     const auto dir_x = Opm::Connection::Direction::X;
     const auto kind = Opm::Connection::CTFKind::DeckValue;
-    const Opm::EclipseGrid grid { 20,20,20, 1.0, 1.0, 25.0, 2500.0 };
+    Opm::EclipseGrid grid { 20,20,20, 1.0, 1.0, 25.0, 2500.0 };
 
     const auto depth = 0.0;
     const auto state = Opm::Connection::State::OPEN;
@@ -354,8 +352,6 @@ WSEGSICD
     BOOST_CHECK_EQUAL(sicd.widthTransitionRegion(), 0.05);
     BOOST_CHECK_EQUAL(sicd.maxViscosityRatio(), 5.0);
     BOOST_CHECK_EQUAL(sicd.methodFlowScaling(), -1);
-    // the scaling factor has not been updated properly, so it will throw
-    BOOST_CHECK_THROW(sicd.scalingFactor(), std::runtime_error);
 
     const int outlet_segment_number = segment.outletSegment();
     const double outlet_segment_length = segment_set.segmentLength(outlet_segment_number);
@@ -411,7 +407,7 @@ BOOST_AUTO_TEST_CASE(WrongDistanceCOMPSEGS)
     const auto dir_x = Opm::Connection::Direction::X;
     const auto kind = Opm::Connection::CTFKind::DeckValue;
 
-    const Opm::EclipseGrid grid { 20,20,20, 1.0, 1.0, 25., 2500.0 };
+    Opm::EclipseGrid grid { 20,20,20, 1.0, 1.0, 25., 2500.0 };
 
     const auto depth = 0.0;
     const auto state = Opm::Connection::State::OPEN;
@@ -493,7 +489,7 @@ BOOST_AUTO_TEST_CASE(NegativeDepthCOMPSEGS)
     const auto dir_x = Opm::Connection::Direction::X;
     const auto kind = Opm::Connection::CTFKind::DeckValue;
 
-    const Opm::EclipseGrid grid { 20,20,20, 1.0, 1.0, 25.0, 2500.0 };
+    Opm::EclipseGrid grid { 20,20,20, 1.0, 1.0, 25.0, 2500.0 };
 
     const auto depth = 0.0;
     const auto state = Opm::Connection::State::OPEN;
@@ -574,7 +570,7 @@ BOOST_AUTO_TEST_CASE(testwsegvalv)
     const auto dir_z = Opm::Connection::Direction::Z;
     const auto dir_x = Opm::Connection::Direction::X;
     const auto kind = Opm::Connection::CTFKind::DeckValue;
-    const Opm::EclipseGrid grid { 20,20,20, 1.0, 1.0, 25.0, 2500.0 };
+    Opm::EclipseGrid grid { 20,20,20, 1.0, 1.0, 25.0, 2500.0 };
 
     const auto depth = 0.0;
     const auto state = Opm::Connection::State::OPEN;
