@@ -21,6 +21,8 @@
 
 #include <boost/test/unit_test.hpp>
 
+#include <opm/common/utility/OpmInputError.hpp>
+
 #include <opm/input/eclipse/Parser/Parser.hpp>
 #include <opm/input/eclipse/Deck/Deck.hpp>
 
@@ -2349,6 +2351,8 @@ BOOST_AUTO_TEST_CASE( TestParseDIFFC ) {
 
 BOOST_AUTO_TEST_CASE( TestParseDIFFCWATGAS ) {
     const std::string data = R"(
+      GAS
+      WATER
       TABDIMS
         1* 1 /
 
@@ -2373,6 +2377,69 @@ BOOST_AUTO_TEST_CASE( TestParseDIFFCWATGAS ) {
     BOOST_CHECK_CLOSE( 1.4, diffcgas[0].h2o_in_gas*conversion_factor, epsilon() );
 }
 
+BOOST_AUTO_TEST_CASE( TestParseDIFFAWATGAS ) {
+    const std::string data = R"(
+      GAS
+      WATER
+      TABDIMS
+        1* 1 /
+
+      DIFFAWAT
+        1.1 1.2 /
+
+      DIFFAGAS
+        1.3 1.4 /
+    )";
+
+    Opm::Parser parser;
+    auto deck = parser.parseString(data);
+    Opm::TableManager tables( deck );
+    double conversion_factor = (60*60*24);
+
+    const auto& diffawat = tables.getDiffusionCoefficientWaterTable();
+    BOOST_CHECK_CLOSE( 1.1, diffawat[0].co2_in_water*conversion_factor, epsilon() );
+    BOOST_CHECK_CLOSE( 1.2, diffawat[0].h2o_in_water*conversion_factor, epsilon());
+
+    const auto& diffagas = tables.getDiffusionCoefficientGasTable();
+    BOOST_CHECK_CLOSE( 1.3, diffagas[0].co2_in_gas*conversion_factor, epsilon() );
+    BOOST_CHECK_CLOSE( 1.4, diffagas[0].h2o_in_gas*conversion_factor, epsilon() );
+}
+
+BOOST_AUTO_TEST_CASE( TestParseDIFFACWATGAS ) {
+    const std::string data = R"(
+      GAS
+      WATER
+      TABDIMS
+        1* 1 /
+
+      DIFFAWAT
+        1.1 1.2 /
+
+      DIFFCGAS
+        1.3 1.4 /
+    )";
+
+    Opm::Parser parser;
+    BOOST_CHECK_THROW(parser.parseString(data), Opm::OpmInputError);
+}
+
+BOOST_AUTO_TEST_CASE( TestParseDIFFCAWATGAS ) {
+    const std::string data = R"(
+      GAS
+      WATER
+      TABDIMS
+        1* 1 /
+
+      DIFFCWAT
+        1.1 1.2 /
+
+      DIFFAGAS
+        1.3 1.4 /
+    )";
+
+    Opm::Parser parser;
+    BOOST_CHECK_THROW(parser.parseString(data), Opm::OpmInputError);
+}
 
 BOOST_AUTO_TEST_CASE(TestParsePPCWMAX) {
     const std::string data = R"(
@@ -2600,4 +2667,4 @@ TLMIXPAR
     BOOST_CHECK_EQUAL( r1.density_parameter, 0.25);
 
     BOOST_CHECK_THROW(tlm[2], std::out_of_range);
-}
+}
\ No newline at end of file
