@@ -590,8 +590,9 @@ namespace {
             Opm::UnitSystem unit_system(Opm::UnitSystem::UnitType::UNIT_TYPE_METRIC);
             auto deck = parser.parseString(input);
             const auto& record = deck["WCONHIST"].back().getRecord(0);
+            auto table_nr = record.getItem("VFP_TABLE").get< int >(0);
             Opm::Well::WellProductionProperties hist(unit_system, "W");
-            hist.handleWCONHIST(alq_type,
+            hist.handleWCONHIST(alq_type, table_nr, 
                                 Opm::ParserKeywords::FBHPDEF::TARGET_BHP::defaultValue * unit::barsa,
                                 unit_system, record);
 
@@ -645,8 +646,9 @@ namespace {
             auto deck = parser.parseString(input);
             const auto& kwd     = deck["WCONPROD"].back();
             const auto&  record = kwd.getRecord(0);
+            auto table_nr = record.getItem("VFP_TABLE").get< int >(0);
             Opm::Well::WellProductionProperties pred(unit_system, "W");
-            pred.handleWCONPROD(alq_type,
+            pred.handleWCONPROD(alq_type, table_nr,
                                 Opm::ParserKeywords::FBHPDEF::TARGET_BHP::defaultValue * unit::barsa,
                                 unit_system, "WELL", record, {});
 
@@ -658,8 +660,8 @@ namespace {
 
 BOOST_AUTO_TEST_CASE(WCH_All_Specified_BHP_Defaulted)
 {
-    Opm::SummaryState st(TimeService::now());
-    const Opm::Well::WellProductionProperties& p =
+    Opm::SummaryState st(TimeService::now(), 0.0);
+    const Opm::Well::WellProductionProperties p =
         WCONHIST::properties(WCONHIST::all_specified());
 
     BOOST_CHECK(p.hasProductionControl(Opm::Well::ProducerCMode::ORAT));
@@ -678,8 +680,8 @@ BOOST_AUTO_TEST_CASE(WCH_All_Specified_BHP_Defaulted)
 
 BOOST_AUTO_TEST_CASE(WCH_ORAT_Defaulted_BHP_Defaulted)
 {
-    Opm::SummaryState st(TimeService::now());
-    const Opm::Well::WellProductionProperties& p =
+    Opm::SummaryState st(TimeService::now(), 0.0);
+    const Opm::Well::WellProductionProperties p =
         WCONHIST::properties(WCONHIST::orat_defaulted());
 
     BOOST_CHECK( !p.hasProductionControl(Opm::Well::ProducerCMode::ORAT));
@@ -696,8 +698,8 @@ BOOST_AUTO_TEST_CASE(WCH_ORAT_Defaulted_BHP_Defaulted)
 
 BOOST_AUTO_TEST_CASE(WCH_OWRAT_Defaulted_BHP_Defaulted)
 {
-    Opm::SummaryState st(TimeService::now());
-    const Opm::Well::WellProductionProperties& p =
+    Opm::SummaryState st(TimeService::now(), 0.0);
+    const Opm::Well::WellProductionProperties p =
         WCONHIST::properties(WCONHIST::owrat_defaulted());
 
     BOOST_CHECK( !p.hasProductionControl(Opm::Well::ProducerCMode::ORAT));
@@ -714,8 +716,8 @@ BOOST_AUTO_TEST_CASE(WCH_OWRAT_Defaulted_BHP_Defaulted)
 
 BOOST_AUTO_TEST_CASE(WCH_Rates_Defaulted_BHP_Defaulted)
 {
-    Opm::SummaryState st(TimeService::now());
-    const Opm::Well::WellProductionProperties& p =
+    Opm::SummaryState st(TimeService::now(), 0.0);
+    const Opm::Well::WellProductionProperties p =
         WCONHIST::properties(WCONHIST::all_defaulted());
 
     BOOST_CHECK( !p.hasProductionControl(Opm::Well::ProducerCMode::ORAT));
@@ -732,8 +734,8 @@ BOOST_AUTO_TEST_CASE(WCH_Rates_Defaulted_BHP_Defaulted)
 
 BOOST_AUTO_TEST_CASE(WCH_Rates_Defaulted_BHP_Specified)
 {
-    Opm::SummaryState st(TimeService::now());
-    const Opm::Well::WellProductionProperties& p =
+    Opm::SummaryState st(TimeService::now(), 0.0);
+    const Opm::Well::WellProductionProperties p =
         WCONHIST::properties(WCONHIST::all_defaulted_with_bhp());
 
     BOOST_CHECK( !p.hasProductionControl(Opm::Well::ProducerCMode::ORAT));
@@ -751,8 +753,8 @@ BOOST_AUTO_TEST_CASE(WCH_Rates_Defaulted_BHP_Specified)
 
 BOOST_AUTO_TEST_CASE(WCH_Rates_NON_Defaulted_VFP)
 {
-    Opm::SummaryState st(TimeService::now());
-    const Opm::Well::WellProductionProperties& p =
+    Opm::SummaryState st(TimeService::now(), 0.0);
+    const Opm::Well::WellProductionProperties p =
         WCONHIST::properties(WCONHIST::all_defaulted_with_bhp_vfp_table(), VFPProdTable::ALQ_TYPE::ALQ_UNDEF);
 
     BOOST_CHECK( !p.hasProductionControl(Opm::Well::ProducerCMode::ORAT));
@@ -772,8 +774,8 @@ BOOST_AUTO_TEST_CASE(WCH_Rates_NON_Defaulted_VFP)
 
 BOOST_AUTO_TEST_CASE(WCH_BHP_Specified)
 {
-    Opm::SummaryState st(TimeService::now());
-    const Opm::Well::WellProductionProperties& p =
+    Opm::SummaryState st(TimeService::now(), 0.0);
+    const Opm::Well::WellProductionProperties p =
         WCONHIST::properties(WCONHIST::bhp_defaulted());
 
     BOOST_CHECK( !p.hasProductionControl(Opm::Well::ProducerCMode::ORAT));
@@ -880,7 +882,7 @@ BOOST_AUTO_TEST_CASE(WELL_CONTROLS) {
     auto unit_system = UnitSystem::newMETRIC();
     Opm::Well well("WELL", "GROUP", 0, 0, 0, 0, 1000, Opm::WellType(Opm::Phase::OIL), Opm::Well::ProducerCMode::CMODE_UNDEFINED, Opm::Connection::Order::DEPTH, unit_system, 0, 1.0, false, false, 0, Opm::Well::GasInflowEquation::STD);
     Opm::Well::WellProductionProperties prod(unit_system, "OP1");
-    Opm::SummaryState st(Opm::TimeService::now());
+    Opm::SummaryState st(Opm::TimeService::now(), 0.0);
     well.productionControls(st);
 
     // Use a scalar FIELD variable - that should work; although it is a bit weird.
@@ -910,7 +912,7 @@ BOOST_AUTO_TEST_CASE(ExtraAccessors) {
     prod_props->VFPTableNumber = 200;
     prod.updateProduction(prod_props);
 
-    BOOST_CHECK_THROW(prod.temperature(), std::runtime_error);
+    BOOST_CHECK_THROW(prod.inj_temperature(), std::runtime_error);
     BOOST_CHECK_EQUAL(inj.vfp_table_number(), 100);
     BOOST_CHECK_EQUAL(prod.vfp_table_number(), 200);
 }
