@@ -31,6 +31,7 @@
 #include <opm/input/eclipse/EclipseState/EclipseState.hpp>
 #include <opm/input/eclipse/Parser/Parser.hpp>
 #include <opm/input/eclipse/Parser/ParseContext.hpp>
+#include <opm/input/eclipse/Python/Python.hpp>
 
 #include <opm/io/eclipse/rst/header.hpp>
 
@@ -507,7 +508,7 @@ namespace {
     {
         const auto deck = Opm::Parser{}.parseString(deck_string);
 
-        const Opm::EclipseGrid grid(10, 10, 10);
+        Opm::EclipseGrid grid(10, 10, 10);
         const Opm::TableManager table(deck);
         const Opm::FieldPropsManager fp(deck, Opm::Phases{true, true, true}, grid, table);
         const Opm::Runspec runspec(deck);
