@@ -112,59 +112,14 @@ namespace {
     {
         return std::get<1>(vec);
     }
-} // Anonymous namespace
 
-namespace Opm {
-namespace data {
-
-// Some test specific equivalence definitions and pretty-printing. Not fit
-// as a general purpose implementation, but does its job for testing and
-// pretty-pringing for debugging purposes.
-
-std::ostream& operator<<( std::ostream& stream, const Rates& r ) {
-    return stream << "{ "
-                  << "wat: " << r.get( Rates::opt::wat, 0.0 ) << ", "
-                  << "oil: " << r.get( Rates::opt::oil, 0.0 ) << ", "
-                  << "gas: " << r.get( Rates::opt::gas, 0.0 ) << " "
-                  << "}";
-}
-
-std::ostream& operator<<( std::ostream& stream, const Connection& c ) {
-    return stream << "{ index: "
-                  << c.index << ", "
-                  << c.rates << ", "
-                  << c.pressure << " }";
-}
-
-std::ostream& operator<<( std::ostream& stream,
-                          const std::map< std::string, Well >& m ) {
-    stream << "\n";
-
-    for( const auto& p : m ) {
-        stream << p.first << ": \n"
-               << "\t" << "bhp: " << p.second.bhp << "\n"
-               << "\t" << "temp: " << p.second.temperature << "\n"
-               << "\t" << "rates: " << p.second.rates << "\n"
-               << "\t" << "connections: [\n";
-
-        for( const auto& c : p.second.connections )
-            stream << c << " ";
-
-        stream << "]\n";
-    }
-
-    return stream;
-}
-
-} // namespace data
-
-namespace {
-
-data::GroupAndNetworkValues mkGroups() {
+data::GroupAndNetworkValues mkGroups()
+{
     return {};
 }
 
-data::Wells mkWells() {
+data::Wells mkWells()
+{
     data::Rates r1, r2, rc1, rc2, rc3;
     r1.set( data::Rates::opt::wat, 5.67 );
     r1.set( data::Rates::opt::oil, 6.78 );
@@ -238,17 +193,18 @@ data::Solution mkSolution(int numCells)
     fun::iota rvi( 400.0, 400.0 + numCells );
 
     sol.insert("RS", measure::identity,
-               { rsi.begin(), rsi.end() },
+               std::vector<double>{ rsi.begin(), rsi.end() },
                data::TargetType::RESTART_SOLUTION);
     sol.insert("RV", measure::identity,
-               { rvi.begin(), rvi.end() },
+               std::vector<double>{ rvi.begin(), rvi.end() },
                data::TargetType::RESTART_SOLUTION);
 
     return sol;
 }
 
 data::Solution mkSolutionFIP(const int numCells)
-{    using measure = UnitSystem::measure;
+{
+    using measure = UnitSystem::measure;
 
     auto sol = data::Solution {
         { "PRESSURE", data::CellData { measure::pressure, {}, data::TargetType::RESTART_SOLUTION } },
@@ -271,7 +227,11 @@ data::Solution mkSolutionFIP(const int numCells)
 
 Opm::SummaryState sim_state(const Opm::Schedule& sched)
 {
-    auto state = Opm::SummaryState{TimeService::now()};
+    auto state = Opm::SummaryState {
+        TimeService::now(),
+        sched.back().udq().params().undefinedValue()
+    };
+
     for (const auto& well : sched.getWellsatEnd()) {
         for (const auto& connection : well.getConnections()) {
             state.update_conn_var(well.name(), "CPR", connection.global_index() + 1, 111);
@@ -453,7 +413,7 @@ first_sim(const Setup&   setup,
           bool           write_double)
 {
     WellTestState wtest_state;
-    EclipseIO eclWriter( setup.es, setup.grid, setup.schedule, setup.summary_config);
+    EclipseIO eclWriter(setup.es, setup.grid, setup.schedule, setup.summary_config);
 
     const auto num_cells = setup.grid.getNumActive( );
     const int report_step = 1;
@@ -475,15 +435,15 @@ first_sim(const Setup&   setup,
              st, udq_state);
 
     RestartValue restart_value(sol, wells, groups, {});
-    eclWriter.writeTimeStep( action_state,
-                             wtest_state,
-                             st,
-                             udq_state,
-                             report_step,
-                             false,
-                             std::difftime(first_step, start_time),
-                             restart_value,
-                             write_double);
+    eclWriter.writeTimeStep(action_state,
+                            wtest_state,
+                            st,
+                            udq_state,
+                            report_step,
+                            false,
+                            std::difftime(first_step, start_time),
+                            restart_value,
+                            write_double);
 
     return restart_value;
 }
@@ -495,10 +455,9 @@ second_sim(const Setup&                   setup,
            const std::vector<RestartKey>& solution_keys)
 {
     EclipseIO writer(setup.es, setup.grid, setup.schedule, setup.summary_config);
-    return writer.loadRestart( action_state, summary_state, solution_keys );
+    return writer.loadRestart(action_state, summary_state, solution_keys);
 }
 
-
 void compare(const RestartValue&            fst,
              const RestartValue&            snd,
              const std::vector<RestartKey>& solution_keys)
@@ -522,11 +481,15 @@ void compare(const RestartValue&            fst,
 
 } // Anonymous namespace
 
-BOOST_AUTO_TEST_CASE(EclipseReadWriteWellStateData) {
-    std::vector<RestartKey> keys {{"PRESSURE" , UnitSystem::measure::pressure},
-                                  {"SWAT" , UnitSystem::measure::identity},
-                                  {"SGAS" , UnitSystem::measure::identity},
-                                  {"TEMP" , UnitSystem::measure::temperature}};
+BOOST_AUTO_TEST_CASE(EclipseReadWriteWellStateData)
+{
+    const std::vector<RestartKey> keys {
+        {"PRESSURE" , UnitSystem::measure::pressure},
+        {"SWAT" , UnitSystem::measure::identity},
+        {"SGAS" , UnitSystem::measure::identity},
+        {"TEMP" , UnitSystem::measure::temperature},
+    };
+
     WorkArea test_area("test_restart");
     test_area.copyIn("BASE_SIM.DATA");
     test_area.copyIn("RESTART_SIM.DATA");
@@ -535,17 +498,18 @@ BOOST_AUTO_TEST_CASE(EclipseReadWriteWellStateData) {
     auto st = sim_state(base_setup.schedule);
     Action::State action_state;
     UDQState udq_state(19);
-    auto state1 = first_sim( base_setup , action_state, st, udq_state, false );
+    const auto state1 = first_sim( base_setup , action_state, st, udq_state, false );
 
     Setup restart_setup("RESTART_SIM.DATA");
-    auto state2 = second_sim( restart_setup , action_state, st , keys );
+    const auto state2 = second_sim( restart_setup , action_state, st , keys );
     compare(state1, state2 , keys);
 
     BOOST_CHECK_THROW( second_sim( restart_setup, action_state, st, {{"SOIL", UnitSystem::measure::pressure}} ) , std::runtime_error );
     BOOST_CHECK_THROW( second_sim( restart_setup, action_state, st, {{"SOIL", UnitSystem::measure::pressure, true}}) , std::runtime_error );
 }
 
-BOOST_AUTO_TEST_CASE(ECL_FORMATTED) {
+BOOST_AUTO_TEST_CASE(ECL_FORMATTED)
+{
     namespace OS = ::Opm::EclIO::OutputStream;
 
     WorkArea test_area("test_Restart");
@@ -567,7 +531,7 @@ BOOST_AUTO_TEST_CASE(ECL_FORMATTED) {
             RestartValue restart_value(cells, wells, groups, {});
 
             io_config.setEclCompatibleRST( false );
-            restart_value.addExtra("EXTRA", UnitSystem::measure::pressure, {10,1,2,3});
+            restart_value.addExtra("EXTRA", UnitSystem::measure::pressure, std::vector<double>{10.0,1.0,2.0,3.0});
 
             const auto outputDir = test_area.currentWorkingDirectory();
 
@@ -632,8 +596,6 @@ BOOST_AUTO_TEST_CASE(ECL_FORMATTED) {
 
                 BOOST_CHECK_MESSAGE(rst.hasKey("SWAT"), "Restart file must have SWAT vector");
                 BOOST_CHECK_MESSAGE(!rst.hasKey("EXTRA"), "Restart file must NOT have EXTRA vector");
-                BOOST_CHECK_MESSAGE(!rst.hasKey("OPM_IWEL"), "Restart file must NOT have OPM_IWEL vector");
-                BOOST_CHECK_MESSAGE(!rst.hasKey("OPM_XWEL"), "Restart file must NOT have OPM_XWEL vector");
             }
         }
     }
@@ -658,7 +620,8 @@ void compare_equal(const RestartValue&            fst,
 
 } // Anonymous namespace
 
-BOOST_AUTO_TEST_CASE(EclipseReadWriteWellStateData_double) {
+BOOST_AUTO_TEST_CASE(EclipseReadWriteWellStateData_double)
+{
     /*
       Observe that the purpose of this test is to verify that with
       write_double == true we can load solution fields which are
@@ -685,7 +648,8 @@ BOOST_AUTO_TEST_CASE(EclipseReadWriteWellStateData_double) {
     compare_equal( state1 , state2 , solution_keys);
 }
 
-BOOST_AUTO_TEST_CASE(WriteWrongSOlutionSize) {
+BOOST_AUTO_TEST_CASE(WriteWrongSolutionSize)
+{
     namespace OS = ::Opm::EclIO::OutputStream;
 
     WorkArea test_area("test_Restart");
@@ -697,7 +661,7 @@ BOOST_AUTO_TEST_CASE(WriteWrongSOlutionSize) {
         auto cells = mkSolution( num_cells );
         auto wells = mkWells();
         auto groups = mkGroups();
-        Opm::SummaryState sumState(TimeService::now());
+        Opm::SummaryState sumState(TimeService::now(), 0.0);
         Opm::Action::State action_state;
         Opm::UDQState udq_state(19);
         Opm::WellTestState wtest_state;
@@ -724,7 +688,8 @@ BOOST_AUTO_TEST_CASE(WriteWrongSOlutionSize) {
     }
 }
 
-BOOST_AUTO_TEST_CASE(ExtraData_KEYS) {
+BOOST_AUTO_TEST_CASE(ExtraData_KEYS)
+{
     Setup setup("BASE_SIM.DATA");
     auto num_cells = setup.grid.getNumActive( );
     auto cells = mkSolution( num_cells );
@@ -732,20 +697,21 @@ BOOST_AUTO_TEST_CASE(ExtraData_KEYS) {
     auto groups = mkGroups();
     RestartValue restart_value(cells, wells, groups, {});
 
-    BOOST_CHECK_THROW( restart_value.addExtra("TOO-LONG-KEY", {0,1,2}), std::runtime_error);
+    BOOST_CHECK_THROW( restart_value.addExtra("TOO-LONG-KEY", std::vector{0.0,1.0,2.0}), std::runtime_error);
 
     // Keys must be unique
-    restart_value.addExtra("KEY", {0,1,1});
-    BOOST_CHECK_THROW( restart_value.addExtra("KEY", {0,1,1}), std::runtime_error);
+    restart_value.addExtra("KEY", std::vector{0.0,1.0,1.0});
+    BOOST_CHECK_THROW( restart_value.addExtra("KEY", std::vector{0.0,1.0,1.0}), std::runtime_error);
 
     /* The keys must be unique across solution and extra_data */
-    BOOST_CHECK_THROW( restart_value.addExtra("PRESSURE", {0,1}), std::runtime_error);
+    BOOST_CHECK_THROW( restart_value.addExtra("PRESSURE", std::vector{0.0,1.0}), std::runtime_error);
 
     /* Must avoid using reserved keys like 'LOGIHEAD' */
-    BOOST_CHECK_THROW( restart_value.addExtra("LOGIHEAD", {0,1}), std::runtime_error);
+    BOOST_CHECK_THROW( restart_value.addExtra("LOGIHEAD", std::vector{0.0,1.0}), std::runtime_error);
 }
 
-BOOST_AUTO_TEST_CASE(ExtraData_content) {
+BOOST_AUTO_TEST_CASE(ExtraData_content)
+{
     namespace OS = ::Opm::EclIO::OutputStream;
 
     WorkArea test_area("test_Restart");
@@ -764,10 +730,10 @@ BOOST_AUTO_TEST_CASE(ExtraData_content) {
         const auto& units = setup.es.getUnits();
         {
             RestartValue restart_value(cells, wells, groups, {});
-            SummaryState st(TimeService::now());
+            SummaryState st(TimeService::now(), 0.0);
             const auto sumState = sim_state(setup.schedule);
 
-            restart_value.addExtra("EXTRA", UnitSystem::measure::pressure, {10,1,2,3});
+            restart_value.addExtra("EXTRA", UnitSystem::measure::pressure, std::vector<double>{10.0,1.0,2.0,3.0});
 
             const auto outputDir = test_area.currentWorkingDirectory();
 
@@ -831,7 +797,8 @@ BOOST_AUTO_TEST_CASE(ExtraData_content) {
     }
 }
 
-BOOST_AUTO_TEST_CASE(STORE_THPRES) {
+BOOST_AUTO_TEST_CASE(STORE_THPRES)
+{
     namespace OS = ::Opm::EclIO::OutputStream;
 
     WorkArea test_area("test_Restart_THPRES");
@@ -861,7 +828,7 @@ BOOST_AUTO_TEST_CASE(STORE_THPRES) {
                                                setup.schedule), std::runtime_error);
             */
 
-            restart_value.addExtra("THRESHPR", UnitSystem::measure::pressure, {0,1});
+            restart_value.addExtra("THRESHPR", UnitSystem::measure::pressure, std::vector<double>{0.0,1.0});
             const auto sumState = sim_state(base_setup.schedule);
             Action::State action_state;
             UDQState udq_state(99);
@@ -976,10 +943,10 @@ BOOST_AUTO_TEST_CASE(Restore_Cumulatives)
     }
 
     Action::State action_state;
-    SummaryState rstSumState(TimeService::now());
+    SummaryState rstSumState(TimeService::now(), 0.0);
     RestartIO::load(OS::outputFileName(rset, "UNRST"), seqnum, action_state, rstSumState,
                     /* solution_keys = */ {
-                                           RestartKey("SWAT", UnitSystem::measure::identity),
+                        RestartKey("SWAT", UnitSystem::measure::identity),
                     },
                     setup.es, setup.grid, setup.schedule,
                     /* extra_keys = */ {});
@@ -1119,24 +1086,29 @@ void init_st(SummaryState& st)
 
 } // Anonymous namespace
 
-BOOST_AUTO_TEST_CASE(UDQ_RESTART) {
-    std::vector<RestartKey> keys {{"PRESSURE" , UnitSystem::measure::pressure},
+BOOST_AUTO_TEST_CASE(UDQ_RESTART)
+{
+    const std::vector<RestartKey> keys {
+        {"PRESSURE" , UnitSystem::measure::pressure},
         {"SWAT" , UnitSystem::measure::identity},
-        {"SGAS" , UnitSystem::measure::identity}};
+        {"SGAS" , UnitSystem::measure::identity},
+    };
+
     WorkArea test_area("test_udq_restart");
     test_area.copyIn("UDQ_BASE.DATA");
     test_area.copyIn("UDQ_RESTART.DATA");
 
     Setup base_setup("UDQ_BASE.DATA");
-    SummaryState st1(TimeService::now());
-    SummaryState st2(TimeService::now());
+    SummaryState st1(TimeService::now(), 0.0);
+    SummaryState st2(TimeService::now(), 0.0);
     Action::State action_state;
     UDQState udq_state(1);
     init_st(st1);
-    auto state1 = first_sim( base_setup , action_state, st1, udq_state, false );
+    const auto state1 = first_sim(base_setup, action_state, st1, udq_state, false);
 
     Setup restart_setup("UDQ_RESTART.DATA");
-    auto state2 = second_sim( restart_setup , action_state, st2 , keys );
+    const auto state2 = second_sim(restart_setup, action_state, st2, keys);
+
     BOOST_CHECK(st1.wells() == st2.wells());
     BOOST_CHECK(st1.groups() == st2.groups());
 
@@ -1144,56 +1116,72 @@ BOOST_AUTO_TEST_CASE(UDQ_RESTART) {
     for (const auto& well : st1.wells()) {
         for (const auto& def : udq.definitions(UDQVarType::WELL_VAR)) {
             const auto& kw = def.keyword();
-            BOOST_CHECK_EQUAL( st1.has_well_var(well, kw), st2.has_well_var(well, kw));
-            if (st1.has_well_var(well, def.keyword()))
+
+            BOOST_CHECK_EQUAL(st1.has_well_var(well, kw), st2.has_well_var(well, kw));
+
+            if (st1.has_well_var(well, def.keyword())) {
                 BOOST_CHECK_EQUAL(st1.get_well_var(well, kw), st2.get_well_var(well, kw));
+            }
         }
     }
 
     for (const auto& group : st1.groups()) {
         for (const auto& def : udq.definitions(UDQVarType::GROUP_VAR)) {
             const auto& kw = def.keyword();
+
             BOOST_CHECK_EQUAL( st1.has_group_var(group, kw), st2.has_group_var(group, kw));
-            if (st1.has_group_var(group, def.keyword()))
+
+            if (st1.has_group_var(group, def.keyword())) {
                 BOOST_CHECK_EQUAL(st1.get_group_var(group, kw), st2.get_group_var(group, kw));
+            }
         }
     }
 
     for (const auto& well : st1.wells()) {
         for (const auto& def : udq.assignments(UDQVarType::WELL_VAR)) {
             const auto& kw = def.keyword();
+
             BOOST_CHECK_EQUAL( st1.has_well_var(well, kw), st2.has_well_var(well, kw));
-            if (st1.has_well_var(well, def.keyword()))
+
+            if (st1.has_well_var(well, def.keyword())) {
                 BOOST_CHECK_EQUAL(st1.get_well_var(well, kw), st2.get_well_var(well, kw));
+            }
         }
     }
 
     for (const auto& group : st1.groups()) {
         for (const auto& def : udq.assignments(UDQVarType::GROUP_VAR)) {
             const auto& kw = def.keyword();
-            BOOST_CHECK_EQUAL( st1.has_group_var(group, kw), st2.has_group_var(group, kw));
-            if (st1.has_group_var(group, def.keyword()))
+
+            BOOST_CHECK_EQUAL(st1.has_group_var(group, kw), st2.has_group_var(group, kw));
+
+            if (st1.has_group_var(group, def.keyword())) {
                 BOOST_CHECK_EQUAL(st1.get_group_var(group, kw), st2.get_group_var(group, kw));
+            }
         }
     }
 
     for (const auto& def : udq.assignments(UDQVarType::FIELD_VAR)) {
         const auto& kw = def.keyword();
-        BOOST_CHECK_EQUAL( st1.has(kw), st2.has(kw));
-            if (st1.has(kw))
-                BOOST_CHECK_EQUAL(st1.get(kw), st2.get(kw));
+
+        BOOST_CHECK_EQUAL(st1.has(kw), st2.has(kw));
+
+        if (st1.has(kw)) {
+            BOOST_CHECK_EQUAL(st1.get(kw), st2.get(kw));
+        }
     }
 
     for (const auto& def : udq.definitions(UDQVarType::FIELD_VAR)) {
         const auto& kw = def.keyword();
-        BOOST_CHECK_EQUAL( st1.has(kw), st2.has(kw));
-        if (st1.has(kw))
+
+        BOOST_CHECK_EQUAL(st1.has(kw), st2.has(kw));
+
+        if (st1.has(kw)) {
             BOOST_CHECK_EQUAL(st1.get(kw), st2.get(kw));
+        }
     }
 }
 
-} // namespace Opm
-
 namespace {
 
 struct MessageBuffer
@@ -1280,7 +1268,8 @@ Opm::data::AquiferData getNumericalAquifer(const int aquiferID = 2)
 
     return aquifer;
 }
-} // Anonymous
+
+} // Anonymous namespace
 
 BOOST_AUTO_TEST_CASE(ReadWrite_CarterTracy_Data)
 {
@@ -1342,7 +1331,7 @@ namespace {
             BOOST_CHECK_CLOSE(vi, x, tol);
         }
     }
-}
+} // Anonymous namespace
 
 BOOST_AUTO_TEST_CASE(Fluid_In_Place)
 {
