@@ -65,6 +65,7 @@
 
 #include <tests/WorkArea.hpp>
 
+#include <algorithm>
 #include <cstddef>
 #include <memory>
 #include <string>
@@ -392,7 +393,7 @@ END
         const auto& units    = simCase.es.getUnits();
         const auto  sim_step = rptStep - 1;
 
-        const auto sumState     = Opm::SummaryState { Opm::TimeService::now() };
+        const auto sumState     = Opm::SummaryState { Opm::TimeService::now(), 0.0 };
         const auto action_state = Opm::Action::State{};
         const auto wtest_state  = Opm::WellTestState{};
 
@@ -486,7 +487,7 @@ BOOST_AUTO_TEST_CASE(group_test)
     // Report Step 2: 2011-01-20 --> 2013-06-15
     const auto rptStep = std::size_t{2};
     const auto sim_step = rptStep - 1;
-    Opm::SummaryState sumState(Opm::TimeService::now());
+    Opm::SummaryState sumState(Opm::TimeService::now(), 0.0);
 
     const auto ih = Opm::RestartIO::Helpers::createInteHead(simCase.es,
                                                             simCase.grid,
@@ -514,8 +515,8 @@ BOOST_AUTO_TEST_CASE(group_test)
 
     Opm::UnitSystem unit_system(Opm::UnitSystem::UnitType::UNIT_TYPE_METRIC);
     std::vector<std::string> zgrp;
-    for (const auto& s8: zgrp8)
-        zgrp.push_back(s8.c_str());
+    std::transform(zgrp8.begin(), zgrp8.end(), std::back_inserter(zgrp),
+                   [](const auto& s8) { return s8.c_str(); });
 
     Opm::RestartIO::RstHeader header(simCase.es.runspec(), unit_system,ih,lh,dh);
     for (int ig=0; ig < header.ngroup; ig++) {
@@ -965,7 +966,7 @@ BOOST_AUTO_TEST_CASE(Historic_Period)
             state.header.udq_undefined
         };
 
-        const auto ctrl = op1.productionControls(Opm::SummaryState { Opm::TimeService::now() });
+        const auto ctrl = op1.productionControls(Opm::SummaryState { Opm::TimeService::now(), 0.0 });
         BOOST_CHECK_MESSAGE(ctrl.cmode == Opm::WellProducerCMode::GRAT,
                             "Well loaded from restart file must be controlled "
                             "by observed surface gas flow rate (GRAT)");
@@ -982,10 +983,10 @@ BOOST_AUTO_TEST_CASE(Historic_Period)
         };
 
         auto prop = op1.getProductionProperties();
-        prop.handleWCONHIST(std::nullopt, 101325.0, Opm::UnitSystem::newMETRIC(),
+        prop.handleWCONHIST(std::nullopt, 0, 101325.0, Opm::UnitSystem::newMETRIC(),
                             deck.get<Opm::ParserKeywords::WCONHIST>().back().getRecord(0));
 
-        const auto ctrl = prop.controls(Opm::SummaryState { Opm::TimeService::now() }, state.header.udq_undefined);
+        const auto ctrl = prop.controls(Opm::SummaryState { Opm::TimeService::now(), state.header.udq_undefined }, state.header.udq_undefined);
         BOOST_CHECK_MESSAGE(ctrl.cmode == Opm::WellProducerCMode::RESV,
                             "Well loaded from restart file must be controlled by "
                             "observed reservoir voidage rate (RESV) after WCONHIST");
@@ -1002,10 +1003,10 @@ BOOST_AUTO_TEST_CASE(Historic_Period)
         };
 
         auto prop = op1.getProductionProperties();
-        prop.handleWCONHIST(std::nullopt, 101325.0, Opm::UnitSystem::newMETRIC(),
+        prop.handleWCONHIST(std::nullopt, 0, 101325.0, Opm::UnitSystem::newMETRIC(),
                             deck.get<Opm::ParserKeywords::WCONHIST>().back().getRecord(0));
 
-        const auto ctrl = prop.controls(Opm::SummaryState { Opm::TimeService::now() }, state.header.udq_undefined);
+        const auto ctrl = prop.controls(Opm::SummaryState { Opm::TimeService::now(), state.header.udq_undefined }, state.header.udq_undefined);
         BOOST_CHECK_MESSAGE(ctrl.cmode == Opm::WellProducerCMode::ORAT,
                             "Well loaded from restart file must be controlled by "
                             "observed surface oil flow rate (ORAT) after WCONHIST");
@@ -1035,7 +1036,7 @@ BOOST_AUTO_TEST_CASE(Historic_Period_WHistCtl)
             state.header.udq_undefined
         };
 
-        const auto ctrl = op1.productionControls(Opm::SummaryState { Opm::TimeService::now() });
+        const auto ctrl = op1.productionControls(Opm::SummaryState { Opm::TimeService::now(), state.header.udq_undefined });
         BOOST_CHECK_MESSAGE(ctrl.cmode == Opm::WellProducerCMode::GRAT,
                             "Well loaded from restart file must be controlled "
                             "by observed surface gas flow rate (GRAT)");
@@ -1052,10 +1053,10 @@ BOOST_AUTO_TEST_CASE(Historic_Period_WHistCtl)
         };
 
         auto prop = op1.getProductionProperties();
-        prop.handleWCONHIST(std::nullopt, 101325.0, Opm::UnitSystem::newMETRIC(),
+        prop.handleWCONHIST(std::nullopt, 0, 101325.0, Opm::UnitSystem::newMETRIC(),
                             deck.get<Opm::ParserKeywords::WCONHIST>().back().getRecord(0));
 
-        const auto ctrl = prop.controls(Opm::SummaryState { Opm::TimeService::now() }, state.header.udq_undefined);
+        const auto ctrl = prop.controls(Opm::SummaryState { Opm::TimeService::now(), state.header.udq_undefined }, state.header.udq_undefined);
         BOOST_CHECK_MESSAGE(ctrl.cmode == Opm::WellProducerCMode::ORAT,
                             "Well loaded from restart file must be controlled by "
                             "observed surface oil flow rate (ORAT) after WCONHIST");
@@ -1072,10 +1073,10 @@ BOOST_AUTO_TEST_CASE(Historic_Period_WHistCtl)
         };
 
         auto prop = op1.getProductionProperties();
-        prop.handleWCONHIST(std::nullopt, 101325.0, Opm::UnitSystem::newMETRIC(),
+        prop.handleWCONHIST(std::nullopt, 0, 101325.0, Opm::UnitSystem::newMETRIC(),
                             deck.get<Opm::ParserKeywords::WCONHIST>().back().getRecord(0));
 
-        const auto ctrl = prop.controls(Opm::SummaryState { Opm::TimeService::now() }, state.header.udq_undefined);
+        const auto ctrl = prop.controls(Opm::SummaryState { Opm::TimeService::now(), state.header.udq_undefined }, state.header.udq_undefined);
         BOOST_CHECK_MESSAGE(ctrl.cmode == Opm::WellProducerCMode::RESV,
                             "Well loaded from restart file must be controlled by "
                             "observed reservoir voidage rate (RESV) after WCONHIST");
